/*! adapterjs - v0.15.0 - 2017-12-14 */

// Adapter's interface.
var AdapterJS = AdapterJS || {};

AdapterJS.options = AdapterJS.options || {};

// uncomment to get virtual webcams
// AdapterJS.options.getAllCams = true;
AdapterJS.options.getAllCams = !!AdapterJS.options.getAllCams;

// uncomment to prevent the install prompt when the plugin in not yet installed
// AdapterJS.options.hidePluginInstallPrompt = true;
AdapterJS.options.hidePluginInstallPrompt = !!AdapterJS.options.hidePluginInstallPrompt;

// uncomment to force the use of the plugin on Safari
// AdapterJS.options.forceSafariPlugin = true;
AdapterJS.options.forceSafariPlugin = !!AdapterJS.options.forceSafariPlugin;

// AdapterJS version
AdapterJS.VERSION = '0.15.0';

// This function will be called when the WebRTC API is ready to be used
// Whether it is the native implementation (Chrome, Firefox, Opera) or
// the plugin
// You may Override this function to synchronise the start of your application
// with the WebRTC API being ready.
// If you decide not to override use this synchronisation, it may result in
// an extensive CPU usage on the plugin start (once per tab loaded)
// Params:
//    - isUsingPlugin: true is the WebRTC plugin is being used, false otherwise
//
AdapterJS.onwebrtcready = AdapterJS.onwebrtcready || function(isUsingPlugin) {
        // The WebRTC API is ready.
        // Override me and do whatever you want here
    };

// New interface to store multiple callbacks, private
AdapterJS._onwebrtcreadies = [];

// Sets a callback function to be called when the WebRTC interface is ready.
// The first argument is the function to callback.\
// Throws an error if the first argument is not a function
AdapterJS.webRTCReady = function (baseCallback) {
    if (typeof baseCallback !== 'function') {
        throw new Error('Callback provided is not a function');
    }

    var callback = function () {
        // Make users having requirejs to use the webRTCReady function to define first
        // When you set a setTimeout(definePolyfill, 0), it overrides the WebRTC function
        // This is be more than 0s
        if (typeof window.require === 'function' &&
            typeof AdapterJS._defineMediaSourcePolyfill === 'function') {
            AdapterJS._defineMediaSourcePolyfill();
        }

        // All WebRTC interfaces are ready, just call the callback
        baseCallback(null !== AdapterJS.WebRTCPlugin.plugin);
    };



    if (true === AdapterJS.onwebrtcreadyDone) {
        callback();
    } else {
        // will be triggered automatically when your browser/plugin is ready.
        AdapterJS._onwebrtcreadies.push(callback);
    }
};

// Plugin namespace
AdapterJS.WebRTCPlugin = AdapterJS.WebRTCPlugin || {};

// The object to store plugin information
/* jshint ignore:start */
AdapterJS.WebRTCPlugin.pluginInfo = AdapterJS.WebRTCPlugin.pluginInfo || {
        prefix : 'Tem',
        plugName : 'TemWebRTCPlugin',
        pluginId : 'plugin0',
        type : 'application/x-temwebrtcplugin',
        onload : '__TemWebRTCReady0',
        portalLink : 'https://skylink.io/plugin/',
        downloadLink : null, //set below
        companyName: 'Temasys',
        downloadLinks : {
            //mac: 'https://bit.ly/webrtcpluginpkg',
            win: 'https://turn2.easemob.com/rtc-ws/TemWebRTCPlugin.msi'
        }
    };
if(typeof AdapterJS.WebRTCPlugin.pluginInfo.downloadLinks !== "undefined" && AdapterJS.WebRTCPlugin.pluginInfo.downloadLinks !== null) {
    if(!!navigator.platform.match(/^Mac/i)) {
        AdapterJS.WebRTCPlugin.pluginInfo.downloadLink = AdapterJS.WebRTCPlugin.pluginInfo.downloadLinks.mac;
    }
    else if(!!navigator.platform.match(/^Win/i)) {
        AdapterJS.WebRTCPlugin.pluginInfo.downloadLink = AdapterJS.WebRTCPlugin.pluginInfo.downloadLinks.win;
    }
}

/* jshint ignore:end */

AdapterJS.WebRTCPlugin.TAGS = {
    NONE  : 'none',
    AUDIO : 'audio',
    VIDEO : 'video'
};

// Unique identifier of each opened page
AdapterJS.WebRTCPlugin.pageId = Math.random().toString(36).slice(2);

// Use this whenever you want to call the plugin.
AdapterJS.WebRTCPlugin.plugin = null;

// Set log level for the plugin once it is ready.
// The different values are
// This is an asynchronous function that will run when the plugin is ready
AdapterJS.WebRTCPlugin.setLogLevel = null;

// Defines webrtc's JS interface according to the plugin's implementation.
// Define plugin Browsers as WebRTC Interface.
AdapterJS.WebRTCPlugin.defineWebRTCInterface = null;

// This function detects whether or not a plugin is installed.
// Checks if Not IE (firefox, for example), else if it's IE,
// we're running IE and do something. If not it is not supported.
AdapterJS.WebRTCPlugin.isPluginInstalled = null;

// Lets adapter.js wait until the the document is ready before injecting the plugin
AdapterJS.WebRTCPlugin.pluginInjectionInterval = null;

// Inject the HTML DOM object element into the page.
AdapterJS.WebRTCPlugin.injectPlugin = null;

// States of readiness that the plugin goes through when
// being injected and stated
AdapterJS.WebRTCPlugin.PLUGIN_STATES = {
    NONE : 0,           // no plugin use
    INITIALIZING : 1,   // Detected need for plugin
    INJECTING : 2,      // Injecting plugin
    INJECTED: 3,        // Plugin element injected but not usable yet
    READY: 4            // Plugin ready to be used
};

// Current state of the plugin. You cannot use the plugin before this is
// equal to AdapterJS.WebRTCPlugin.PLUGIN_STATES.READY
AdapterJS.WebRTCPlugin.pluginState = AdapterJS.WebRTCPlugin.PLUGIN_STATES.NONE;

// True is AdapterJS.onwebrtcready was already called, false otherwise
// Used to make sure AdapterJS.onwebrtcready is only called once
AdapterJS.onwebrtcreadyDone = false;

// Log levels for the plugin.
// To be set by calling AdapterJS.WebRTCPlugin.setLogLevel
/*
 Log outputs are prefixed in some cases.
 INFO: Information reported by the plugin.
 ERROR: Errors originating from within the plugin.
 WEBRTC: Error originating from within the libWebRTC library
 */
// From the least verbose to the most verbose
AdapterJS.WebRTCPlugin.PLUGIN_LOG_LEVELS = {
    NONE : 'NONE',
    ERROR : 'ERROR',
    WARNING : 'WARNING',
    INFO: 'INFO',
    VERBOSE: 'VERBOSE',
    SENSITIVE: 'SENSITIVE'
};

// Does a waiting check before proceeding to load the plugin.
AdapterJS.WebRTCPlugin.WaitForPluginReady = null;

// This methid will use an interval to wait for the plugin to be ready.
AdapterJS.WebRTCPlugin.callWhenPluginReady = null;

// !!!! WARNING: DO NOT OVERRIDE THIS FUNCTION. !!!
// This function will be called when plugin is ready. It sends necessary
// details to the plugin.
// The function will wait for the document to be ready and the set the
// plugin state to AdapterJS.WebRTCPlugin.PLUGIN_STATES.READY,
// indicating that it can start being requested.
// This function is not in the IE/Safari condition brackets so that
// TemPluginLoaded function might be called on Chrome/Firefox.
// This function is the only private function that is not encapsulated to
// allow the plugin method to be called.
__TemWebRTCReady0 = function () {
    if (document.readyState === 'interactive' || document.readyState === 'complete') {
        AdapterJS.WebRTCPlugin.pluginState = AdapterJS.WebRTCPlugin.PLUGIN_STATES.READY;
        AdapterJS.maybeThroughWebRTCReady();
    } else {
        // Try again in 100ms
        setTimeout(__TemWebRTCReady0, 100);
    }
};

AdapterJS.maybeThroughWebRTCReady = function() {
    if (!AdapterJS.onwebrtcreadyDone) {
        AdapterJS.onwebrtcreadyDone = true;

        // If new interface for multiple callbacks used
        if (AdapterJS._onwebrtcreadies.length) {
            AdapterJS._onwebrtcreadies.forEach(function (callback) {
                if (typeof(callback) === 'function') {
                    callback(AdapterJS.WebRTCPlugin.plugin !== null);
                }
            });
            // Else if no callbacks on new interface assuming user used old(deprecated) way to set callback through AdapterJS.onwebrtcready = ...
        } else if (typeof(AdapterJS.onwebrtcready) === 'function') {
            AdapterJS.onwebrtcready(AdapterJS.WebRTCPlugin.plugin !== null);
        }
    }
};

// Text namespace
AdapterJS.TEXT = {
    PLUGIN: {
        REQUIRE_INSTALLATION: 'This website requires you to install a WebRTC-enabling plugin ' +
        'to work on this browser.',
        NOT_SUPPORTED: 'Your browser does not support WebRTC.',
        BUTTON: 'Install Now'
    },
    REFRESH: {
        REQUIRE_REFRESH: 'Please refresh page',
        BUTTON: 'Refresh Page'
    }
};

// The result of ice connection states.
// - starting: Ice connection is starting.
// - checking: Ice connection is checking.
// - connected Ice connection is connected.
// - completed Ice connection is connected.
// - done Ice connection has been completed.
// - disconnected Ice connection has been disconnected.
// - failed Ice connection has failed.
// - closed Ice connection is closed.
AdapterJS._iceConnectionStates = {
    starting : 'starting',
    checking : 'checking',
    connected : 'connected',
    completed : 'connected',
    done : 'completed',
    disconnected : 'disconnected',
    failed : 'failed',
    closed : 'closed'
};

//The IceConnection states that has been fired for each peer.
AdapterJS._iceConnectionFiredStates = [];


// Check if WebRTC Interface is defined.
AdapterJS.isDefined = null;

// This function helps to retrieve the webrtc detected browser information.
// This sets:
// - webrtcDetectedBrowser: The browser agent name.
// - webrtcDetectedVersion: The browser version.
// - webrtcMinimumVersion: The minimum browser version still supported by AJS.
// - webrtcDetectedType: The types of webRTC support.
//   - 'moz': Mozilla implementation of webRTC.
//   - 'webkit': WebKit implementation of webRTC.
//   - 'plugin': Using the plugin implementation.
AdapterJS.parseWebrtcDetectedBrowser = function () {
    var hasMatch = null;

    // Detect Opera (8.0+)
    if ((!!window.opr && !!opr.addons) || !!window.opera || navigator.userAgent.indexOf(' OPR/') >= 0) {
        hasMatch = navigator.userAgent.match(/OPR\/(\d+)/i) || [];

        webrtcDetectedBrowser   = 'opera';
        webrtcDetectedVersion   = parseInt(hasMatch[1] || '0', 10);
        webrtcMinimumVersion    = 26;
        webrtcDetectedType      = 'webkit';
        webrtcDetectedDCSupport = 'SCTP'; // Opera 20+ uses Chrome 33

        // Detect Bowser on iOS
    } else if (navigator.userAgent.match(/Bowser\/[0-9.]*/g)) {
        hasMatch = navigator.userAgent.match(/Bowser\/[0-9.]*/g) || [];

        var chromiumVersion = parseInt((navigator.userAgent.match(/Chrom(e|ium)\/([0-9]+)\./i) || [])[2] || '0', 10);

        webrtcDetectedBrowser   = 'bowser';
        webrtcDetectedVersion   = parseFloat((hasMatch[0] || '0/0').split('/')[1], 10);
        webrtcMinimumVersion    = 0;
        webrtcDetectedType      = 'webkit';
        webrtcDetectedDCSupport = chromiumVersion > 30 ? 'SCTP' : 'RTP';


        // Detect Opera on iOS (does not support WebRTC yet)
    } else if (navigator.userAgent.indexOf('OPiOS') > 0) {
        hasMatch = navigator.userAgent.match(/OPiOS\/([0-9]+)\./);

        // Browser which do not support webrtc yet
        webrtcDetectedBrowser   = 'opera';
        webrtcDetectedVersion   = parseInt(hasMatch[1] || '0', 10);
        webrtcMinimumVersion    = 0;
        webrtcDetectedType      = null;
        webrtcDetectedDCSupport = null;

        // Detect Chrome on iOS (does not support WebRTC yet)
    } else if (navigator.userAgent.indexOf('CriOS') > 0) {
        hasMatch = navigator.userAgent.match(/CriOS\/([0-9]+)\./) || [];

        webrtcDetectedBrowser   = 'chrome';
        webrtcDetectedVersion   = parseInt(hasMatch[1] || '0', 10);
        webrtcMinimumVersion    = 0;
        webrtcDetectedType      = null;
        webrtcDetectedDCSupport = null;

        // Detect Firefox on iOS (does not support WebRTC yet)
    } else if (navigator.userAgent.indexOf('FxiOS') > 0) {
        hasMatch = navigator.userAgent.match(/FxiOS\/([0-9]+)\./) || [];

        // Browser which do not support webrtc yet
        webrtcDetectedBrowser   = 'firefox';
        webrtcDetectedVersion   = parseInt(hasMatch[1] || '0', 10);
        webrtcMinimumVersion    = 0;
        webrtcDetectedType      = null;
        webrtcDetectedDCSupport = null;

        // Detect IE (6-11)
    } else if (/*@cc_on!@*/false || !!document.documentMode) {
        hasMatch = /\brv[ :]+(\d+)/g.exec(navigator.userAgent) || [];

        webrtcDetectedBrowser   = 'IE';
        webrtcDetectedVersion   = parseInt(hasMatch[1], 10);
        webrtcMinimumVersion    = 9;
        webrtcDetectedType      = 'plugin';
        webrtcDetectedDCSupport = 'SCTP';

        if (!webrtcDetectedVersion) {
            hasMatch = /\bMSIE[ :]+(\d+)/g.exec(navigator.userAgent) || [];

            webrtcDetectedVersion = parseInt(hasMatch[1] || '0', 10);
        }

        // Detect Edge (20+)
    } else if (!!window.StyleMedia || navigator.userAgent.match(/Edge\/(\d+).(\d+)$/)) {
        hasMatch = navigator.userAgent.match(/Edge\/(\d+).(\d+)$/) || [];

        // Previous webrtc/adapter uses minimum version as 10547 but checking in the Edge release history,
        // It's close to 13.10547 and ObjectRTC API is fully supported in that version

        webrtcDetectedBrowser   = 'edge';
        webrtcDetectedVersion   = parseFloat((hasMatch[0] || '0/0').split('/')[1], 10);
        webrtcMinimumVersion    = 13.10547;
        webrtcDetectedType      = 'ms';
        webrtcDetectedDCSupport = null;

        // Detect Firefox (1.0+)
        // Placed before Safari check to ensure Firefox on Android is detected
    } else if (typeof InstallTrigger !== 'undefined' || navigator.userAgent.indexOf('irefox') > 0) {
        hasMatch = navigator.userAgent.match(/Firefox\/([0-9]+)\./) || [];

        webrtcDetectedBrowser   = 'firefox';
        webrtcDetectedVersion   = parseInt(hasMatch[1] || '0', 10);
        webrtcMinimumVersion    = 33;
        webrtcDetectedType      = 'moz';
        webrtcDetectedDCSupport = 'SCTP';

        // Detect Chrome (1+ and mobile)
        // Placed before Safari check to ensure Chrome on Android is detected
    } else if ((!!window.chrome && !!window.chrome.webstore) || navigator.userAgent.indexOf('Chrom') > 0) {
        hasMatch = navigator.userAgent.match(/Chrom(e|ium)\/([0-9]+)\./i) || [];

        webrtcDetectedBrowser   = 'chrome';
        webrtcDetectedVersion   = parseInt(hasMatch[2] || '0', 10);
        webrtcMinimumVersion    = 38;
        webrtcDetectedType      = 'webkit';
        webrtcDetectedDCSupport = webrtcDetectedVersion > 30 ? 'SCTP' : 'RTP'; // Chrome 31+ supports SCTP without flags

        // Detect Safari
    } else if (/constructor/i.test(window.HTMLElement) || (function (p) { return p.toString() === "[object SafariRemoteNotification]"; })(!window['safari'] || safari.pushNotification) || navigator.userAgent.match(/AppleWebKit\/(\d+)\./) || navigator.userAgent.match(/Version\/(\d+).(\d+)/)) {
        hasMatch = navigator.userAgent.match(/version\/(\d+)/i) || [];
        AppleWebKitBuild = navigator.userAgent.match(/AppleWebKit\/(\d+)/i) || [];

        var isMobile      = navigator.userAgent.match(/(iPhone|iPad)/gi);
        var hasNativeImpl = AppleWebKitBuild.length >= 1 && AppleWebKitBuild[1] >= 604;
        webrtcDetectedBrowser   = 'safari';
        webrtcDetectedVersion   = parseInt(hasMatch[1] || '0', 10);
        webrtcMinimumVersion    = 7;
        if (isMobile) {
            webrtcDetectedType    = hasNativeImpl ? 'AppleWebKit' : null;
        } else { // desktop
            webrtcDetectedType    = hasNativeImpl && !AdapterJS.options.forceSafariPlugin ? 'AppleWebKit' : 'plugin';
        }
        webrtcDetectedDCSupport = 'SCTP';
    }

    // Scope it to AdapterJS and window for better consistency
    AdapterJS.webrtcDetectedBrowser   = window.webrtcDetectedBrowser   = webrtcDetectedBrowser;
    AdapterJS.webrtcDetectedVersion   = window.webrtcDetectedVersion   = webrtcDetectedVersion;
    AdapterJS.webrtcMinimumVersion    = window.webrtcMinimumVersion    = webrtcMinimumVersion;
    AdapterJS.webrtcDetectedType      = window.webrtcDetectedType      = webrtcDetectedType;
    AdapterJS.webrtcDetectedDCSupport = window.webrtcDetectedDCSupport = webrtcDetectedDCSupport;
};

AdapterJS.addEvent = function(elem, evnt, func) {
    if (elem.addEventListener) { // W3C DOM
        elem.addEventListener(evnt, func, false);
    } else if (elem.attachEvent) {// OLD IE DOM
        elem.attachEvent('on'+evnt, func);
    } else { // No much to do
        elem[evnt] = func;
    }
};

AdapterJS.renderNotificationBar = function (message, buttonText, buttonCallback) {
    // only inject once the page is ready
    if (document.readyState !== 'interactive' && document.readyState !== 'complete') {
        return;
    }

    var w = window;
    var i = document.createElement('iframe');
    i.name = 'adapterjs-alert';
    i.style.position = 'fixed';
    i.style.top = '-41px';
    i.style.left = 0;
    i.style.right = 0;
    i.style.width = '100%';
    i.style.height = '40px';
    i.style.backgroundColor = '#ffffe1';
    i.style.border = 'none';
    i.style.borderBottom = '1px solid #888888';
    i.style.zIndex = '9999999';
    if(typeof i.style.webkitTransition === 'string') {
        i.style.webkitTransition = 'all .5s ease-out';
    } else if(typeof i.style.transition === 'string') {
        i.style.transition = 'all .5s ease-out';
    }
    document.body.appendChild(i);
    var c = (i.contentWindow) ? i.contentWindow :
        (i.contentDocument.document) ? i.contentDocument.document : i.contentDocument;
    c.document.open();
    c.document.write('<span style="display: inline-block; font-family: Helvetica, Arial,' +
        'sans-serif; font-size: .9rem; padding: 4px; vertical-align: ' +
        'middle; cursor: default;">' + message + '</span>');
    if(buttonText && typeof buttonCallback === 'function') {
        c.document.write('<button id="okay">' + buttonText + '</button><button id="cancel">Cancel</button>');
        c.document.close();

        // On click on okay
        AdapterJS.addEvent(c.document.getElementById('okay'), 'click', function (e) {
            e.preventDefault();
            try {
                e.cancelBubble = true;
            } catch(error) { }
            buttonCallback(e);
        });

        // On click on Cancel - all bars has same logic so keeping it that way for now
        AdapterJS.addEvent(c.document.getElementById('cancel'), 'click', function(e) {
            w.document.body.removeChild(i);
        });
    } else {
        c.document.close();
    }
    setTimeout(function() {
        if(typeof i.style.webkitTransform === 'string') {
            i.style.webkitTransform = 'translateY(40px)';
        } else if(typeof i.style.transform === 'string') {
            i.style.transform = 'translateY(40px)';
        } else {
            i.style.top = '0px';
        }
    }, 300);
};

// -----------------------------------------------------------
// Detected webrtc implementation. Types are:
// - 'moz': Mozilla implementation of webRTC.
// - 'webkit': WebKit implementation of webRTC.
// - 'plugin': Using the plugin implementation.
webrtcDetectedType = null;

// Set the settings for creating DataChannels, MediaStream for
// Cross-browser compability.
// - This is only for SCTP based support browsers.
// the 'urls' attribute.
checkMediaDataChannelSettings =
    function (peerBrowserAgent, peerBrowserVersion, callback, constraints) {
        if (typeof callback !== 'function') {
            return;
        }
        var beOfferer = true;
        var isLocalFirefox = AdapterJS.webrtcDetectedBrowser === 'firefox';
        // Nightly version does not require MozDontOfferDataChannel for interop
        var isLocalFirefoxInterop = AdapterJS.webrtcDetectedType === 'moz' && AdapterJS.webrtcDetectedVersion > 30;
        var isPeerFirefox = peerBrowserAgent === 'firefox';
        var isPeerFirefoxInterop = peerBrowserAgent === 'firefox' &&
            ((peerBrowserVersion) ? (peerBrowserVersion > 30) : false);

        // Resends an updated version of constraints for MozDataChannel to work
        // If other userAgent is firefox and user is firefox, remove MozDataChannel
        if ((isLocalFirefox && isPeerFirefox) || (isLocalFirefoxInterop)) {
            try {
                delete constraints.mandatory.MozDontOfferDataChannel;
            } catch (error) {
                console.error('Failed deleting MozDontOfferDataChannel');
                console.error(error);
            }
        } else if ((isLocalFirefox && !isPeerFirefox)) {
            constraints.mandatory.MozDontOfferDataChannel = true;
        }
        if (!isLocalFirefox) {
            // temporary measure to remove Moz* constraints in non Firefox browsers
            for (var prop in constraints.mandatory) {
                if (constraints.mandatory.hasOwnProperty(prop)) {
                    if (prop.indexOf('Moz') !== -1) {
                        delete constraints.mandatory[prop];
                    }
                }
            }
        }
        // Firefox (not interopable) cannot offer DataChannel as it will cause problems to the
        // interopability of the media stream
        if (isLocalFirefox && !isPeerFirefox && !isLocalFirefoxInterop) {
            beOfferer = false;
        }
        callback(beOfferer, constraints);
    };

// Handles the differences for all browsers ice connection state output.
// - Tested outcomes are:
//   - Chrome (offerer)  : 'checking' > 'completed' > 'completed'
//   - Chrome (answerer) : 'checking' > 'connected'
//   - Firefox (offerer) : 'checking' > 'connected'
//   - Firefox (answerer): 'checking' > 'connected'
checkIceConnectionState = function (peerId, iceConnectionState, callback) {
    if (typeof callback !== 'function') {
        console.warn('No callback specified in checkIceConnectionState. Aborted.');
        return;
    }
    peerId = (peerId) ? peerId : 'peer';

    if (!AdapterJS._iceConnectionFiredStates[peerId] ||
        iceConnectionState === AdapterJS._iceConnectionStates.disconnected ||
        iceConnectionState === AdapterJS._iceConnectionStates.failed ||
        iceConnectionState === AdapterJS._iceConnectionStates.closed) {
        AdapterJS._iceConnectionFiredStates[peerId] = [];
    }
    iceConnectionState = AdapterJS._iceConnectionStates[iceConnectionState];
    if (AdapterJS._iceConnectionFiredStates[peerId].indexOf(iceConnectionState) < 0) {
        AdapterJS._iceConnectionFiredStates[peerId].push(iceConnectionState);
        if (iceConnectionState === AdapterJS._iceConnectionStates.connected) {
            setTimeout(function () {
                AdapterJS._iceConnectionFiredStates[peerId]
                    .push(AdapterJS._iceConnectionStates.done);
                callback(AdapterJS._iceConnectionStates.done);
            }, 1000);
        }
        callback(iceConnectionState);
    }
    return;
};

// Firefox:
// - Creates iceServer from the url for Firefox.
// - Create iceServer with stun url.
// - Create iceServer with turn url.
//   - Ignore the transport parameter from TURN url for FF version <=27.
//   - Return null for createIceServer if transport=tcp.
// - FF 27 and above supports transport parameters in TURN url,
// - So passing in the full url to create iceServer.
// Chrome:
// - Creates iceServer from the url for Chrome M33 and earlier.
//   - Create iceServer with stun url.
//   - Chrome M28 & above uses below TURN format.
// Plugin:
// - Creates Ice Server for Plugin Browsers
//   - If Stun - Create iceServer with stun url.
//   - Else - Create iceServer with turn url
//   - This is a WebRTC Function
createIceServer = null;

// Firefox:
// - Creates IceServers for Firefox
//   - Use .url for FireFox.
//   - Multiple Urls support
// Chrome:
// - Creates iceServers from the urls for Chrome M34 and above.
//   - .urls is supported since Chrome M34.
//   - Multiple Urls support
// Plugin:
// - Creates Ice Servers for Plugin Browsers
//   - Multiple Urls support
//   - This is a WebRTC Function
createIceServers = null;
//------------------------------------------------------------

//Creates MediaStream object.
MediaStream = (typeof MediaStream === 'function') ? MediaStream : null;

//The RTCPeerConnection object.
RTCPeerConnection = (typeof RTCPeerConnection === 'function') ?
    RTCPeerConnection : null;

// Creates RTCSessionDescription object for Plugin Browsers
RTCSessionDescription = (typeof RTCSessionDescription === 'function') ?
    RTCSessionDescription : null;

// Creates RTCIceCandidate object for Plugin Browsers
RTCIceCandidate = (typeof RTCIceCandidate === 'function') ?
    RTCIceCandidate : null;

// Get UserMedia (only difference is the prefix).
// Code from Adam Barth.
getUserMedia = null;

// Attach a media stream to an element.
attachMediaStream = null;

// Re-attach a media stream to an element.
reattachMediaStream = null;


// Detected browser agent name. Types are:
// - 'firefox': Firefox browser.
// - 'chrome': Chrome browser.
// - 'opera': Opera browser.
// - 'safari': Safari browser.
// - 'IE' - Internet Explorer browser.
webrtcDetectedBrowser = null;

// Detected browser version.
webrtcDetectedVersion = null;

// The minimum browser version still supported by AJS.
webrtcMinimumVersion  = null;

// The type of DC supported by the browser
webrtcDetectedDCSupport = null;

// The requestUserMedia used by plugin gUM
requestUserMedia = null;

// Check for browser types and react accordingly
AdapterJS.parseWebrtcDetectedBrowser();
if (['webkit', 'moz', 'ms', 'AppleWebKit'].indexOf(AdapterJS.webrtcDetectedType) > -1) {

    ///////////////////////////////////////////////////////////////////
    // INJECTION OF GOOGLE'S ADAPTER.JS CONTENT

    // Store the original native RTCPC in msRTCPeerConnection object
    if (navigator.userAgent.match(/Edge\/(\d+).(\d+)$/) && window.RTCPeerConnection) {
        window.msRTCPeerConnection = window.RTCPeerConnection;
    }

    /* jshint ignore:start */
    (function(f){if(typeof exports==="object"&&typeof module!=="undefined"){module.exports=f()}else if(typeof define==="function"&&define.amd){define([],f)}else{var g;if(typeof window!=="undefined"){g=window}else if(typeof global!=="undefined"){g=global}else if(typeof self!=="undefined"){g=self}else{g=this}g.adapter = f()}})(function(){var define,module,exports;return (function e(t,n,r){function s(o,u){if(!n[o]){if(!t[o]){var a=typeof require=="function"&&require;if(!u&&a)return a(o,!0);if(i)return i(o,!0);var f=new Error("Cannot find module '"+o+"'");throw f.code="MODULE_NOT_FOUND",f}var l=n[o]={exports:{}};t[o][0].call(l.exports,function(e){var n=t[o][1][e];return s(n?n:e)},l,l.exports,e,t,n,r)}return n[o].exports}var i=typeof require=="function"&&require;for(var o=0;o<r.length;o++)s(r[o]);return s})({1:[function(requirecopy,module,exports){
        /*
         *  Copyright (c) 2017 The WebRTC project authors. All Rights Reserved.
         *
         *  Use of this source code is governed by a BSD-style license
         *  that can be found in the LICENSE file in the root of the source
         *  tree.
         */
        /* eslint-env node */
        'use strict';

        var SDPUtils = requirecopy('sdp');

        function writeMediaSection(transceiver, caps, type, stream, dtlsRole) {
            var sdp = SDPUtils.writeRtpDescription(transceiver.kind, caps);

            // Map ICE parameters (ufrag, pwd) to SDP.
            sdp += SDPUtils.writeIceParameters(
                transceiver.iceGatherer.getLocalParameters());

            // Map DTLS parameters to SDP.
            sdp += SDPUtils.writeDtlsParameters(
                transceiver.dtlsTransport.getLocalParameters(),
                type === 'offer' ? 'actpass' : dtlsRole || 'active');

            sdp += 'a=mid:' + transceiver.mid + '\r\n';

            if (transceiver.direction) {
                sdp += 'a=' + transceiver.direction + '\r\n';
            } else if (transceiver.rtpSender && transceiver.rtpReceiver) {
                sdp += 'a=sendrecv\r\n';
            } else if (transceiver.rtpSender) {
                sdp += 'a=sendonly\r\n';
            } else if (transceiver.rtpReceiver) {
                sdp += 'a=recvonly\r\n';
            } else {
                sdp += 'a=inactive\r\n';
            }

            if (transceiver.rtpSender) {
                // spec.
                var msid = 'msid:' + stream.id + ' ' +
                    transceiver.rtpSender.track.id + '\r\n';
                sdp += 'a=' + msid;

                // for Chrome.
                sdp += 'a=ssrc:' + transceiver.sendEncodingParameters[0].ssrc +
                    ' ' + msid;
                if (transceiver.sendEncodingParameters[0].rtx) {
                    sdp += 'a=ssrc:' + transceiver.sendEncodingParameters[0].rtx.ssrc +
                        ' ' + msid;
                    sdp += 'a=ssrc-group:FID ' +
                        transceiver.sendEncodingParameters[0].ssrc + ' ' +
                        transceiver.sendEncodingParameters[0].rtx.ssrc +
                        '\r\n';
                }
            }
            // FIXME: this should be written by writeRtpDescription.
            sdp += 'a=ssrc:' + transceiver.sendEncodingParameters[0].ssrc +
                ' cname:' + SDPUtils.localCName + '\r\n';
            if (transceiver.rtpSender && transceiver.sendEncodingParameters[0].rtx) {
                sdp += 'a=ssrc:' + transceiver.sendEncodingParameters[0].rtx.ssrc +
                    ' cname:' + SDPUtils.localCName + '\r\n';
            }
            return sdp;
        }

// Edge does not like
// 1) stun: filtered after 14393 unless ?transport=udp is present
// 2) turn: that does not have all of turn:host:port?transport=udp
// 3) turn: with ipv6 addresses
// 4) turn: occurring muliple times
        function filterIceServers(iceServers, edgeVersion) {
            var hasTurn = false;
            iceServers = JSON.parse(JSON.stringify(iceServers));
            return iceServers.filter(function(server) {
                if (server && (server.urls || server.url)) {
                    var urls = server.urls || server.url;
                    if (server.url && !server.urls) {
                        console.warn('RTCIceServer.url is deprecated! Use urls instead.');
                    }
                    var isString = typeof urls === 'string';
                    if (isString) {
                        urls = [urls];
                    }
                    urls = urls.filter(function(url) {
                        var validTurn = url.indexOf('turn:') === 0 &&
                            url.indexOf('transport=udp') !== -1 &&
                            url.indexOf('turn:[') === -1 &&
                            !hasTurn;

                        if (validTurn) {
                            hasTurn = true;
                            return true;
                        }
                        return url.indexOf('stun:') === 0 && edgeVersion >= 14393 &&
                            url.indexOf('?transport=udp') === -1;
                    });

                    delete server.url;
                    server.urls = isString ? urls[0] : urls;
                    return !!urls.length;
                }
                return false;
            });
        }

// Determines the intersection of local and remote capabilities.
        function getCommonCapabilities(localCapabilities, remoteCapabilities) {
            var commonCapabilities = {
                codecs: [],
                headerExtensions: [],
                fecMechanisms: []
            };

            var findCodecByPayloadType = function(pt, codecs) {
                pt = parseInt(pt, 10);
                for (var i = 0; i < codecs.length; i++) {
                    if (codecs[i].payloadType === pt ||
                        codecs[i].preferredPayloadType === pt) {
                        return codecs[i];
                    }
                }
            };

            var rtxCapabilityMatches = function(lRtx, rRtx, lCodecs, rCodecs) {
                var lCodec = findCodecByPayloadType(lRtx.parameters.apt, lCodecs);
                var rCodec = findCodecByPayloadType(rRtx.parameters.apt, rCodecs);
                return lCodec && rCodec &&
                    lCodec.name.toLowerCase() === rCodec.name.toLowerCase();
            };

            localCapabilities.codecs.forEach(function(lCodec) {
                for (var i = 0; i < remoteCapabilities.codecs.length; i++) {
                    var rCodec = remoteCapabilities.codecs[i];
                    if (lCodec.name.toLowerCase() === rCodec.name.toLowerCase() &&
                        lCodec.clockRate === rCodec.clockRate) {
                        if (lCodec.name.toLowerCase() === 'rtx' &&
                            lCodec.parameters && rCodec.parameters.apt) {
                            // for RTX we need to find the local rtx that has a apt
                            // which points to the same local codec as the remote one.
                            if (!rtxCapabilityMatches(lCodec, rCodec,
                                    localCapabilities.codecs, remoteCapabilities.codecs)) {
                                continue;
                            }
                        }
                        rCodec = JSON.parse(JSON.stringify(rCodec)); // deepcopy
                        // number of channels is the highest common number of channels
                        rCodec.numChannels = Math.min(lCodec.numChannels,
                            rCodec.numChannels);
                        // push rCodec so we reply with offerer payload type
                        commonCapabilities.codecs.push(rCodec);

                        // determine common feedback mechanisms
                        rCodec.rtcpFeedback = rCodec.rtcpFeedback.filter(function(fb) {
                            for (var j = 0; j < lCodec.rtcpFeedback.length; j++) {
                                if (lCodec.rtcpFeedback[j].type === fb.type &&
                                    lCodec.rtcpFeedback[j].parameter === fb.parameter) {
                                    return true;
                                }
                            }
                            return false;
                        });
                        // FIXME: also need to determine .parameters
                        //  see https://github.com/openpeer/ortc/issues/569
                        break;
                    }
                }
            });

            localCapabilities.headerExtensions.forEach(function(lHeaderExtension) {
                for (var i = 0; i < remoteCapabilities.headerExtensions.length;
                     i++) {
                    var rHeaderExtension = remoteCapabilities.headerExtensions[i];
                    if (lHeaderExtension.uri === rHeaderExtension.uri) {
                        commonCapabilities.headerExtensions.push(rHeaderExtension);
                        break;
                    }
                }
            });

            // FIXME: fecMechanisms
            return commonCapabilities;
        }

// is action=setLocalDescription with type allowed in signalingState
        function isActionAllowedInSignalingState(action, type, signalingState) {
            return {
                    offer: {
                        setLocalDescription: ['stable', 'have-local-offer'],
                        setRemoteDescription: ['stable', 'have-remote-offer']
                    },
                    answer: {
                        setLocalDescription: ['have-remote-offer', 'have-local-pranswer'],
                        setRemoteDescription: ['have-local-offer', 'have-remote-pranswer']
                    }
                }[type][action].indexOf(signalingState) !== -1;
        }

        function maybeAddCandidate(iceTransport, candidate) {
            // Edge's internal representation adds some fields therefore
            // not all fieldѕ are taken into account.
            var alreadyAdded = iceTransport.getRemoteCandidates()
                .find(function(remoteCandidate) {
                    return candidate.foundation === remoteCandidate.foundation &&
                        candidate.ip === remoteCandidate.ip &&
                        candidate.port === remoteCandidate.port &&
                        candidate.priority === remoteCandidate.priority &&
                        candidate.protocol === remoteCandidate.protocol &&
                        candidate.type === remoteCandidate.type;
                });
            if (!alreadyAdded) {
                iceTransport.addRemoteCandidate(candidate);
            }
            return !alreadyAdded;
        }

        module.exports = function(window, edgeVersion) {
            var RTCPeerConnection = function(config) {
                var self = this;

                var _eventTarget = document.createDocumentFragment();
                ['addEventListener', 'removeEventListener', 'dispatchEvent']
                    .forEach(function(method) {
                        self[method] = _eventTarget[method].bind(_eventTarget);
                    });

                this.onicecandidate = null;
                this.onaddstream = null;
                this.ontrack = null;
                this.onremovestream = null;
                this.onsignalingstatechange = null;
                this.oniceconnectionstatechange = null;
                this.onicegatheringstatechange = null;
                this.onnegotiationneeded = null;
                this.ondatachannel = null;
                this.canTrickleIceCandidates = null;

                this.needNegotiation = false;

                this.localStreams = [];
                this.remoteStreams = [];

                this.localDescription = null;
                this.remoteDescription = null;

                this.signalingState = 'stable';
                this.iceConnectionState = 'new';
                this.iceGatheringState = 'new';

                config = JSON.parse(JSON.stringify(config || {}));

                this.usingBundle = config.bundlePolicy === 'max-bundle';
                if (config.rtcpMuxPolicy === 'negotiate') {
                    var e = new Error('rtcpMuxPolicy \'negotiate\' is not supported');
                    e.name = 'NotSupportedError';
                    throw(e);
                } else if (!config.rtcpMuxPolicy) {
                    config.rtcpMuxPolicy = 'require';
                }

                switch (config.iceTransportPolicy) {
                    case 'all':
                    case 'relay':
                        break;
                    default:
                        config.iceTransportPolicy = 'all';
                        break;
                }

                switch (config.bundlePolicy) {
                    case 'balanced':
                    case 'max-compat':
                    case 'max-bundle':
                        break;
                    default:
                        config.bundlePolicy = 'balanced';
                        break;
                }

                config.iceServers = filterIceServers(config.iceServers || [], edgeVersion);

                this._iceGatherers = [];
                if (config.iceCandidatePoolSize) {
                    for (var i = config.iceCandidatePoolSize; i > 0; i--) {
                        this._iceGatherers = new window.RTCIceGatherer({
                            iceServers: config.iceServers,
                            gatherPolicy: config.iceTransportPolicy
                        });
                    }
                } else {
                    config.iceCandidatePoolSize = 0;
                }

                this._config = config;

                // per-track iceGathers, iceTransports, dtlsTransports, rtpSenders, ...
                // everything that is needed to describe a SDP m-line.
                this.transceivers = [];

                this._sdpSessionId = SDPUtils.generateSessionId();
                this._sdpSessionVersion = 0;

                this._dtlsRole = undefined; // role for a=setup to use in answers.
            };

            RTCPeerConnection.prototype._emitGatheringStateChange = function() {
                var event = new Event('icegatheringstatechange');
                this.dispatchEvent(event);
                if (typeof this.onicegatheringstatechange === 'function') {
                    this.onicegatheringstatechange(event);
                }
            };

            RTCPeerConnection.prototype.getConfiguration = function() {
                return this._config;
            };

            RTCPeerConnection.prototype.getLocalStreams = function() {
                return this.localStreams;
            };

            RTCPeerConnection.prototype.getRemoteStreams = function() {
                return this.remoteStreams;
            };

            // internal helper to create a transceiver object.
            // (whih is not yet the same as the WebRTC 1.0 transceiver)
            RTCPeerConnection.prototype._createTransceiver = function(kind) {
                var hasBundleTransport = this.transceivers.length > 0;
                var transceiver = {
                    track: null,
                    iceGatherer: null,
                    iceTransport: null,
                    dtlsTransport: null,
                    localCapabilities: null,
                    remoteCapabilities: null,
                    rtpSender: null,
                    rtpReceiver: null,
                    kind: kind,
                    mid: null,
                    sendEncodingParameters: null,
                    recvEncodingParameters: null,
                    stream: null,
                    wantReceive: true
                };
                if (this.usingBundle && hasBundleTransport) {
                    transceiver.iceTransport = this.transceivers[0].iceTransport;
                    transceiver.dtlsTransport = this.transceivers[0].dtlsTransport;
                } else {
                    var transports = this._createIceAndDtlsTransports();
                    transceiver.iceTransport = transports.iceTransport;
                    transceiver.dtlsTransport = transports.dtlsTransport;
                }
                this.transceivers.push(transceiver);
                return transceiver;
            };

            RTCPeerConnection.prototype.addTrack = function(track, stream) {
                var transceiver;
                for (var i = 0; i < this.transceivers.length; i++) {
                    if (!this.transceivers[i].track &&
                        this.transceivers[i].kind === track.kind) {
                        transceiver = this.transceivers[i];
                    }
                }
                if (!transceiver) {
                    transceiver = this._createTransceiver(track.kind);
                }

                this._maybeFireNegotiationNeeded();

                if (this.localStreams.indexOf(stream) === -1) {
                    this.localStreams.push(stream);
                }

                transceiver.track = track;
                transceiver.stream = stream;
                transceiver.rtpSender = new window.RTCRtpSender(track,
                    transceiver.dtlsTransport);
                return transceiver.rtpSender;
            };

            RTCPeerConnection.prototype.addStream = function(stream) {
                var self = this;
                if (edgeVersion >= 15025) {
                    stream.getTracks().forEach(function(track) {
                        self.addTrack(track, stream);
                    });
                } else {
                    // Clone is necessary for local demos mostly, attaching directly
                    // to two different senders does not work (build 10547).
                    // Fixed in 15025 (or earlier)
                    var clonedStream = stream.clone();
                    stream.getTracks().forEach(function(track, idx) {
                        var clonedTrack = clonedStream.getTracks()[idx];
                        track.addEventListener('enabled', function(event) {
                            clonedTrack.enabled = event.enabled;
                        });
                    });
                    clonedStream.getTracks().forEach(function(track) {
                        self.addTrack(track, clonedStream);
                    });
                }
            };

            RTCPeerConnection.prototype.removeStream = function(stream) {
                var idx = this.localStreams.indexOf(stream);
                if (idx > -1) {
                    this.localStreams.splice(idx, 1);
                    this._maybeFireNegotiationNeeded();
                }
            };

            RTCPeerConnection.prototype.getSenders = function() {
                return this.transceivers.filter(function(transceiver) {
                    return !!transceiver.rtpSender;
                })
                    .map(function(transceiver) {
                        return transceiver.rtpSender;
                    });
            };

            RTCPeerConnection.prototype.getReceivers = function() {
                return this.transceivers.filter(function(transceiver) {
                    return !!transceiver.rtpReceiver;
                })
                    .map(function(transceiver) {
                        return transceiver.rtpReceiver;
                    });
            };


            RTCPeerConnection.prototype._createIceGatherer = function(sdpMLineIndex,
                                                                      usingBundle) {
                var self = this;
                if (usingBundle && sdpMLineIndex > 0) {
                    return this.transceivers[0].iceGatherer;
                } else if (this._iceGatherers.length) {
                    return this._iceGatherers.shift();
                }
                var iceGatherer = new window.RTCIceGatherer({
                    iceServers: this._config.iceServers,
                    gatherPolicy: this._config.iceTransportPolicy
                });
                Object.defineProperty(iceGatherer, 'state',
                    {value: 'new', writable: true}
                );

                this.transceivers[sdpMLineIndex].candidates = [];
                this.transceivers[sdpMLineIndex].bufferCandidates = function(event) {
                    var end = !event.candidate || Object.keys(event.candidate).length === 0;
                    // polyfill since RTCIceGatherer.state is not implemented in
                    // Edge 10547 yet.
                    iceGatherer.state = end ? 'completed' : 'gathering';
                    if (self.transceivers[sdpMLineIndex].candidates !== null) {
                        self.transceivers[sdpMLineIndex].candidates.push(event.candidate);
                    }
                };
                iceGatherer.addEventListener('localcandidate',
                    this.transceivers[sdpMLineIndex].bufferCandidates);
                return iceGatherer;
            };

            // start gathering from an RTCIceGatherer.
            RTCPeerConnection.prototype._gather = function(mid, sdpMLineIndex) {
                var self = this;
                var iceGatherer = this.transceivers[sdpMLineIndex].iceGatherer;
                if (iceGatherer.onlocalcandidate) {
                    return;
                }
                var candidates = this.transceivers[sdpMLineIndex].candidates;
                this.transceivers[sdpMLineIndex].candidates = null;
                iceGatherer.removeEventListener('localcandidate',
                    this.transceivers[sdpMLineIndex].bufferCandidates);
                iceGatherer.onlocalcandidate = function(evt) {
                    if (self.usingBundle && sdpMLineIndex > 0) {
                        // if we know that we use bundle we can drop candidates with
                        // ѕdpMLineIndex > 0. If we don't do this then our state gets
                        // confused since we dispose the extra ice gatherer.
                        return;
                    }
                    var event = new Event('icecandidate');
                    event.candidate = {sdpMid: mid, sdpMLineIndex: sdpMLineIndex};

                    var cand = evt.candidate;
                    // Edge emits an empty object for RTCIceCandidateComplete‥
                    var end = !cand || Object.keys(cand).length === 0;
                    if (end) {
                        // polyfill since RTCIceGatherer.state is not implemented in
                        // Edge 10547 yet.
                        if (iceGatherer.state === 'new' || iceGatherer.state === 'gathering') {
                            iceGatherer.state = 'completed';
                        }
                    } else {
                        if (iceGatherer.state === 'new') {
                            iceGatherer.state = 'gathering';
                        }
                        // RTCIceCandidate doesn't have a component, needs to be added
                        cand.component = 1;
                        event.candidate.candidate = SDPUtils.writeCandidate(cand);
                    }

                    // update local description.
                    var sections = SDPUtils.splitSections(self.localDescription.sdp);
                    if (!end) {
                        sections[event.candidate.sdpMLineIndex + 1] +=
                            'a=' + event.candidate.candidate + '\r\n';
                    } else {
                        sections[event.candidate.sdpMLineIndex + 1] +=
                            'a=end-of-candidates\r\n';
                    }
                    self.localDescription.sdp = sections.join('');
                    var complete = self.transceivers.every(function(transceiver) {
                        return transceiver.iceGatherer &&
                            transceiver.iceGatherer.state === 'completed';
                    });

                    if (self.iceGatheringState !== 'gathering') {
                        self.iceGatheringState = 'gathering';
                        self._emitGatheringStateChange();
                    }

                    // Emit candidate. Also emit null candidate when all gatherers are
                    // complete.
                    if (!end) {
                        self.dispatchEvent(event);
                        if (typeof self.onicecandidate === 'function') {
                            self.onicecandidate(event);
                        }
                    }
                    if (complete) {
                        self.dispatchEvent(new Event('icecandidate'));
                        if (typeof self.onicecandidate === 'function') {
                            self.onicecandidate(new Event('icecandidate'));
                        }
                        self.iceGatheringState = 'complete';
                        self._emitGatheringStateChange();
                    }
                };

                // emit already gathered candidates.
                window.setTimeout(function() {
                    candidates.forEach(function(candidate) {
                        var e = new Event('RTCIceGatherEvent');
                        e.candidate = candidate;
                        iceGatherer.onlocalcandidate(e);
                    });
                }, 0);
            };

            // Create ICE transport and DTLS transport.
            RTCPeerConnection.prototype._createIceAndDtlsTransports = function() {
                var self = this;
                var iceTransport = new window.RTCIceTransport(null);
                iceTransport.onicestatechange = function() {
                    self._updateConnectionState();
                };

                var dtlsTransport = new window.RTCDtlsTransport(iceTransport);
                dtlsTransport.ondtlsstatechange = function() {
                    self._updateConnectionState();
                };
                dtlsTransport.onerror = function() {
                    // onerror does not set state to failed by itself.
                    Object.defineProperty(dtlsTransport, 'state',
                        {value: 'failed', writable: true});
                    self._updateConnectionState();
                };

                return {
                    iceTransport: iceTransport,
                    dtlsTransport: dtlsTransport
                };
            };

            // Destroy ICE gatherer, ICE transport and DTLS transport.
            // Without triggering the callbacks.
            RTCPeerConnection.prototype._disposeIceAndDtlsTransports = function(
                sdpMLineIndex) {
                var iceGatherer = this.transceivers[sdpMLineIndex].iceGatherer;
                if (iceGatherer) {
                    delete iceGatherer.onlocalcandidate;
                    delete this.transceivers[sdpMLineIndex].iceGatherer;
                }
                var iceTransport = this.transceivers[sdpMLineIndex].iceTransport;
                if (iceTransport) {
                    delete iceTransport.onicestatechange;
                    delete this.transceivers[sdpMLineIndex].iceTransport;
                }
                var dtlsTransport = this.transceivers[sdpMLineIndex].dtlsTransport;
                if (dtlsTransport) {
                    delete dtlsTransport.ondtlsstatechange;
                    delete dtlsTransport.onerror;
                    delete this.transceivers[sdpMLineIndex].dtlsTransport;
                }
            };

            // Start the RTP Sender and Receiver for a transceiver.
            RTCPeerConnection.prototype._transceive = function(transceiver,
                                                               send, recv) {
                var params = getCommonCapabilities(transceiver.localCapabilities,
                    transceiver.remoteCapabilities);
                if (send && transceiver.rtpSender) {
                    params.encodings = transceiver.sendEncodingParameters;
                    params.rtcp = {
                        cname: SDPUtils.localCName,
                        compound: transceiver.rtcpParameters.compound
                    };
                    if (transceiver.recvEncodingParameters.length) {
                        params.rtcp.ssrc = transceiver.recvEncodingParameters[0].ssrc;
                    }
                    transceiver.rtpSender.send(params);
                }
                if (recv && transceiver.rtpReceiver && params.codecs.length > 0) {
                    // remove RTX field in Edge 14942
                    if (transceiver.kind === 'video'
                        && transceiver.recvEncodingParameters
                        && edgeVersion < 15019) {
                        transceiver.recvEncodingParameters.forEach(function(p) {
                            delete p.rtx;
                        });
                    }
                    params.encodings = transceiver.recvEncodingParameters;
                    params.rtcp = {
                        cname: transceiver.rtcpParameters.cname,
                        compound: transceiver.rtcpParameters.compound
                    };
                    if (transceiver.sendEncodingParameters.length) {
                        params.rtcp.ssrc = transceiver.sendEncodingParameters[0].ssrc;
                    }
                    transceiver.rtpReceiver.receive(params);
                }
            };

            RTCPeerConnection.prototype.setLocalDescription = function(description) {
                var self = this;
                var args = arguments;

                if (!isActionAllowedInSignalingState('setLocalDescription',
                        description.type, this.signalingState)) {
                    return new Promise(function(resolve, reject) {
                        var e = new Error('Can not set local ' + description.type +
                            ' in state ' + self.signalingState);
                        e.name = 'InvalidStateError';
                        if (args.length > 2 && typeof args[2] === 'function') {
                            args[2].apply(null, [e]);
                        }
                        reject(e);
                    });
                }

                var sections;
                var sessionpart;
                if (description.type === 'offer') {
                    // VERY limited support for SDP munging. Limited to:
                    // * changing the order of codecs
                    sections = SDPUtils.splitSections(description.sdp);
                    sessionpart = sections.shift();
                    sections.forEach(function(mediaSection, sdpMLineIndex) {
                        var caps = SDPUtils.parseRtpParameters(mediaSection);
                        self.transceivers[sdpMLineIndex].localCapabilities = caps;
                    });

                    this.transceivers.forEach(function(transceiver, sdpMLineIndex) {
                        self._gather(transceiver.mid, sdpMLineIndex);
                    });
                } else if (description.type === 'answer') {
                    sections = SDPUtils.splitSections(self.remoteDescription.sdp);
                    sessionpart = sections.shift();
                    var isIceLite = SDPUtils.matchPrefix(sessionpart,
                            'a=ice-lite').length > 0;
                    sections.forEach(function(mediaSection, sdpMLineIndex) {
                        var transceiver = self.transceivers[sdpMLineIndex];
                        var iceGatherer = transceiver.iceGatherer;
                        var iceTransport = transceiver.iceTransport;
                        var dtlsTransport = transceiver.dtlsTransport;
                        var localCapabilities = transceiver.localCapabilities;
                        var remoteCapabilities = transceiver.remoteCapabilities;

                        // treat bundle-only as not-rejected.
                        var rejected = SDPUtils.isRejected(mediaSection) &&
                            !SDPUtils.matchPrefix(mediaSection, 'a=bundle-only').length === 1;

                        if (!rejected && !transceiver.isDatachannel) {
                            var remoteIceParameters = SDPUtils.getIceParameters(
                                mediaSection, sessionpart);
                            var remoteDtlsParameters = SDPUtils.getDtlsParameters(
                                mediaSection, sessionpart);
                            if (isIceLite) {
                                remoteDtlsParameters.role = 'server';
                            }

                            if (!self.usingBundle || sdpMLineIndex === 0) {
                                self._gather(transceiver.mid, sdpMLineIndex);
                                if (iceTransport.state === 'new') {
                                    iceTransport.start(iceGatherer, remoteIceParameters,
                                        isIceLite ? 'controlling' : 'controlled');
                                }
                                if (dtlsTransport.state === 'new') {
                                    dtlsTransport.start(remoteDtlsParameters);
                                }
                            }

                            // Calculate intersection of capabilities.
                            var params = getCommonCapabilities(localCapabilities,
                                remoteCapabilities);

                            // Start the RTCRtpSender. The RTCRtpReceiver for this
                            // transceiver has already been started in setRemoteDescription.
                            self._transceive(transceiver,
                                params.codecs.length > 0,
                                false);
                        }
                    });
                }

                this.localDescription = {
                    type: description.type,
                    sdp: description.sdp
                };
                switch (description.type) {
                    case 'offer':
                        this._updateSignalingState('have-local-offer');
                        break;
                    case 'answer':
                        this._updateSignalingState('stable');
                        break;
                    default:
                        throw new TypeError('unsupported type "' + description.type +
                            '"');
                }

                // If a success callback was provided, emit ICE candidates after it
                // has been executed. Otherwise, emit callback after the Promise is
                // resolved.
                var cb = arguments.length > 1 && typeof arguments[1] === 'function' &&
                    arguments[1];
                return new Promise(function(resolve) {
                    if (cb) {
                        cb.apply(null);
                    }
                    resolve();
                });
            };

            RTCPeerConnection.prototype.setRemoteDescription = function(description) {
                var self = this;
                var args = arguments;

                if (!isActionAllowedInSignalingState('setRemoteDescription',
                        description.type, this.signalingState)) {
                    return new Promise(function(resolve, reject) {
                        var e = new Error('Can not set remote ' + description.type +
                            ' in state ' + self.signalingState);
                        e.name = 'InvalidStateError';
                        if (args.length > 2 && typeof args[2] === 'function') {
                            args[2].apply(null, [e]);
                        }
                        reject(e);
                    });
                }

                var streams = {};
                this.remoteStreams.forEach(function(stream) {
                    streams[stream.id] = stream;
                });
                var receiverList = [];
                var sections = SDPUtils.splitSections(description.sdp);
                var sessionpart = sections.shift();
                var isIceLite = SDPUtils.matchPrefix(sessionpart,
                        'a=ice-lite').length > 0;
                var usingBundle = SDPUtils.matchPrefix(sessionpart,
                        'a=group:BUNDLE ').length > 0;
                this.usingBundle = usingBundle;
                var iceOptions = SDPUtils.matchPrefix(sessionpart,
                    'a=ice-options:')[0];
                if (iceOptions) {
                    this.canTrickleIceCandidates = iceOptions.substr(14).split(' ')
                            .indexOf('trickle') >= 0;
                } else {
                    this.canTrickleIceCandidates = false;
                }

                sections.forEach(function(mediaSection, sdpMLineIndex) {
                    var lines = SDPUtils.splitLines(mediaSection);
                    var kind = SDPUtils.getKind(mediaSection);
                    // treat bundle-only as not-rejected.
                    var rejected = SDPUtils.isRejected(mediaSection) &&
                        !SDPUtils.matchPrefix(mediaSection, 'a=bundle-only').length === 1;
                    var protocol = lines[0].substr(2).split(' ')[2];

                    var direction = SDPUtils.getDirection(mediaSection, sessionpart);
                    var remoteMsid = SDPUtils.parseMsid(mediaSection);

                    var mid = SDPUtils.getMid(mediaSection) || SDPUtils.generateIdentifier();

                    // Reject datachannels which are not implemented yet.
                    if (kind === 'application' && protocol === 'DTLS/SCTP') {
                        self.transceivers[sdpMLineIndex] = {
                            mid: mid,
                            isDatachannel: true
                        };
                        return;
                    }

                    var transceiver;
                    var iceGatherer;
                    var iceTransport;
                    var dtlsTransport;
                    var rtpReceiver;
                    var sendEncodingParameters;
                    var recvEncodingParameters;
                    var localCapabilities;

                    var track;
                    // FIXME: ensure the mediaSection has rtcp-mux set.
                    var remoteCapabilities = SDPUtils.parseRtpParameters(mediaSection);
                    var remoteIceParameters;
                    var remoteDtlsParameters;
                    if (!rejected) {
                        remoteIceParameters = SDPUtils.getIceParameters(mediaSection,
                            sessionpart);
                        remoteDtlsParameters = SDPUtils.getDtlsParameters(mediaSection,
                            sessionpart);
                        remoteDtlsParameters.role = 'client';
                    }
                    recvEncodingParameters =
                        SDPUtils.parseRtpEncodingParameters(mediaSection);

                    var rtcpParameters = SDPUtils.parseRtcpParameters(mediaSection);

                    var isComplete = SDPUtils.matchPrefix(mediaSection,
                            'a=end-of-candidates', sessionpart).length > 0;
                    var cands = SDPUtils.matchPrefix(mediaSection, 'a=candidate:')
                        .map(function(cand) {
                            return SDPUtils.parseCandidate(cand);
                        })
                        .filter(function(cand) {
                            return cand.component === 1;
                        });

                    // Check if we can use BUNDLE and dispose transports.
                    if ((description.type === 'offer' || description.type === 'answer') &&
                        !rejected && usingBundle && sdpMLineIndex > 0 &&
                        self.transceivers[sdpMLineIndex]) {
                        self._disposeIceAndDtlsTransports(sdpMLineIndex);
                        self.transceivers[sdpMLineIndex].iceGatherer =
                            self.transceivers[0].iceGatherer;
                        self.transceivers[sdpMLineIndex].iceTransport =
                            self.transceivers[0].iceTransport;
                        self.transceivers[sdpMLineIndex].dtlsTransport =
                            self.transceivers[0].dtlsTransport;
                        if (self.transceivers[sdpMLineIndex].rtpSender) {
                            self.transceivers[sdpMLineIndex].rtpSender.setTransport(
                                self.transceivers[0].dtlsTransport);
                        }
                        if (self.transceivers[sdpMLineIndex].rtpReceiver) {
                            self.transceivers[sdpMLineIndex].rtpReceiver.setTransport(
                                self.transceivers[0].dtlsTransport);
                        }
                    }
                    if (description.type === 'offer' && !rejected) {
                        transceiver = self.transceivers[sdpMLineIndex] ||
                            self._createTransceiver(kind);
                        transceiver.mid = mid;

                        if (!transceiver.iceGatherer) {
                            transceiver.iceGatherer = self._createIceGatherer(sdpMLineIndex,
                                usingBundle);
                        }

                        if (cands.length && transceiver.iceTransport.state === 'new') {
                            if (isComplete && (!usingBundle || sdpMLineIndex === 0)) {
                                transceiver.iceTransport.setRemoteCandidates(cands);
                            } else {
                                cands.forEach(function(candidate) {
                                    maybeAddCandidate(transceiver.iceTransport, candidate);
                                });
                            }
                        }

                        localCapabilities = window.RTCRtpReceiver.getCapabilities(kind);

                        // filter RTX until additional stuff needed for RTX is implemented
                        // in adapter.js
                        if (edgeVersion < 15019) {
                            localCapabilities.codecs = localCapabilities.codecs.filter(
                                function(codec) {
                                    return codec.name !== 'rtx';
                                });
                        }

                        sendEncodingParameters = transceiver.sendEncodingParameters || [{
                                ssrc: (2 * sdpMLineIndex + 2) * 1001
                            }];

                        var isNewTrack = false;
                        if (direction === 'sendrecv' || direction === 'sendonly') {
                            isNewTrack = !transceiver.rtpReceiver;
                            rtpReceiver = transceiver.rtpReceiver ||
                                new window.RTCRtpReceiver(transceiver.dtlsTransport, kind);

                            if (isNewTrack) {
                                var stream;
                                track = rtpReceiver.track;
                                // FIXME: does not work with Plan B.
                                if (remoteMsid) {
                                    if (!streams[remoteMsid.stream]) {
                                        streams[remoteMsid.stream] = new window.MediaStream();
                                        Object.defineProperty(streams[remoteMsid.stream], 'id', {
                                            get: function() {
                                                return remoteMsid.stream;
                                            }
                                        });
                                    }
                                    Object.defineProperty(track, 'id', {
                                        get: function() {
                                            return remoteMsid.track;
                                        }
                                    });
                                    stream = streams[remoteMsid.stream];
                                } else {
                                    if (!streams.default) {
                                        streams.default = new window.MediaStream();
                                    }
                                    stream = streams.default;
                                }
                                stream.addTrack(track);
                                receiverList.push([track, rtpReceiver, stream]);
                            }
                        }

                        transceiver.localCapabilities = localCapabilities;
                        transceiver.remoteCapabilities = remoteCapabilities;
                        transceiver.rtpReceiver = rtpReceiver;
                        transceiver.rtcpParameters = rtcpParameters;
                        transceiver.sendEncodingParameters = sendEncodingParameters;
                        transceiver.recvEncodingParameters = recvEncodingParameters;

                        // Start the RTCRtpReceiver now. The RTPSender is started in
                        // setLocalDescription.
                        self._transceive(self.transceivers[sdpMLineIndex],
                            false,
                            isNewTrack);
                    } else if (description.type === 'answer' && !rejected) {
                        transceiver = self.transceivers[sdpMLineIndex];
                        iceGatherer = transceiver.iceGatherer;
                        iceTransport = transceiver.iceTransport;
                        dtlsTransport = transceiver.dtlsTransport;
                        rtpReceiver = transceiver.rtpReceiver;
                        sendEncodingParameters = transceiver.sendEncodingParameters;
                        localCapabilities = transceiver.localCapabilities;

                        self.transceivers[sdpMLineIndex].recvEncodingParameters =
                            recvEncodingParameters;
                        self.transceivers[sdpMLineIndex].remoteCapabilities =
                            remoteCapabilities;
                        self.transceivers[sdpMLineIndex].rtcpParameters = rtcpParameters;

                        if (cands.length && iceTransport.state === 'new') {
                            if ((isIceLite || isComplete) &&
                                (!usingBundle || sdpMLineIndex === 0)) {
                                iceTransport.setRemoteCandidates(cands);
                            } else {
                                cands.forEach(function(candidate) {
                                    maybeAddCandidate(transceiver.iceTransport, candidate);
                                });
                            }
                        }

                        if (!usingBundle || sdpMLineIndex === 0) {
                            if (iceTransport.state === 'new') {
                                iceTransport.start(iceGatherer, remoteIceParameters,
                                    'controlling');
                            }
                            if (dtlsTransport.state === 'new') {
                                dtlsTransport.start(remoteDtlsParameters);
                            }
                        }

                        self._transceive(transceiver,
                            direction === 'sendrecv' || direction === 'recvonly',
                            direction === 'sendrecv' || direction === 'sendonly');

                        if (rtpReceiver &&
                            (direction === 'sendrecv' || direction === 'sendonly')) {
                            track = rtpReceiver.track;
                            if (remoteMsid) {
                                if (!streams[remoteMsid.stream]) {
                                    streams[remoteMsid.stream] = new window.MediaStream();
                                }
                                streams[remoteMsid.stream].addTrack(track);
                                receiverList.push([track, rtpReceiver, streams[remoteMsid.stream]]);
                            } else {
                                if (!streams.default) {
                                    streams.default = new window.MediaStream();
                                }
                                streams.default.addTrack(track);
                                receiverList.push([track, rtpReceiver, streams.default]);
                            }
                        } else {
                            // FIXME: actually the receiver should be created later.
                            delete transceiver.rtpReceiver;
                        }
                    }
                });

                if (this._dtlsRole === undefined) {
                    this._dtlsRole = description.type === 'offer' ? 'active' : 'passive';
                }

                this.remoteDescription = {
                    type: description.type,
                    sdp: description.sdp
                };
                switch (description.type) {
                    case 'offer':
                        this._updateSignalingState('have-remote-offer');
                        break;
                    case 'answer':
                        this._updateSignalingState('stable');
                        break;
                    default:
                        throw new TypeError('unsupported type "' + description.type +
                            '"');
                }
                Object.keys(streams).forEach(function(sid) {
                    var stream = streams[sid];
                    if (stream.getTracks().length) {
                        if (self.remoteStreams.indexOf(stream) === -1) {
                            self.remoteStreams.push(stream);
                            var event = new Event('addstream');
                            event.stream = stream;
                            window.setTimeout(function() {
                                self.dispatchEvent(event);
                                if (typeof self.onaddstream === 'function') {
                                    self.onaddstream(event);
                                }
                            });
                        }

                        receiverList.forEach(function(item) {
                            var track = item[0];
                            var receiver = item[1];
                            if (stream.id !== item[2].id) {
                                return;
                            }
                            var trackEvent = new Event('track');
                            trackEvent.track = track;
                            trackEvent.receiver = receiver;
                            trackEvent.transceiver = {receiver: receiver};
                            trackEvent.streams = [stream];
                            window.setTimeout(function() {
                                self.dispatchEvent(trackEvent);
                                if (typeof self.ontrack === 'function') {
                                    self.ontrack(trackEvent);
                                }
                            });
                        });
                    }
                });

                // check whether addIceCandidate({}) was called within four seconds after
                // setRemoteDescription.
                window.setTimeout(function() {
                    if (!(self && self.transceivers)) {
                        return;
                    }
                    self.transceivers.forEach(function(transceiver) {
                        if (transceiver.iceTransport &&
                            transceiver.iceTransport.state === 'new' &&
                            transceiver.iceTransport.getRemoteCandidates().length > 0) {
                            console.warn('Timeout for addRemoteCandidate. Consider sending ' +
                                'an end-of-candidates notification');
                            transceiver.iceTransport.addRemoteCandidate({});
                        }
                    });
                }, 4000);

                return new Promise(function(resolve) {
                    if (args.length > 1 && typeof args[1] === 'function') {
                        args[1].apply(null);
                    }
                    resolve();
                });
            };

            RTCPeerConnection.prototype.close = function() {
                this.transceivers.forEach(function(transceiver) {
                    /* not yet
                     if (transceiver.iceGatherer) {
                     transceiver.iceGatherer.close();
                     }
                     */
                    if (transceiver.iceTransport) {
                        transceiver.iceTransport.stop();
                    }
                    if (transceiver.dtlsTransport) {
                        transceiver.dtlsTransport.stop();
                    }
                    if (transceiver.rtpSender) {
                        transceiver.rtpSender.stop();
                    }
                    if (transceiver.rtpReceiver) {
                        transceiver.rtpReceiver.stop();
                    }
                });
                // FIXME: clean up tracks, local streams, remote streams, etc
                this._updateSignalingState('closed');
            };

            // Update the signaling state.
            RTCPeerConnection.prototype._updateSignalingState = function(newState) {
                this.signalingState = newState;
                var event = new Event('signalingstatechange');
                this.dispatchEvent(event);
                if (typeof this.onsignalingstatechange === 'function') {
                    this.onsignalingstatechange(event);
                }
            };

            // Determine whether to fire the negotiationneeded event.
            RTCPeerConnection.prototype._maybeFireNegotiationNeeded = function() {
                var self = this;
                if (this.signalingState !== 'stable' || this.needNegotiation === true) {
                    return;
                }
                this.needNegotiation = true;
                window.setTimeout(function() {
                    if (self.needNegotiation === false) {
                        return;
                    }
                    self.needNegotiation = false;
                    var event = new Event('negotiationneeded');
                    self.dispatchEvent(event);
                    if (typeof self.onnegotiationneeded === 'function') {
                        self.onnegotiationneeded(event);
                    }
                }, 0);
            };

            // Update the connection state.
            RTCPeerConnection.prototype._updateConnectionState = function() {
                var newState;
                var states = {
                    'new': 0,
                    closed: 0,
                    connecting: 0,
                    checking: 0,
                    connected: 0,
                    completed: 0,
                    disconnected: 0,
                    failed: 0
                };
                this.transceivers.forEach(function(transceiver) {
                    states[transceiver.iceTransport.state]++;
                    states[transceiver.dtlsTransport.state]++;
                });
                // ICETransport.completed and connected are the same for this purpose.
                states.connected += states.completed;

                newState = 'new';
                if (states.failed > 0) {
                    newState = 'failed';
                } else if (states.connecting > 0 || states.checking > 0) {
                    newState = 'connecting';
                } else if (states.disconnected > 0) {
                    newState = 'disconnected';
                } else if (states.new > 0) {
                    newState = 'new';
                } else if (states.connected > 0 || states.completed > 0) {
                    newState = 'connected';
                }

                if (newState !== this.iceConnectionState) {
                    this.iceConnectionState = newState;
                    var event = new Event('iceconnectionstatechange');
                    this.dispatchEvent(event);
                    if (typeof this.oniceconnectionstatechange === 'function') {
                        this.oniceconnectionstatechange(event);
                    }
                }
            };

            RTCPeerConnection.prototype.createOffer = function() {
                var self = this;
                var args = arguments;

                var offerOptions;
                if (arguments.length === 1 && typeof arguments[0] !== 'function') {
                    offerOptions = arguments[0];
                } else if (arguments.length === 3) {
                    offerOptions = arguments[2];
                }

                var numAudioTracks = this.transceivers.filter(function(t) {
                    return t.kind === 'audio';
                }).length;
                var numVideoTracks = this.transceivers.filter(function(t) {
                    return t.kind === 'video';
                }).length;

                // Determine number of audio and video tracks we need to send/recv.
                if (offerOptions) {
                    // Reject Chrome legacy constraints.
                    if (offerOptions.mandatory || offerOptions.optional) {
                        throw new TypeError(
                            'Legacy mandatory/optional constraints not supported.');
                    }
                    if (offerOptions.offerToReceiveAudio !== undefined) {
                        if (offerOptions.offerToReceiveAudio === true) {
                            numAudioTracks = 1;
                        } else if (offerOptions.offerToReceiveAudio === false) {
                            numAudioTracks = 0;
                        } else {
                            numAudioTracks = offerOptions.offerToReceiveAudio;
                        }
                    }
                    if (offerOptions.offerToReceiveVideo !== undefined) {
                        if (offerOptions.offerToReceiveVideo === true) {
                            numVideoTracks = 1;
                        } else if (offerOptions.offerToReceiveVideo === false) {
                            numVideoTracks = 0;
                        } else {
                            numVideoTracks = offerOptions.offerToReceiveVideo;
                        }
                    }
                }

                this.transceivers.forEach(function(transceiver) {
                    if (transceiver.kind === 'audio') {
                        numAudioTracks--;
                        if (numAudioTracks < 0) {
                            transceiver.wantReceive = false;
                        }
                    } else if (transceiver.kind === 'video') {
                        numVideoTracks--;
                        if (numVideoTracks < 0) {
                            transceiver.wantReceive = false;
                        }
                    }
                });

                // Create M-lines for recvonly streams.
                while (numAudioTracks > 0 || numVideoTracks > 0) {
                    if (numAudioTracks > 0) {
                        this._createTransceiver('audio');
                        numAudioTracks--;
                    }
                    if (numVideoTracks > 0) {
                        this._createTransceiver('video');
                        numVideoTracks--;
                    }
                }

                var sdp = SDPUtils.writeSessionBoilerplate(this._sdpSessionId,
                    this._sdpSessionVersion++);
                this.transceivers.forEach(function(transceiver, sdpMLineIndex) {
                    // For each track, create an ice gatherer, ice transport,
                    // dtls transport, potentially rtpsender and rtpreceiver.
                    var track = transceiver.track;
                    var kind = transceiver.kind;
                    var mid = SDPUtils.generateIdentifier();
                    transceiver.mid = mid;

                    if (!transceiver.iceGatherer) {
                        transceiver.iceGatherer = self._createIceGatherer(sdpMLineIndex,
                            self.usingBundle);
                    }

                    var localCapabilities = window.RTCRtpSender.getCapabilities(kind);
                    // filter RTX until additional stuff needed for RTX is implemented
                    // in adapter.js
                    if (edgeVersion < 15019) {
                        localCapabilities.codecs = localCapabilities.codecs.filter(
                            function(codec) {
                                return codec.name !== 'rtx';
                            });
                    }
                    localCapabilities.codecs.forEach(function(codec) {
                        // work around https://bugs.chromium.org/p/webrtc/issues/detail?id=6552
                        // by adding level-asymmetry-allowed=1
                        if (codec.name === 'H264' &&
                            codec.parameters['level-asymmetry-allowed'] === undefined) {
                            codec.parameters['level-asymmetry-allowed'] = '1';
                        }
                    });

                    // generate an ssrc now, to be used later in rtpSender.send
                    var sendEncodingParameters = transceiver.sendEncodingParameters || [{
                            ssrc: (2 * sdpMLineIndex + 1) * 1001
                        }];
                    if (track) {
                        // add RTX
                        if (edgeVersion >= 15019 && kind === 'video' &&
                            !sendEncodingParameters[0].rtx) {
                            sendEncodingParameters[0].rtx = {
                                ssrc: sendEncodingParameters[0].ssrc + 1
                            };
                        }
                    }

                    if (transceiver.wantReceive) {
                        transceiver.rtpReceiver = new window.RTCRtpReceiver(
                            transceiver.dtlsTransport, kind);
                    }

                    transceiver.localCapabilities = localCapabilities;
                    transceiver.sendEncodingParameters = sendEncodingParameters;
                });

                // always offer BUNDLE and dispose on return if not supported.
                if (this._config.bundlePolicy !== 'max-compat') {
                    sdp += 'a=group:BUNDLE ' + this.transceivers.map(function(t) {
                            return t.mid;
                        }).join(' ') + '\r\n';
                }
                sdp += 'a=ice-options:trickle\r\n';

                this.transceivers.forEach(function(transceiver, sdpMLineIndex) {
                    sdp += writeMediaSection(transceiver, transceiver.localCapabilities,
                        'offer', transceiver.stream, self._dtlsRole);
                    sdp += 'a=rtcp-rsize\r\n';

                    if (transceiver.iceGatherer && self.iceGatheringState !== 'new' &&
                        (sdpMLineIndex === 0 || !self.usingBundle)) {
                        transceiver.iceGatherer.getLocalCandidates().forEach(function(cand) {
                            cand.component = 1;
                            sdp += 'a=' + SDPUtils.writeCandidate(cand) + '\r\n';
                        });

                        if (transceiver.iceGatherer.state === 'completed') {
                            sdp += 'a=end-of-candidates\r\n';
                        }
                    }
                });

                var desc = new window.RTCSessionDescription({
                    type: 'offer',
                    sdp: sdp
                });
                return new Promise(function(resolve) {
                    if (args.length > 0 && typeof args[0] === 'function') {
                        args[0].apply(null, [desc]);
                        resolve();
                        return;
                    }
                    resolve(desc);
                });
            };

            RTCPeerConnection.prototype.createAnswer = function() {
                var self = this;
                var args = arguments;

                var sdp = SDPUtils.writeSessionBoilerplate(this._sdpSessionId,
                    this._sdpSessionVersion++);
                if (this.usingBundle) {
                    sdp += 'a=group:BUNDLE ' + this.transceivers.map(function(t) {
                            return t.mid;
                        }).join(' ') + '\r\n';
                }
                var mediaSectionsInOffer = SDPUtils.splitSections(
                        this.remoteDescription.sdp).length - 1;
                this.transceivers.forEach(function(transceiver, sdpMLineIndex) {
                    if (sdpMLineIndex + 1 > mediaSectionsInOffer) {
                        return;
                    }
                    if (transceiver.isDatachannel) {
                        sdp += 'm=application 0 DTLS/SCTP 5000\r\n' +
                            'c=IN IP4 0.0.0.0\r\n' +
                            'a=mid:' + transceiver.mid + '\r\n';
                        return;
                    }

                    // FIXME: look at direction.
                    if (transceiver.stream) {
                        var localTrack;
                        if (transceiver.kind === 'audio') {
                            localTrack = transceiver.stream.getAudioTracks()[0];
                        } else if (transceiver.kind === 'video') {
                            localTrack = transceiver.stream.getVideoTracks()[0];
                        }
                        if (localTrack) {
                            // add RTX
                            if (edgeVersion >= 15019 && transceiver.kind === 'video' &&
                                !transceiver.sendEncodingParameters[0].rtx) {
                                transceiver.sendEncodingParameters[0].rtx = {
                                    ssrc: transceiver.sendEncodingParameters[0].ssrc + 1
                                };
                            }
                        }
                    }

                    // Calculate intersection of capabilities.
                    var commonCapabilities = getCommonCapabilities(
                        transceiver.localCapabilities,
                        transceiver.remoteCapabilities);

                    var hasRtx = commonCapabilities.codecs.filter(function(c) {
                        return c.name.toLowerCase() === 'rtx';
                    }).length;
                    if (!hasRtx && transceiver.sendEncodingParameters[0].rtx) {
                        delete transceiver.sendEncodingParameters[0].rtx;
                    }

                    sdp += writeMediaSection(transceiver, commonCapabilities,
                        'answer', transceiver.stream, self._dtlsRole);
                    if (transceiver.rtcpParameters &&
                        transceiver.rtcpParameters.reducedSize) {
                        sdp += 'a=rtcp-rsize\r\n';
                    }
                });

                var desc = new window.RTCSessionDescription({
                    type: 'answer',
                    sdp: sdp
                });
                return new Promise(function(resolve) {
                    if (args.length > 0 && typeof args[0] === 'function') {
                        args[0].apply(null, [desc]);
                        resolve();
                        return;
                    }
                    resolve(desc);
                });
            };

            RTCPeerConnection.prototype.addIceCandidate = function(candidate) {
                var err;
                var sections;
                if (!candidate || candidate.candidate === '') {
                    for (var j = 0; j < this.transceivers.length; j++) {
                        if (this.transceivers[j].isDatachannel) {
                            continue;
                        }
                        this.transceivers[j].iceTransport.addRemoteCandidate({});
                        sections = SDPUtils.splitSections(this.remoteDescription.sdp);
                        sections[j + 1] += 'a=end-of-candidates\r\n';
                        this.remoteDescription.sdp = sections.join('');
                        if (this.usingBundle) {
                            break;
                        }
                    }
                } else if (!(candidate.sdpMLineIndex !== undefined || candidate.sdpMid)) {
                    throw new TypeError('sdpMLineIndex or sdpMid required');
                } else if (!this.remoteDescription) {
                    err = new Error('Can not add ICE candidate without ' +
                        'a remote description');
                    err.name = 'InvalidStateError';
                } else {
                    var sdpMLineIndex = candidate.sdpMLineIndex;
                    if (candidate.sdpMid) {
                        for (var i = 0; i < this.transceivers.length; i++) {
                            if (this.transceivers[i].mid === candidate.sdpMid) {
                                sdpMLineIndex = i;
                                break;
                            }
                        }
                    }
                    var transceiver = this.transceivers[sdpMLineIndex];
                    if (transceiver) {
                        if (transceiver.isDatachannel) {
                            return Promise.resolve();
                        }
                        var cand = Object.keys(candidate.candidate).length > 0 ?
                            SDPUtils.parseCandidate(candidate.candidate) : {};
                        // Ignore Chrome's invalid candidates since Edge does not like them.
                        if (cand.protocol === 'tcp' && (cand.port === 0 || cand.port === 9)) {
                            return Promise.resolve();
                        }
                        // Ignore RTCP candidates, we assume RTCP-MUX.
                        if (cand.component && cand.component !== 1) {
                            return Promise.resolve();
                        }
                        // when using bundle, avoid adding candidates to the wrong
                        // ice transport. And avoid adding candidates added in the SDP.
                        if (sdpMLineIndex === 0 || (sdpMLineIndex > 0 &&
                            transceiver.iceTransport !== this.transceivers[0].iceTransport)) {
                            if (!maybeAddCandidate(transceiver.iceTransport, cand)) {
                                err = new Error('Can not add ICE candidate');
                                err.name = 'OperationError';
                            }
                        }

                        if (!err) {
                            // update the remoteDescription.
                            var candidateString = candidate.candidate.trim();
                            if (candidateString.indexOf('a=') === 0) {
                                candidateString = candidateString.substr(2);
                            }
                            sections = SDPUtils.splitSections(this.remoteDescription.sdp);
                            sections[sdpMLineIndex + 1] += 'a=' +
                                (cand.type ? candidateString : 'end-of-candidates')
                                + '\r\n';
                            this.remoteDescription.sdp = sections.join('');
                        }
                    } else {
                        err = new Error('Can not add ICE candidate');
                        err.name = 'OperationError';
                    }
                }
                var args = arguments;
                return new Promise(function(resolve, reject) {
                    if (err) {
                        if (args.length > 2 && typeof args[2] === 'function') {
                            args[2].apply(null, [err]);
                        }
                        reject(err);
                    } else {
                        if (args.length > 1 && typeof args[1] === 'function') {
                            args[1].apply(null);
                        }
                        resolve();
                    }
                });
            };

            RTCPeerConnection.prototype.getStats = function() {
                var promises = [];
                this.transceivers.forEach(function(transceiver) {
                    ['rtpSender', 'rtpReceiver', 'iceGatherer', 'iceTransport',
                        'dtlsTransport'].forEach(function(method) {
                        if (transceiver[method]) {
                            promises.push(transceiver[method].getStats());
                        }
                    });
                });
                var cb = arguments.length > 1 && typeof arguments[1] === 'function' &&
                    arguments[1];
                var fixStatsType = function(stat) {
                    return {
                            inboundrtp: 'inbound-rtp',
                            outboundrtp: 'outbound-rtp',
                            candidatepair: 'candidate-pair',
                            localcandidate: 'local-candidate',
                            remotecandidate: 'remote-candidate'
                        }[stat.type] || stat.type;
                };
                return new Promise(function(resolve) {
                    // shim getStats with maplike support
                    var results = new Map();
                    Promise.all(promises).then(function(res) {
                        res.forEach(function(result) {
                            Object.keys(result).forEach(function(id) {
                                result[id].type = fixStatsType(result[id]);
                                results.set(id, result[id]);
                            });
                        });
                        if (cb) {
                            cb.apply(null, results);
                        }
                        resolve(results);
                    });
                });
            };
            return RTCPeerConnection;
        };

    },{"sdp":2}],2:[function(requirecopy,module,exports){
        /* eslint-env node */
        'use strict';

// SDP helpers.
        var SDPUtils = {};

// Generate an alphanumeric identifier for cname or mids.
// TODO: use UUIDs instead? https://gist.github.com/jed/982883
        SDPUtils.generateIdentifier = function() {
            return Math.random().toString(36).substr(2, 10);
        };

// The RTCP CNAME used by all peerconnections from the same JS.
        SDPUtils.localCName = SDPUtils.generateIdentifier();

// Splits SDP into lines, dealing with both CRLF and LF.
        SDPUtils.splitLines = function(blob) {
            return blob.trim().split('\n').map(function(line) {
                return line.trim();
            });
        };
// Splits SDP into sessionpart and mediasections. Ensures CRLF.
        SDPUtils.splitSections = function(blob) {
            var parts = blob.split('\nm=');
            return parts.map(function(part, index) {
                return (index > 0 ? 'm=' + part : part).trim() + '\r\n';
            });
        };

// Returns lines that start with a certain prefix.
        SDPUtils.matchPrefix = function(blob, prefix) {
            return SDPUtils.splitLines(blob).filter(function(line) {
                return line.indexOf(prefix) === 0;
            });
        };

// Parses an ICE candidate line. Sample input:
// candidate:702786350 2 udp 41819902 8.8.8.8 60769 typ relay raddr 8.8.8.8
// rport 55996"
        SDPUtils.parseCandidate = function(line) {
            var parts;
            // Parse both variants.
            if (line.indexOf('a=candidate:') === 0) {
                parts = line.substring(12).split(' ');
            } else {
                parts = line.substring(10).split(' ');
            }

            var candidate = {
                foundation: parts[0],
                component: parseInt(parts[1], 10),
                protocol: parts[2].toLowerCase(),
                priority: parseInt(parts[3], 10),
                ip: parts[4],
                port: parseInt(parts[5], 10),
                // skip parts[6] == 'typ'
                type: parts[7]
            };

            for (var i = 8; i < parts.length; i += 2) {
                switch (parts[i]) {
                    case 'raddr':
                        candidate.relatedAddress = parts[i + 1];
                        break;
                    case 'rport':
                        candidate.relatedPort = parseInt(parts[i + 1], 10);
                        break;
                    case 'tcptype':
                        candidate.tcpType = parts[i + 1];
                        break;
                    case 'ufrag':
                        candidate.ufrag = parts[i + 1]; // for backward compability.
                        candidate.usernameFragment = parts[i + 1];
                        break;
                    default: // extension handling, in particular ufrag
                        candidate[parts[i]] = parts[i + 1];
                        break;
                }
            }
            return candidate;
        };

// Translates a candidate object into SDP candidate attribute.
        SDPUtils.writeCandidate = function(candidate) {
            var sdp = [];
            sdp.push(candidate.foundation);
            sdp.push(candidate.component);
            sdp.push(candidate.protocol.toUpperCase());
            sdp.push(candidate.priority);
            sdp.push(candidate.ip);
            sdp.push(candidate.port);

            var type = candidate.type;
            sdp.push('typ');
            sdp.push(type);
            if (type !== 'host' && candidate.relatedAddress &&
                candidate.relatedPort) {
                sdp.push('raddr');
                sdp.push(candidate.relatedAddress); // was: relAddr
                sdp.push('rport');
                sdp.push(candidate.relatedPort); // was: relPort
            }
            if (candidate.tcpType && candidate.protocol.toLowerCase() === 'tcp') {
                sdp.push('tcptype');
                sdp.push(candidate.tcpType);
            }
            if (candidate.ufrag) {
                sdp.push('ufrag');
                sdp.push(candidate.ufrag);
            }
            return 'candidate:' + sdp.join(' ');
        };

// Parses an ice-options line, returns an array of option tags.
// a=ice-options:foo bar
        SDPUtils.parseIceOptions = function(line) {
            return line.substr(14).split(' ');
        }

// Parses an rtpmap line, returns RTCRtpCoddecParameters. Sample input:
// a=rtpmap:111 opus/48000/2
        SDPUtils.parseRtpMap = function(line) {
            var parts = line.substr(9).split(' ');
            var parsed = {
                payloadType: parseInt(parts.shift(), 10) // was: id
            };

            parts = parts[0].split('/');

            parsed.name = parts[0];
            parsed.clockRate = parseInt(parts[1], 10); // was: clockrate
            // was: channels
            parsed.numChannels = parts.length === 3 ? parseInt(parts[2], 10) : 1;
            return parsed;
        };

// Generate an a=rtpmap line from RTCRtpCodecCapability or
// RTCRtpCodecParameters.
        SDPUtils.writeRtpMap = function(codec) {
            var pt = codec.payloadType;
            if (codec.preferredPayloadType !== undefined) {
                pt = codec.preferredPayloadType;
            }
            return 'a=rtpmap:' + pt + ' ' + codec.name + '/' + codec.clockRate +
                (codec.numChannels !== 1 ? '/' + codec.numChannels : '') + '\r\n';
        };

// Parses an a=extmap line (headerextension from RFC 5285). Sample input:
// a=extmap:2 urn:ietf:params:rtp-hdrext:toffset
// a=extmap:2/sendonly urn:ietf:params:rtp-hdrext:toffset
        SDPUtils.parseExtmap = function(line) {
            var parts = line.substr(9).split(' ');
            return {
                id: parseInt(parts[0], 10),
                direction: parts[0].indexOf('/') > 0 ? parts[0].split('/')[1] : 'sendrecv',
                uri: parts[1]
            };
        };

// Generates a=extmap line from RTCRtpHeaderExtensionParameters or
// RTCRtpHeaderExtension.
        SDPUtils.writeExtmap = function(headerExtension) {
            return 'a=extmap:' + (headerExtension.id || headerExtension.preferredId) +
                (headerExtension.direction && headerExtension.direction !== 'sendrecv'
                    ? '/' + headerExtension.direction
                    : '') +
                ' ' + headerExtension.uri + '\r\n';
        };

// Parses an ftmp line, returns dictionary. Sample input:
// a=fmtp:96 vbr=on;cng=on
// Also deals with vbr=on; cng=on
        SDPUtils.parseFmtp = function(line) {
            var parsed = {};
            var kv;
            var parts = line.substr(line.indexOf(' ') + 1).split(';');
            for (var j = 0; j < parts.length; j++) {
                kv = parts[j].trim().split('=');
                parsed[kv[0].trim()] = kv[1];
            }
            return parsed;
        };

// Generates an a=ftmp line from RTCRtpCodecCapability or RTCRtpCodecParameters.
        SDPUtils.writeFmtp = function(codec) {
            var line = '';
            var pt = codec.payloadType;
            if (codec.preferredPayloadType !== undefined) {
                pt = codec.preferredPayloadType;
            }
            if (codec.parameters && Object.keys(codec.parameters).length) {
                var params = [];
                Object.keys(codec.parameters).forEach(function(param) {
                    params.push(param + '=' + codec.parameters[param]);
                });
                line += 'a=fmtp:' + pt + ' ' + params.join(';') + '\r\n';
            }
            return line;
        };

// Parses an rtcp-fb line, returns RTCPRtcpFeedback object. Sample input:
// a=rtcp-fb:98 nack rpsi
        SDPUtils.parseRtcpFb = function(line) {
            var parts = line.substr(line.indexOf(' ') + 1).split(' ');
            return {
                type: parts.shift(),
                parameter: parts.join(' ')
            };
        };
// Generate a=rtcp-fb lines from RTCRtpCodecCapability or RTCRtpCodecParameters.
        SDPUtils.writeRtcpFb = function(codec) {
            var lines = '';
            var pt = codec.payloadType;
            if (codec.preferredPayloadType !== undefined) {
                pt = codec.preferredPayloadType;
            }
            if (codec.rtcpFeedback && codec.rtcpFeedback.length) {
                // FIXME: special handling for trr-int?
                codec.rtcpFeedback.forEach(function(fb) {
                    lines += 'a=rtcp-fb:' + pt + ' ' + fb.type +
                        (fb.parameter && fb.parameter.length ? ' ' + fb.parameter : '') +
                        '\r\n';
                });
            }
            return lines;
        };

// Parses an RFC 5576 ssrc media attribute. Sample input:
// a=ssrc:3735928559 cname:something
        SDPUtils.parseSsrcMedia = function(line) {
            var sp = line.indexOf(' ');
            var parts = {
                ssrc: parseInt(line.substr(7, sp - 7), 10)
            };
            var colon = line.indexOf(':', sp);
            if (colon > -1) {
                parts.attribute = line.substr(sp + 1, colon - sp - 1);
                parts.value = line.substr(colon + 1);
            } else {
                parts.attribute = line.substr(sp + 1);
            }
            return parts;
        };

// Extracts the MID (RFC 5888) from a media section.
// returns the MID or undefined if no mid line was found.
        SDPUtils.getMid = function(mediaSection) {
            var mid = SDPUtils.matchPrefix(mediaSection, 'a=mid:')[0];
            if (mid) {
                return mid.substr(6);
            }
        }

        SDPUtils.parseFingerprint = function(line) {
            var parts = line.substr(14).split(' ');
            return {
                algorithm: parts[0].toLowerCase(), // algorithm is case-sensitive in Edge.
                value: parts[1]
            };
        };

// Extracts DTLS parameters from SDP media section or sessionpart.
// FIXME: for consistency with other functions this should only
//   get the fingerprint line as input. See also getIceParameters.
        SDPUtils.getDtlsParameters = function(mediaSection, sessionpart) {
            var lines = SDPUtils.matchPrefix(mediaSection + sessionpart,
                'a=fingerprint:');
            // Note: a=setup line is ignored since we use the 'auto' role.
            // Note2: 'algorithm' is not case sensitive except in Edge.
            return {
                role: 'auto',
                fingerprints: lines.map(SDPUtils.parseFingerprint)
            };
        };

// Serializes DTLS parameters to SDP.
        SDPUtils.writeDtlsParameters = function(params, setupType) {
            var sdp = 'a=setup:' + setupType + '\r\n';
            params.fingerprints.forEach(function(fp) {
                sdp += 'a=fingerprint:' + fp.algorithm + ' ' + fp.value + '\r\n';
            });
            return sdp;
        };
// Parses ICE information from SDP media section or sessionpart.
// FIXME: for consistency with other functions this should only
//   get the ice-ufrag and ice-pwd lines as input.
        SDPUtils.getIceParameters = function(mediaSection, sessionpart) {
            var lines = SDPUtils.splitLines(mediaSection);
            // Search in session part, too.
            lines = lines.concat(SDPUtils.splitLines(sessionpart));
            var iceParameters = {
                usernameFragment: lines.filter(function(line) {
                    return line.indexOf('a=ice-ufrag:') === 0;
                })[0].substr(12),
                password: lines.filter(function(line) {
                    return line.indexOf('a=ice-pwd:') === 0;
                })[0].substr(10)
            };
            return iceParameters;
        };

// Serializes ICE parameters to SDP.
        SDPUtils.writeIceParameters = function(params) {
            return 'a=ice-ufrag:' + params.usernameFragment + '\r\n' +
                'a=ice-pwd:' + params.password + '\r\n';
        };

// Parses the SDP media section and returns RTCRtpParameters.
        SDPUtils.parseRtpParameters = function(mediaSection) {
            var description = {
                codecs: [],
                headerExtensions: [],
                fecMechanisms: [],
                rtcp: []
            };
            var lines = SDPUtils.splitLines(mediaSection);
            var mline = lines[0].split(' ');
            for (var i = 3; i < mline.length; i++) { // find all codecs from mline[3..]
                var pt = mline[i];
                var rtpmapline = SDPUtils.matchPrefix(
                    mediaSection, 'a=rtpmap:' + pt + ' ')[0];
                if (rtpmapline) {
                    var codec = SDPUtils.parseRtpMap(rtpmapline);
                    var fmtps = SDPUtils.matchPrefix(
                        mediaSection, 'a=fmtp:' + pt + ' ');
                    // Only the first a=fmtp:<pt> is considered.
                    codec.parameters = fmtps.length ? SDPUtils.parseFmtp(fmtps[0]) : {};
                    codec.rtcpFeedback = SDPUtils.matchPrefix(
                        mediaSection, 'a=rtcp-fb:' + pt + ' ')
                        .map(SDPUtils.parseRtcpFb);
                    description.codecs.push(codec);
                    // parse FEC mechanisms from rtpmap lines.
                    switch (codec.name.toUpperCase()) {
                        case 'RED':
                        case 'ULPFEC':
                            description.fecMechanisms.push(codec.name.toUpperCase());
                            break;
                        default: // only RED and ULPFEC are recognized as FEC mechanisms.
                            break;
                    }
                }
            }
            SDPUtils.matchPrefix(mediaSection, 'a=extmap:').forEach(function(line) {
                description.headerExtensions.push(SDPUtils.parseExtmap(line));
            });
            // FIXME: parse rtcp.
            return description;
        };

// Generates parts of the SDP media section describing the capabilities /
// parameters.
        SDPUtils.writeRtpDescription = function(kind, caps) {
            var sdp = '';

            // Build the mline.
            sdp += 'm=' + kind + ' ';
            sdp += caps.codecs.length > 0 ? '9' : '0'; // reject if no codecs.
            sdp += ' UDP/TLS/RTP/SAVPF ';
            sdp += caps.codecs.map(function(codec) {
                    if (codec.preferredPayloadType !== undefined) {
                        return codec.preferredPayloadType;
                    }
                    return codec.payloadType;
                }).join(' ') + '\r\n';

            sdp += 'c=IN IP4 0.0.0.0\r\n';
            sdp += 'a=rtcp:9 IN IP4 0.0.0.0\r\n';

            // Add a=rtpmap lines for each codec. Also fmtp and rtcp-fb.
            caps.codecs.forEach(function(codec) {
                sdp += SDPUtils.writeRtpMap(codec);
                sdp += SDPUtils.writeFmtp(codec);
                sdp += SDPUtils.writeRtcpFb(codec);
            });
            var maxptime = 0;
            caps.codecs.forEach(function(codec) {
                if (codec.maxptime > maxptime) {
                    maxptime = codec.maxptime;
                }
            });
            if (maxptime > 0) {
                sdp += 'a=maxptime:' + maxptime + '\r\n';
            }
            sdp += 'a=rtcp-mux\r\n';

            caps.headerExtensions.forEach(function(extension) {
                sdp += SDPUtils.writeExtmap(extension);
            });
            // FIXME: write fecMechanisms.
            return sdp;
        };

// Parses the SDP media section and returns an array of
// RTCRtpEncodingParameters.
        SDPUtils.parseRtpEncodingParameters = function(mediaSection) {
            var encodingParameters = [];
            var description = SDPUtils.parseRtpParameters(mediaSection);
            var hasRed = description.fecMechanisms.indexOf('RED') !== -1;
            var hasUlpfec = description.fecMechanisms.indexOf('ULPFEC') !== -1;

            // filter a=ssrc:... cname:, ignore PlanB-msid
            var ssrcs = SDPUtils.matchPrefix(mediaSection, 'a=ssrc:')
                .map(function(line) {
                    return SDPUtils.parseSsrcMedia(line);
                })
                .filter(function(parts) {
                    return parts.attribute === 'cname';
                });
            var primarySsrc = ssrcs.length > 0 && ssrcs[0].ssrc;
            var secondarySsrc;

            var flows = SDPUtils.matchPrefix(mediaSection, 'a=ssrc-group:FID')
                .map(function(line) {
                    var parts = line.split(' ');
                    parts.shift();
                    return parts.map(function(part) {
                        return parseInt(part, 10);
                    });
                });
            if (flows.length > 0 && flows[0].length > 1 && flows[0][0] === primarySsrc) {
                secondarySsrc = flows[0][1];
            }

            description.codecs.forEach(function(codec) {
                if (codec.name.toUpperCase() === 'RTX' && codec.parameters.apt) {
                    var encParam = {
                        ssrc: primarySsrc,
                        codecPayloadType: parseInt(codec.parameters.apt, 10),
                        rtx: {
                            ssrc: secondarySsrc
                        }
                    };
                    encodingParameters.push(encParam);
                    if (hasRed) {
                        encParam = JSON.parse(JSON.stringify(encParam));
                        encParam.fec = {
                            ssrc: secondarySsrc,
                            mechanism: hasUlpfec ? 'red+ulpfec' : 'red'
                        };
                        encodingParameters.push(encParam);
                    }
                }
            });
            if (encodingParameters.length === 0 && primarySsrc) {
                encodingParameters.push({
                    ssrc: primarySsrc
                });
            }

            // we support both b=AS and b=TIAS but interpret AS as TIAS.
            var bandwidth = SDPUtils.matchPrefix(mediaSection, 'b=');
            if (bandwidth.length) {
                if (bandwidth[0].indexOf('b=TIAS:') === 0) {
                    bandwidth = parseInt(bandwidth[0].substr(7), 10);
                } else if (bandwidth[0].indexOf('b=AS:') === 0) {
                    // use formula from JSEP to convert b=AS to TIAS value.
                    bandwidth = parseInt(bandwidth[0].substr(5), 10) * 1000 * 0.95
                        - (50 * 40 * 8);
                } else {
                    bandwidth = undefined;
                }
                encodingParameters.forEach(function(params) {
                    params.maxBitrate = bandwidth;
                });
            }
            return encodingParameters;
        };

// parses http://draft.ortc.org/#rtcrtcpparameters*
        SDPUtils.parseRtcpParameters = function(mediaSection) {
            var rtcpParameters = {};

            var cname;
            // Gets the first SSRC. Note that with RTX there might be multiple
            // SSRCs.
            var remoteSsrc = SDPUtils.matchPrefix(mediaSection, 'a=ssrc:')
                .map(function(line) {
                    return SDPUtils.parseSsrcMedia(line);
                })
                .filter(function(obj) {
                    return obj.attribute === 'cname';
                })[0];
            if (remoteSsrc) {
                rtcpParameters.cname = remoteSsrc.value;
                rtcpParameters.ssrc = remoteSsrc.ssrc;
            }

            // Edge uses the compound attribute instead of reducedSize
            // compound is !reducedSize
            var rsize = SDPUtils.matchPrefix(mediaSection, 'a=rtcp-rsize');
            rtcpParameters.reducedSize = rsize.length > 0;
            rtcpParameters.compound = rsize.length === 0;

            // parses the rtcp-mux attrіbute.
            // Note that Edge does not support unmuxed RTCP.
            var mux = SDPUtils.matchPrefix(mediaSection, 'a=rtcp-mux');
            rtcpParameters.mux = mux.length > 0;

            return rtcpParameters;
        };

// parses either a=msid: or a=ssrc:... msid lines and returns
// the id of the MediaStream and MediaStreamTrack.
        SDPUtils.parseMsid = function(mediaSection) {
            var parts;
            var spec = SDPUtils.matchPrefix(mediaSection, 'a=msid:');
            if (spec.length === 1) {
                parts = spec[0].substr(7).split(' ');
                return {stream: parts[0], track: parts[1]};
            }
            var planB = SDPUtils.matchPrefix(mediaSection, 'a=ssrc:')
                .map(function(line) {
                    return SDPUtils.parseSsrcMedia(line);
                })
                .filter(function(parts) {
                    return parts.attribute === 'msid';
                });
            if (planB.length > 0) {
                parts = planB[0].value.split(' ');
                return {stream: parts[0], track: parts[1]};
            }
        };

// Generate a session ID for SDP.
// https://tools.ietf.org/html/draft-ietf-rtcweb-jsep-20#section-5.2.1
// recommends using a cryptographically random +ve 64-bit value
// but right now this should be acceptable and within the right range
        SDPUtils.generateSessionId = function() {
            return Math.random().toString().substr(2, 21);
        };

// Write boilder plate for start of SDP
// sessId argument is optional - if not supplied it will
// be generated randomly
// sessVersion is optional and defaults to 2
        SDPUtils.writeSessionBoilerplate = function(sessId, sessVer) {
            var sessionId;
            var version = sessVer !== undefined ? sessVer : 2;
            if (sessId) {
                sessionId = sessId;
            } else {
                sessionId = SDPUtils.generateSessionId();
            }
            // FIXME: sess-id should be an NTP timestamp.
            return 'v=0\r\n' +
                'o=thisisadapterortc ' + sessionId + ' ' + version + ' IN IP4 127.0.0.1\r\n' +
                's=-\r\n' +
                't=0 0\r\n';
        };

        SDPUtils.writeMediaSection = function(transceiver, caps, type, stream) {
            var sdp = SDPUtils.writeRtpDescription(transceiver.kind, caps);

            // Map ICE parameters (ufrag, pwd) to SDP.
            sdp += SDPUtils.writeIceParameters(
                transceiver.iceGatherer.getLocalParameters());

            // Map DTLS parameters to SDP.
            sdp += SDPUtils.writeDtlsParameters(
                transceiver.dtlsTransport.getLocalParameters(),
                type === 'offer' ? 'actpass' : 'active');

            sdp += 'a=mid:' + transceiver.mid + '\r\n';

            if (transceiver.direction) {
                sdp += 'a=' + transceiver.direction + '\r\n';
            } else if (transceiver.rtpSender && transceiver.rtpReceiver) {
                sdp += 'a=sendrecv\r\n';
            } else if (transceiver.rtpSender) {
                sdp += 'a=sendonly\r\n';
            } else if (transceiver.rtpReceiver) {
                sdp += 'a=recvonly\r\n';
            } else {
                sdp += 'a=inactive\r\n';
            }

            if (transceiver.rtpSender) {
                // spec.
                var msid = 'msid:' + stream.id + ' ' +
                    transceiver.rtpSender.track.id + '\r\n';
                sdp += 'a=' + msid;

                // for Chrome.
                sdp += 'a=ssrc:' + transceiver.sendEncodingParameters[0].ssrc +
                    ' ' + msid;
                if (transceiver.sendEncodingParameters[0].rtx) {
                    sdp += 'a=ssrc:' + transceiver.sendEncodingParameters[0].rtx.ssrc +
                        ' ' + msid;
                    sdp += 'a=ssrc-group:FID ' +
                        transceiver.sendEncodingParameters[0].ssrc + ' ' +
                        transceiver.sendEncodingParameters[0].rtx.ssrc +
                        '\r\n';
                }
            }
            // FIXME: this should be written by writeRtpDescription.
            sdp += 'a=ssrc:' + transceiver.sendEncodingParameters[0].ssrc +
                ' cname:' + SDPUtils.localCName + '\r\n';
            if (transceiver.rtpSender && transceiver.sendEncodingParameters[0].rtx) {
                sdp += 'a=ssrc:' + transceiver.sendEncodingParameters[0].rtx.ssrc +
                    ' cname:' + SDPUtils.localCName + '\r\n';
            }
            return sdp;
        };

// Gets the direction from the mediaSection or the sessionpart.
        SDPUtils.getDirection = function(mediaSection, sessionpart) {
            // Look for sendrecv, sendonly, recvonly, inactive, default to sendrecv.
            var lines = SDPUtils.splitLines(mediaSection);
            for (var i = 0; i < lines.length; i++) {
                switch (lines[i]) {
                    case 'a=sendrecv':
                    case 'a=sendonly':
                    case 'a=recvonly':
                    case 'a=inactive':
                        return lines[i].substr(2);
                    default:
                    // FIXME: What should happen here?
                }
            }
            if (sessionpart) {
                return SDPUtils.getDirection(sessionpart);
            }
            return 'sendrecv';
        };

        SDPUtils.getKind = function(mediaSection) {
            var lines = SDPUtils.splitLines(mediaSection);
            var mline = lines[0].split(' ');
            return mline[0].substr(2);
        };

        SDPUtils.isRejected = function(mediaSection) {
            return mediaSection.split(' ', 2)[1] === '0';
        };

        SDPUtils.parseMLine = function(mediaSection) {
            var lines = SDPUtils.splitLines(mediaSection);
            var mline = lines[0].split(' ');
            return {
                kind: mline[0].substr(2),
                port: parseInt(mline[1], 10),
                protocol: mline[2],
                fmt: mline.slice(3).join(' ')
            };
        };

// Expose public methods.
        if (typeof module === 'object') {
            module.exports = SDPUtils;
        }

    },{}],3:[function(requirecopy,module,exports){
        (function (global){
            /*
             *  Copyright (c) 2016 The WebRTC project authors. All Rights Reserved.
             *
             *  Use of this source code is governed by a BSD-style license
             *  that can be found in the LICENSE file in the root of the source
             *  tree.
             */
            /* eslint-env node */

            'use strict';

            var adapterFactory = requirecopy('./adapter_factory.js');
            module.exports = adapterFactory({window: global.window});

        }).call(this,typeof global !== "undefined" ? global : typeof self !== "undefined" ? self : typeof window !== "undefined" ? window : {})
    },{"./adapter_factory.js":4}],4:[function(requirecopy,module,exports){
        /*
         *  Copyright (c) 2016 The WebRTC project authors. All Rights Reserved.
         *
         *  Use of this source code is governed by a BSD-style license
         *  that can be found in the LICENSE file in the root of the source
         *  tree.
         */
        /* eslint-env node */

        'use strict';

        var utils = requirecopy('./utils');
// Shimming starts here.
        module.exports = function(dependencies, opts) {
            var window = dependencies && dependencies.window;

            var options = {
                shimChrome: true,
                shimFirefox: true,
                shimEdge: true,
                shimSafari: true,
            };

            for (var key in opts) {
                if (hasOwnProperty.call(opts, key)) {
                    options[key] = opts[key];
                }
            }

            // Utils.
            var logging = utils.log;
            var browserDetails = utils.detectBrowser(window);

            // Export to the adapter global object visible in the browser.
            var adapter = {
                browserDetails: browserDetails,
                extractVersion: utils.extractVersion,
                disableLog: utils.disableLog,
                disableWarnings: utils.disableWarnings
            };

            // Uncomment the line below if you want logging to occur, including logging
            // for the switch statement below. Can also be turned on in the browser via
            // adapter.disableLog(false), but then logging from the switch statement below
            // will not appear.
            // requirecopy('./utils').disableLog(false);

            // Browser shims.
            var chromeShim = requirecopy('./chrome/chrome_shim') || null;
            var edgeShim = requirecopy('./edge/edge_shim') || null;
            var firefoxShim = requirecopy('./firefox/firefox_shim') || null;
            var safariShim = requirecopy('./safari/safari_shim') || null;
            var commonShim = requirecopy('./common_shim') || null;

            // Shim browser if found.
            switch (browserDetails.browser) {
                case 'chrome':
                    if (!chromeShim || !chromeShim.shimPeerConnection ||
                        !options.shimChrome) {
                        logging('Chrome shim is not included in this adapter release.');
                        return adapter;
                    }
                    logging('adapter.js shimming chrome.');
                    // Export to the adapter global object visible in the browser.
                    adapter.browserShim = chromeShim;
                    commonShim.shimCreateObjectURL(window);

                    chromeShim.shimGetUserMedia(window);
                    chromeShim.shimMediaStream(window);
                    chromeShim.shimSourceObject(window);
                    chromeShim.shimPeerConnection(window);
                    chromeShim.shimOnTrack(window);
                    chromeShim.shimAddTrackRemoveTrack(window);
                    chromeShim.shimGetSendersWithDtmf(window);

                    commonShim.shimRTCIceCandidate(window);
                    break;
                case 'firefox':
                    if (!firefoxShim || !firefoxShim.shimPeerConnection ||
                        !options.shimFirefox) {
                        logging('Firefox shim is not included in this adapter release.');
                        return adapter;
                    }
                    logging('adapter.js shimming firefox.');
                    // Export to the adapter global object visible in the browser.
                    adapter.browserShim = firefoxShim;
                    commonShim.shimCreateObjectURL(window);

                    firefoxShim.shimGetUserMedia(window);
                    firefoxShim.shimSourceObject(window);
                    firefoxShim.shimPeerConnection(window);
                    firefoxShim.shimOnTrack(window);
                    firefoxShim.shimRemoveStream(window);

                    commonShim.shimRTCIceCandidate(window);
                    break;
                case 'edge':
                    if (!edgeShim || !edgeShim.shimPeerConnection || !options.shimEdge) {
                        logging('MS edge shim is not included in this adapter release.');
                        return adapter;
                    }
                    logging('adapter.js shimming edge.');
                    // Export to the adapter global object visible in the browser.
                    adapter.browserShim = edgeShim;
                    commonShim.shimCreateObjectURL(window);

                    edgeShim.shimGetUserMedia(window);
                    edgeShim.shimPeerConnection(window);
                    edgeShim.shimReplaceTrack(window);

                    // the edge shim implements the full RTCIceCandidate object.
                    break;
                case 'safari':
                    if (!safariShim || !options.shimSafari) {
                        logging('Safari shim is not included in this adapter release.');
                        return adapter;
                    }
                    logging('adapter.js shimming safari.');
                    // Export to the adapter global object visible in the browser.
                    adapter.browserShim = safariShim;
                    commonShim.shimCreateObjectURL(window);

                    safariShim.shimRTCIceServerUrls(window);
                    safariShim.shimCallbacksAPI(window);
                    safariShim.shimLocalStreamsAPI(window);
                    safariShim.shimRemoteStreamsAPI(window);
                    safariShim.shimTrackEventTransceiver(window);
                    safariShim.shimGetUserMedia(window);
                    safariShim.shimCreateOfferLegacy(window);

                    commonShim.shimRTCIceCandidate(window);
                    break;
                default:
                    logging('Unsupported browser!');
                    break;
            }

            return adapter;
        };

    },{"./chrome/chrome_shim":5,"./common_shim":7,"./edge/edge_shim":8,"./firefox/firefox_shim":10,"./safari/safari_shim":12,"./utils":13}],5:[function(requirecopy,module,exports){

        /*
         *  Copyright (c) 2016 The WebRTC project authors. All Rights Reserved.
         *
         *  Use of this source code is governed by a BSD-style license
         *  that can be found in the LICENSE file in the root of the source
         *  tree.
         */
        /* eslint-env node */
        'use strict';
        var utils = requirecopy('../utils.js');
        var logging = utils.log;

        var chromeShim = {
            shimMediaStream: function(window) {
                window.MediaStream = window.MediaStream || window.webkitMediaStream;
            },

            shimOnTrack: function(window) {
                if (typeof window === 'object' && window.RTCPeerConnection && !('ontrack' in
                    window.RTCPeerConnection.prototype)) {
                    Object.defineProperty(window.RTCPeerConnection.prototype, 'ontrack', {
                        get: function() {
                            return this._ontrack;
                        },
                        set: function(f) {
                            if (this._ontrack) {
                                this.removeEventListener('track', this._ontrack);
                            }
                            this.addEventListener('track', this._ontrack = f);
                        }
                    });
                    var origSetRemoteDescription =
                        window.RTCPeerConnection.prototype.setRemoteDescription;
                    window.RTCPeerConnection.prototype.setRemoteDescription = function() {
                        var pc = this;
                        if (!pc._ontrackpoly) {
                            pc._ontrackpoly = function(e) {
                                // onaddstream does not fire when a track is added to an existing
                                // stream. But stream.onaddtrack is implemented so we use that.
                                e.stream.addEventListener('addtrack', function(te) {
                                    var receiver;
                                    if (window.RTCPeerConnection.prototype.getReceivers) {
                                        receiver = pc.getReceivers().find(function(r) {
                                            return r.track && r.track.id === te.track.id;
                                        });
                                    } else {
                                        receiver = {track: te.track};
                                    }

                                    var event = new Event('track');
                                    event.track = te.track;
                                    event.receiver = receiver;
                                    event.transceiver = {receiver: receiver};
                                    event.streams = [e.stream];
                                    pc.dispatchEvent(event);
                                });
                                e.stream.getTracks().forEach(function(track) {
                                    var receiver;
                                    if (window.RTCPeerConnection.prototype.getReceivers) {
                                        receiver = pc.getReceivers().find(function(r) {
                                            return r.track && r.track.id === track.id;
                                        });
                                    } else {
                                        receiver = {track: track};
                                    }
                                    var event = new Event('track');
                                    event.track = track;
                                    event.receiver = receiver;
                                    event.transceiver = {receiver: receiver};
                                    event.streams = [e.stream];
                                    pc.dispatchEvent(event);
                                });
                            };
                            pc.addEventListener('addstream', pc._ontrackpoly);
                        }
                        return origSetRemoteDescription.apply(pc, arguments);
                    };
                }
            },

            shimGetSendersWithDtmf: function(window) {
                // Overrides addTrack/removeTrack, depends on shimAddTrackRemoveTrack.
                if (typeof window === 'object' && window.RTCPeerConnection &&
                    !('getSenders' in window.RTCPeerConnection.prototype) &&
                    'createDTMFSender' in window.RTCPeerConnection.prototype) {
                    var shimSenderWithDtmf = function(pc, track) {
                        return {
                            track: track,
                            get dtmf() {
                                if (this._dtmf === undefined) {
                                    if (track.kind === 'audio') {
                                        this._dtmf = pc.createDTMFSender(track);
                                    } else {
                                        this._dtmf = null;
                                    }
                                }
                                return this._dtmf;
                            },
                            _pc: pc
                        };
                    };

                    // augment addTrack when getSenders is not available.
                    if (!window.RTCPeerConnection.prototype.getSenders) {
                        window.RTCPeerConnection.prototype.getSenders = function() {
                            this._senders = this._senders || [];
                            return this._senders.slice(); // return a copy of the internal state.
                        };
                        var origAddTrack = window.RTCPeerConnection.prototype.addTrack;
                        window.RTCPeerConnection.prototype.addTrack = function(track, stream) {
                            var pc = this;
                            var sender = origAddTrack.apply(pc, arguments);
                            if (!sender) {
                                sender = shimSenderWithDtmf(pc, track);
                                pc._senders.push(sender);
                            }
                            return sender;
                        };

                        var origRemoveTrack = window.RTCPeerConnection.prototype.removeTrack;
                        window.RTCPeerConnection.prototype.removeTrack = function(sender) {
                            var pc = this;
                            origRemoveTrack.apply(pc, arguments);
                            var idx = pc._senders.indexOf(sender);
                            if (idx !== -1) {
                                pc._senders.splice(idx, 1);
                            }
                        };
                    }
                    var origAddStream = window.RTCPeerConnection.prototype.addStream;
                    window.RTCPeerConnection.prototype.addStream = function(stream) {
                        var pc = this;
                        pc._senders = pc._senders || [];
                        origAddStream.apply(pc, [stream]);
                        stream.getTracks().forEach(function(track) {
                            pc._senders.push(shimSenderWithDtmf(pc, track));
                        });
                    };

                    var origRemoveStream = window.RTCPeerConnection.prototype.removeStream;
                    window.RTCPeerConnection.prototype.removeStream = function(stream) {
                        var pc = this;
                        pc._senders = pc._senders || [];
                        origRemoveStream.apply(pc, [stream]);

                        stream.getTracks().forEach(function(track) {
                            var sender = pc._senders.find(function(s) {
                                return s.track === track;
                            });
                            if (sender) {
                                pc._senders.splice(pc._senders.indexOf(sender), 1); // remove sender
                            }
                        });
                    };
                } else if (typeof window === 'object' && window.RTCPeerConnection &&
                    'getSenders' in window.RTCPeerConnection.prototype &&
                    'createDTMFSender' in window.RTCPeerConnection.prototype &&
                    window.RTCRtpSender &&
                    !('dtmf' in window.RTCRtpSender.prototype)) {
                    var origGetSenders = window.RTCPeerConnection.prototype.getSenders;
                    window.RTCPeerConnection.prototype.getSenders = function() {
                        var pc = this;
                        var senders = origGetSenders.apply(pc, []);
                        senders.forEach(function(sender) {
                            sender._pc = pc;
                        });
                        return senders;
                    };

                    Object.defineProperty(window.RTCRtpSender.prototype, 'dtmf', {
                        get: function() {
                            if (this._dtmf === undefined) {
                                if (this.track.kind === 'audio') {
                                    this._dtmf = this._pc.createDTMFSender(this.track);
                                } else {
                                    this._dtmf = null;
                                }
                            }
                            return this._dtmf;
                        }
                    });
                }
            },

            shimSourceObject: function(window) {
                var URL = window && window.URL;

                if (typeof window === 'object') {
                    if (window.HTMLMediaElement &&
                        !('srcObject' in window.HTMLMediaElement.prototype)) {
                        // Shim the srcObject property, once, when HTMLMediaElement is found.
                        Object.defineProperty(window.HTMLMediaElement.prototype, 'srcObject', {
                            get: function() {
                                return this._srcObject;
                            },
                            set: function(stream) {
                                var self = this;
                                // Use _srcObject as a private property for this shim
                                this._srcObject = stream;
                                if (this.src) {
                                    URL.revokeObjectURL(this.src);
                                }

                                if (!stream) {
                                    this.src = '';
                                    return undefined;
                                }
                                this.src = URL.createObjectURL(stream);
                                // We need to recreate the blob url when a track is added or
                                // removed. Doing it manually since we want to avoid a recursion.
                                stream.addEventListener('addtrack', function() {
                                    if (self.src) {
                                        URL.revokeObjectURL(self.src);
                                    }
                                    self.src = URL.createObjectURL(stream);
                                });
                                stream.addEventListener('removetrack', function() {
                                    if (self.src) {
                                        URL.revokeObjectURL(self.src);
                                    }
                                    self.src = URL.createObjectURL(stream);
                                });
                            }
                        });
                    }
                }
            },

            shimAddTrackRemoveTrack: function(window) {
                var browserDetails = utils.detectBrowser(window);
                // shim addTrack and removeTrack.
                if (window.RTCPeerConnection.prototype.addTrack &&
                    browserDetails.version >= 64) {
                    return;
                }

                // also shim pc.getLocalStreams when addTrack is shimmed
                // to return the original streams.
                var origGetLocalStreams = window.RTCPeerConnection.prototype
                    .getLocalStreams;
                window.RTCPeerConnection.prototype.getLocalStreams = function() {
                    var self = this;
                    var nativeStreams = origGetLocalStreams.apply(this);
                    self._reverseStreams = self._reverseStreams || {};
                    return nativeStreams.map(function(stream) {
                        return self._reverseStreams[stream.id];
                    });
                };

                var origAddStream = window.RTCPeerConnection.prototype.addStream;
                window.RTCPeerConnection.prototype.addStream = function(stream) {
                    var pc = this;
                    pc._streams = pc._streams || {};
                    pc._reverseStreams = pc._reverseStreams || {};

                    stream.getTracks().forEach(function(track) {
                        var alreadyExists = pc.getSenders().find(function(s) {
                            return s.track === track;
                        });
                        if (alreadyExists) {
                            throw new DOMException('Track already exists.',
                                'InvalidAccessError');
                        }
                    });
                    // Add identity mapping for consistency with addTrack.
                    // Unless this is being used with a stream from addTrack.
                    if (!pc._reverseStreams[stream.id]) {
                        var newStream = new window.MediaStream(stream.getTracks());
                        pc._streams[stream.id] = newStream;
                        pc._reverseStreams[newStream.id] = stream;
                        stream = newStream;
                    }
                    origAddStream.apply(pc, [stream]);
                };

                var origRemoveStream = window.RTCPeerConnection.prototype.removeStream;
                window.RTCPeerConnection.prototype.removeStream = function(stream) {
                    var pc = this;
                    pc._streams = pc._streams || {};
                    pc._reverseStreams = pc._reverseStreams || {};

                    origRemoveStream.apply(pc, [(pc._streams[stream.id] || stream)]);
                    delete pc._reverseStreams[(pc._streams[stream.id] ?
                        pc._streams[stream.id].id : stream.id)];
                    delete pc._streams[stream.id];
                };

                window.RTCPeerConnection.prototype.addTrack = function(track, stream) {
                    var pc = this;
                    if (pc.signalingState === 'closed') {
                        throw new DOMException(
                            'The RTCPeerConnection\'s signalingState is \'closed\'.',
                            'InvalidStateError');
                    }
                    var streams = [].slice.call(arguments, 1);
                    if (streams.length !== 1 ||
                        !streams[0].getTracks().find(function(t) {
                            return t === track;
                        })) {
                        // this is not fully correct but all we can manage without
                        // [[associated MediaStreams]] internal slot.
                        throw new DOMException(
                            'The adapter.js addTrack polyfill only supports a single ' +
                            ' stream which is associated with the specified track.',
                            'NotSupportedError');
                    }

                    var alreadyExists = pc.getSenders().find(function(s) {
                        return s.track === track;
                    });
                    if (alreadyExists) {
                        throw new DOMException('Track already exists.',
                            'InvalidAccessError');
                    }

                    pc._streams = pc._streams || {};
                    pc._reverseStreams = pc._reverseStreams || {};
                    var oldStream = pc._streams[stream.id];
                    if (oldStream) {
                        // this is using odd Chrome behaviour, use with caution:
                        // https://bugs.chromium.org/p/webrtc/issues/detail?id=7815
                        // Note: we rely on the high-level addTrack/dtmf shim to
                        // create the sender with a dtmf sender.
                        oldStream.addTrack(track);

                        // Trigger ONN async.
                        Promise.resolve().then(function() {
                            pc.dispatchEvent(new Event('negotiationneeded'));
                        });
                    } else {
                        var newStream = new window.MediaStream([track]);
                        pc._streams[stream.id] = newStream;
                        pc._reverseStreams[newStream.id] = stream;
                        pc.addStream(newStream);
                    }
                    return pc.getSenders().find(function(s) {
                        return s.track === track;
                    });
                };

                // replace the internal stream id with the external one and
                // vice versa.
                function replaceInternalStreamId(pc, description) {
                    var sdp = description.sdp;
                    Object.keys(pc._reverseStreams || []).forEach(function(internalId) {
                        var externalStream = pc._reverseStreams[internalId];
                        var internalStream = pc._streams[externalStream.id];
                        sdp = sdp.replace(new RegExp(internalStream.id, 'g'),
                            externalStream.id);
                    });
                    return new RTCSessionDescription({
                        type: description.type,
                        sdp: sdp
                    });
                }
                function replaceExternalStreamId(pc, description) {
                    var sdp = description.sdp;
                    Object.keys(pc._reverseStreams || []).forEach(function(internalId) {
                        var externalStream = pc._reverseStreams[internalId];
                        var internalStream = pc._streams[externalStream.id];
                        sdp = sdp.replace(new RegExp(externalStream.id, 'g'),
                            internalStream.id);
                    });
                    return new RTCSessionDescription({
                        type: description.type,
                        sdp: sdp
                    });
                }
                ['createOffer', 'createAnswer'].forEach(function(method) {
                    var nativeMethod = window.RTCPeerConnection.prototype[method];
                    window.RTCPeerConnection.prototype[method] = function() {
                        var pc = this;
                        var args = arguments;
                        var isLegacyCall = arguments.length &&
                            typeof arguments[0] === 'function';
                        if (isLegacyCall) {
                            return nativeMethod.apply(pc, [
                                function(description) {
                                    var desc = replaceInternalStreamId(pc, description);
                                    args[0].apply(null, [desc]);
                                },
                                function(err) {
                                    if (args[1]) {
                                        args[1].apply(null, err);
                                    }
                                }, arguments[2]
                            ]);
                        }
                        return nativeMethod.apply(pc, arguments)
                            .then(function(description) {
                                return replaceInternalStreamId(pc, description);
                            });
                    };
                });

                var origSetLocalDescription =
                    window.RTCPeerConnection.prototype.setLocalDescription;
                window.RTCPeerConnection.prototype.setLocalDescription = function() {
                    var pc = this;
                    if (!arguments.length || !arguments[0].type) {
                        return origSetLocalDescription.apply(pc, arguments);
                    }
                    arguments[0] = replaceExternalStreamId(pc, arguments[0]);
                    return origSetLocalDescription.apply(pc, arguments);
                };

                // TODO: mangle getStats: https://w3c.github.io/webrtc-stats/#dom-rtcmediastreamstats-streamidentifier

                var origLocalDescription = Object.getOwnPropertyDescriptor(
                    window.RTCPeerConnection.prototype, 'localDescription');
                Object.defineProperty(window.RTCPeerConnection.prototype,
                    'localDescription', {
                        get: function() {
                            var pc = this;
                            var description = origLocalDescription.get.apply(this);
                            if (description.type === '') {
                                return description;
                            }
                            return replaceInternalStreamId(pc, description);
                        }
                    });

                window.RTCPeerConnection.prototype.removeTrack = function(sender) {
                    var pc = this;
                    if (pc.signalingState === 'closed') {
                        throw new DOMException(
                            'The RTCPeerConnection\'s signalingState is \'closed\'.',
                            'InvalidStateError');
                    }
                    // We can not yet check for sender instanceof RTCRtpSender
                    // since we shim RTPSender. So we check if sender._pc is set.
                    if (!sender._pc) {
                        throw new DOMException('Argument 1 of RTCPeerConnection.removeTrack ' +
                            'does not implement interface RTCRtpSender.', 'TypeError');
                    }
                    var isLocal = sender._pc === pc;
                    if (!isLocal) {
                        throw new DOMException('Sender was not created by this connection.',
                            'InvalidAccessError');
                    }

                    // Search for the native stream the senders track belongs to.
                    pc._streams = pc._streams || {};
                    var stream;
                    Object.keys(pc._streams).forEach(function(streamid) {
                        var hasTrack = pc._streams[streamid].getTracks().find(function(track) {
                            return sender.track === track;
                        });
                        if (hasTrack) {
                            stream = pc._streams[streamid];
                        }
                    });

                    if (stream) {
                        if (stream.getTracks().length === 1) {
                            // if this is the last track of the stream, remove the stream. This
                            // takes care of any shimmed _senders.
                            pc.removeStream(pc._reverseStreams[stream.id]);
                        } else {
                            // relying on the same odd chrome behaviour as above.
                            stream.removeTrack(sender.track);
                        }
                        pc.dispatchEvent(new Event('negotiationneeded'));
                    }
                };
            },

            shimPeerConnection: function(window) {
                var browserDetails = utils.detectBrowser(window);

                // The RTCPeerConnection object.
                if (!window.RTCPeerConnection) {
                    window.RTCPeerConnection = function(pcConfig, pcConstraints) {
                        // Translate iceTransportPolicy to iceTransports,
                        // see https://code.google.com/p/webrtc/issues/detail?id=4869
                        // this was fixed in M56 along with unprefixing RTCPeerConnection.
                        logging('PeerConnection');
                        if (pcConfig && pcConfig.iceTransportPolicy) {
                            pcConfig.iceTransports = pcConfig.iceTransportPolicy;
                        }

                        return new window.webkitRTCPeerConnection(pcConfig, pcConstraints);
                    };
                    window.RTCPeerConnection.prototype =
                        window.webkitRTCPeerConnection.prototype;
                    // wrap static methods. Currently just generateCertificate.
                    if (window.webkitRTCPeerConnection.generateCertificate) {
                        Object.defineProperty(window.RTCPeerConnection, 'generateCertificate', {
                            get: function() {
                                return window.webkitRTCPeerConnection.generateCertificate;
                            }
                        });
                    }
                } else {
                    // migrate from non-spec RTCIceServer.url to RTCIceServer.urls
                    var OrigPeerConnection = window.RTCPeerConnection;
                    window.RTCPeerConnection = function(pcConfig, pcConstraints) {
                        if (pcConfig && pcConfig.iceServers) {
                            var newIceServers = [];
                            for (var i = 0; i < pcConfig.iceServers.length; i++) {
                                var server = pcConfig.iceServers[i];
                                if (!server.hasOwnProperty('urls') &&
                                    server.hasOwnProperty('url')) {
                                    utils.deprecated('RTCIceServer.url', 'RTCIceServer.urls');
                                    server = JSON.parse(JSON.stringify(server));
                                    server.urls = server.url;
                                    newIceServers.push(server);
                                } else {
                                    newIceServers.push(pcConfig.iceServers[i]);
                                }
                            }
                            pcConfig.iceServers = newIceServers;
                        }
                        return new OrigPeerConnection(pcConfig, pcConstraints);
                    };
                    window.RTCPeerConnection.prototype = OrigPeerConnection.prototype;
                    // wrap static methods. Currently just generateCertificate.
                    Object.defineProperty(window.RTCPeerConnection, 'generateCertificate', {
                        get: function() {
                            return OrigPeerConnection.generateCertificate;
                        }
                    });
                }

                var origGetStats = window.RTCPeerConnection.prototype.getStats;
                window.RTCPeerConnection.prototype.getStats = function(selector,
                                                                       successCallback, errorCallback) {
                    var self = this;
                    var args = arguments;

                    // If selector is a function then we are in the old style stats so just
                    // pass back the original getStats format to avoid breaking old users.
                    if (arguments.length > 0 && typeof selector === 'function') {
                        return origGetStats.apply(this, arguments);
                    }

                    // When spec-style getStats is supported, return those when called with
                    // either no arguments or the selector argument is null.
                    if (origGetStats.length === 0 && (arguments.length === 0 ||
                        typeof arguments[0] !== 'function')) {
                        return origGetStats.apply(this, []);
                    }

                    var fixChromeStats_ = function(response) {
                        var standardReport = {};
                        var reports = response.result();
                        reports.forEach(function(report) {
                            var standardStats = {
                                id: report.id,
                                timestamp: report.timestamp,
                                type: {
                                    localcandidate: 'local-candidate',
                                    remotecandidate: 'remote-candidate'
                                }[report.type] || report.type
                            };
                            report.names().forEach(function(name) {
                                standardStats[name] = report.stat(name);
                            });
                            standardReport[standardStats.id] = standardStats;
                        });

                        return standardReport;
                    };

                    // shim getStats with maplike support
                    var makeMapStats = function(stats) {
                        return new Map(Object.keys(stats).map(function(key) {
                            return [key, stats[key]];
                        }));
                    };

                    if (arguments.length >= 2) {
                        var successCallbackWrapper_ = function(response) {
                            args[1](makeMapStats(fixChromeStats_(response)));
                        };

                        return origGetStats.apply(this, [successCallbackWrapper_,
                            arguments[0]]);
                    }

                    // promise-support
                    return new Promise(function(resolve, reject) {
                        origGetStats.apply(self, [
                            function(response) {
                                resolve(makeMapStats(fixChromeStats_(response)));
                            }, reject]);
                    }).then(successCallback, errorCallback);
                };

                // add promise support -- natively available in Chrome 51
                if (browserDetails.version < 51) {
                    ['setLocalDescription', 'setRemoteDescription', 'addIceCandidate']
                        .forEach(function(method) {
                            var nativeMethod = window.RTCPeerConnection.prototype[method];
                            window.RTCPeerConnection.prototype[method] = function() {
                                var args = arguments;
                                var self = this;
                                var promise = new Promise(function(resolve, reject) {
                                    nativeMethod.apply(self, [args[0], resolve, reject]);
                                });
                                if (args.length < 2) {
                                    return promise;
                                }
                                return promise.then(function() {
                                        args[1].apply(null, []);
                                    },
                                    function(err) {
                                        if (args.length >= 3) {
                                            args[2].apply(null, [err]);
                                        }
                                    });
                            };
                        });
                }

                // promise support for createOffer and createAnswer. Available (without
                // bugs) since M52: crbug/619289
                if (browserDetails.version < 52) {
                    ['createOffer', 'createAnswer'].forEach(function(method) {
                        var nativeMethod = window.RTCPeerConnection.prototype[method];
                        window.RTCPeerConnection.prototype[method] = function() {
                            var self = this;
                            if (arguments.length < 1 || (arguments.length === 1 &&
                                typeof arguments[0] === 'object')) {
                                var opts = arguments.length === 1 ? arguments[0] : undefined;
                                return new Promise(function(resolve, reject) {
                                    nativeMethod.apply(self, [resolve, reject, opts]);
                                });
                            }
                            return nativeMethod.apply(this, arguments);
                        };
                    });
                }

                // shim implicit creation of RTCSessionDescription/RTCIceCandidate
                ['setLocalDescription', 'setRemoteDescription', 'addIceCandidate']
                    .forEach(function(method) {
                        var nativeMethod = window.RTCPeerConnection.prototype[method];
                        window.RTCPeerConnection.prototype[method] = function() {
                            arguments[0] = new ((method === 'addIceCandidate') ?
                                window.RTCIceCandidate :
                                window.RTCSessionDescription)(arguments[0]);
                            return nativeMethod.apply(this, arguments);
                        };
                    });

                // support for addIceCandidate(null or undefined)
                var nativeAddIceCandidate =
                    window.RTCPeerConnection.prototype.addIceCandidate;
                window.RTCPeerConnection.prototype.addIceCandidate = function() {
                    if (!arguments[0]) {
                        if (arguments[1]) {
                            arguments[1].apply(null);
                        }
                        return Promise.resolve();
                    }
                    return nativeAddIceCandidate.apply(this, arguments);
                };
            }
        };


// Expose public methods.
        module.exports = {
            shimMediaStream: chromeShim.shimMediaStream,
            shimOnTrack: chromeShim.shimOnTrack,
            shimAddTrackRemoveTrack: chromeShim.shimAddTrackRemoveTrack,
            shimGetSendersWithDtmf: chromeShim.shimGetSendersWithDtmf,
            shimSourceObject: chromeShim.shimSourceObject,
            shimPeerConnection: chromeShim.shimPeerConnection,
            shimGetUserMedia: requirecopy('./getusermedia')
        };

    },{"../utils.js":13,"./getusermedia":6}],6:[function(requirecopy,module,exports){
        /*
         *  Copyright (c) 2016 The WebRTC project authors. All Rights Reserved.
         *
         *  Use of this source code is governed by a BSD-style license
         *  that can be found in the LICENSE file in the root of the source
         *  tree.
         */
        /* eslint-env node */
        'use strict';
        var utils = requirecopy('../utils.js');
        var logging = utils.log;

// Expose public methods.
        module.exports = function(window) {
            var browserDetails = utils.detectBrowser(window);
            var navigator = window && window.navigator;

            var constraintsToChrome_ = function(c) {
                if (typeof c !== 'object' || c.mandatory || c.optional) {
                    return c;
                }
                var cc = {};
                Object.keys(c).forEach(function(key) {
                    if (key === 'require' || key === 'advanced' || key === 'mediaSource') {
                        return;
                    }
                    var r = (typeof c[key] === 'object') ? c[key] : {ideal: c[key]};
                    if (r.exact !== undefined && typeof r.exact === 'number') {
                        r.min = r.max = r.exact;
                    }
                    var oldname_ = function(prefix, name) {
                        if (prefix) {
                            return prefix + name.charAt(0).toUpperCase() + name.slice(1);
                        }
                        return (name === 'deviceId') ? 'sourceId' : name;
                    };
                    if (r.ideal !== undefined) {
                        cc.optional = cc.optional || [];
                        var oc = {};
                        if (typeof r.ideal === 'number') {
                            oc[oldname_('min', key)] = r.ideal;
                            cc.optional.push(oc);
                            oc = {};
                            oc[oldname_('max', key)] = r.ideal;
                            cc.optional.push(oc);
                        } else {
                            oc[oldname_('', key)] = r.ideal;
                            cc.optional.push(oc);
                        }
                    }
                    if (r.exact !== undefined && typeof r.exact !== 'number') {
                        cc.mandatory = cc.mandatory || {};
                        cc.mandatory[oldname_('', key)] = r.exact;
                    } else {
                        ['min', 'max'].forEach(function(mix) {
                            if (r[mix] !== undefined) {
                                cc.mandatory = cc.mandatory || {};
                                cc.mandatory[oldname_(mix, key)] = r[mix];
                            }
                        });
                    }
                });
                if (c.advanced) {
                    cc.optional = (cc.optional || []).concat(c.advanced);
                }
                return cc;
            };

            var shimConstraints_ = function(constraints, func) {
                if (browserDetails.version >= 61) {
                    return func(constraints);
                }
                constraints = JSON.parse(JSON.stringify(constraints));
                if (constraints && typeof constraints.audio === 'object') {
                    var remap = function(obj, a, b) {
                        if (a in obj && !(b in obj)) {
                            obj[b] = obj[a];
                            delete obj[a];
                        }
                    };
                    constraints = JSON.parse(JSON.stringify(constraints));
                    remap(constraints.audio, 'autoGainControl', 'googAutoGainControl');
                    remap(constraints.audio, 'noiseSuppression', 'googNoiseSuppression');
                    constraints.audio = constraintsToChrome_(constraints.audio);
                }
                if (constraints && typeof constraints.video === 'object') {
                    // Shim facingMode for mobile & surface pro.
                    var face = constraints.video.facingMode;
                    face = face && ((typeof face === 'object') ? face : {ideal: face});
                    var getSupportedFacingModeLies = browserDetails.version < 66;

                    if ((face && (face.exact === 'user' || face.exact === 'environment' ||
                        face.ideal === 'user' || face.ideal === 'environment')) &&
                        !(navigator.mediaDevices.getSupportedConstraints &&
                        navigator.mediaDevices.getSupportedConstraints().facingMode &&
                        !getSupportedFacingModeLies)) {
                        delete constraints.video.facingMode;
                        var matches;
                        if (face.exact === 'environment' || face.ideal === 'environment') {
                            matches = ['back', 'rear'];
                        } else if (face.exact === 'user' || face.ideal === 'user') {
                            matches = ['front'];
                        }
                        if (matches) {
                            // Look for matches in label, or use last cam for back (typical).
                            return navigator.mediaDevices.enumerateDevices()
                                .then(function(devices) {
                                    devices = devices.filter(function(d) {
                                        return d.kind === 'videoinput';
                                    });
                                    var dev = devices.find(function(d) {
                                        return matches.some(function(match) {
                                            return d.label.toLowerCase().indexOf(match) !== -1;
                                        });
                                    });
                                    if (!dev && devices.length && matches.indexOf('back') !== -1) {
                                        dev = devices[devices.length - 1]; // more likely the back cam
                                    }
                                    if (dev) {
                                        constraints.video.deviceId = face.exact ? {exact: dev.deviceId} :
                                            {ideal: dev.deviceId};
                                    }
                                    constraints.video = constraintsToChrome_(constraints.video);
                                    logging('chrome: ' + JSON.stringify(constraints));
                                    return func(constraints);
                                });
                        }
                    }
                    constraints.video = constraintsToChrome_(constraints.video);
                }
                logging('chrome: ' + JSON.stringify(constraints));
                return func(constraints);
            };

            var shimError_ = function(e) {
                return {
                    name: {
                        PermissionDeniedError: 'NotAllowedError',
                        InvalidStateError: 'NotReadableError',
                        DevicesNotFoundError: 'NotFoundError',
                        ConstraintNotSatisfiedError: 'OverconstrainedError',
                        TrackStartError: 'NotReadableError',
                        MediaDeviceFailedDueToShutdown: 'NotReadableError',
                        MediaDeviceKillSwitchOn: 'NotReadableError'
                    }[e.name] || e.name,
                    message: e.message,
                    constraint: e.constraintName,
                    toString: function() {
                        return this.name + (this.message && ': ') + this.message;
                    }
                };
            };

            var getUserMedia_ = function(constraints, onSuccess, onError) {
                shimConstraints_(constraints, function(c) {
                    navigator.webkitGetUserMedia(c, onSuccess, function(e) {
                        if (onError) {
                            onError(shimError_(e));
                        }
                    });
                });
            };

            navigator.getUserMedia = getUserMedia_;

            // Returns the result of getUserMedia as a Promise.
            var getUserMediaPromise_ = function(constraints) {
                return new Promise(function(resolve, reject) {
                    navigator.getUserMedia(constraints, resolve, reject);
                });
            };

            if (!navigator.mediaDevices) {
                navigator.mediaDevices = {
                    getUserMedia: getUserMediaPromise_,
                    enumerateDevices: function() {
                        return new Promise(function(resolve) {
                            var kinds = {audio: 'audioinput', video: 'videoinput'};
                            return window.MediaStreamTrack.getSources(function(devices) {
                                resolve(devices.map(function(device) {
                                    return {label: device.label,
                                        kind: kinds[device.kind],
                                        deviceId: device.id,
                                        groupId: ''};
                                }));
                            });
                        });
                    },
                    getSupportedConstraints: function() {
                        return {
                            deviceId: true, echoCancellation: true, facingMode: true,
                            frameRate: true, height: true, width: true
                        };
                    }
                };
            }

            // A shim for getUserMedia method on the mediaDevices object.
            // TODO(KaptenJansson) remove once implemented in Chrome stable.
            if (!navigator.mediaDevices.getUserMedia) {
                navigator.mediaDevices.getUserMedia = function(constraints) {
                    return getUserMediaPromise_(constraints);
                };
            } else {
                // Even though Chrome 45 has navigator.mediaDevices and a getUserMedia
                // function which returns a Promise, it does not accept spec-style
                // constraints.
                var origGetUserMedia = navigator.mediaDevices.getUserMedia.
                bind(navigator.mediaDevices);
                navigator.mediaDevices.getUserMedia = function(cs) {
                    return shimConstraints_(cs, function(c) {
                        return origGetUserMedia(c).then(function(stream) {
                            if (c.audio && !stream.getAudioTracks().length ||
                                c.video && !stream.getVideoTracks().length) {
                                stream.getTracks().forEach(function(track) {
                                    track.stop();
                                });
                                throw new DOMException('', 'NotFoundError');
                            }
                            return stream;
                        }, function(e) {
                            return Promise.reject(shimError_(e));
                        });
                    });
                };
            }

            // Dummy devicechange event methods.
            // TODO(KaptenJansson) remove once implemented in Chrome stable.
            if (typeof navigator.mediaDevices.addEventListener === 'undefined') {
                navigator.mediaDevices.addEventListener = function() {
                    logging('Dummy mediaDevices.addEventListener called.');
                };
            }
            if (typeof navigator.mediaDevices.removeEventListener === 'undefined') {
                navigator.mediaDevices.removeEventListener = function() {
                    logging('Dummy mediaDevices.removeEventListener called.');
                };
            }
        };

    },{"../utils.js":13}],7:[function(requirecopy,module,exports){
        /*
         *  Copyright (c) 2017 The WebRTC project authors. All Rights Reserved.
         *
         *  Use of this source code is governed by a BSD-style license
         *  that can be found in the LICENSE file in the root of the source
         *  tree.
         */
        /* eslint-env node */
        'use strict';

        var SDPUtils = requirecopy('sdp');
        var utils = requirecopy('./utils');

// Wraps the peerconnection event eventNameToWrap in a function
// which returns the modified event object.
        function wrapPeerConnectionEvent(window, eventNameToWrap, wrapper) {
            if (!window.RTCPeerConnection) {
                return;
            }
            var proto = window.RTCPeerConnection.prototype;
            var nativeAddEventListener = proto.addEventListener;
            proto.addEventListener = function(nativeEventName, cb) {
                if (nativeEventName !== eventNameToWrap) {
                    return nativeAddEventListener.apply(this, arguments);
                }
                var wrappedCallback = function(e) {
                    cb(wrapper(e));
                };
                this._eventMap = this._eventMap || {};
                this._eventMap[cb] = wrappedCallback;
                return nativeAddEventListener.apply(this, [nativeEventName,
                    wrappedCallback]);
            };

            var nativeRemoveEventListener = proto.removeEventListener;
            proto.removeEventListener = function(nativeEventName, cb) {
                if (nativeEventName !== eventNameToWrap || !this._eventMap
                    || !this._eventMap[cb]) {
                    return nativeRemoveEventListener.apply(this, arguments);
                }
                var unwrappedCb = this._eventMap[cb];
                delete this._eventMap[cb];
                return nativeRemoveEventListener.apply(this, [nativeEventName,
                    unwrappedCb]);
            };

            Object.defineProperty(proto, 'on' + eventNameToWrap, {
                get: function() {
                    return this['_on' + eventNameToWrap];
                },
                set: function(cb) {
                    if (this['_on' + eventNameToWrap]) {
                        this.removeEventListener(eventNameToWrap,
                            this['_on' + eventNameToWrap]);
                        delete this['_on' + eventNameToWrap];
                    }
                    if (cb) {
                        this.addEventListener(eventNameToWrap,
                            this['_on' + eventNameToWrap] = cb);
                    }
                }
            });
        }

        module.exports = {
            shimRTCIceCandidate: function(window) {
                // foundation is arbitrarily chosen as an indicator for full support for
                // https://w3c.github.io/webrtc-pc/#rtcicecandidate-interface
                if (window.RTCIceCandidate && 'foundation' in
                    window.RTCIceCandidate.prototype) {
                    return;
                }

                var NativeRTCIceCandidate = window.RTCIceCandidate;
                window.RTCIceCandidate = function(args) {
                    // Remove the a= which shouldn't be part of the candidate string.
                    if (typeof args === 'object' && args.candidate &&
                        args.candidate.indexOf('a=') === 0) {
                        args = JSON.parse(JSON.stringify(args));
                        args.candidate = args.candidate.substr(2);
                    }

                    // Augment the native candidate with the parsed fields.
                    var nativeCandidate = new NativeRTCIceCandidate(args);
                    var parsedCandidate = SDPUtils.parseCandidate(args.candidate);
                    var augmentedCandidate = Object.assign(nativeCandidate,
                        parsedCandidate);

                    // Add a serializer that does not serialize the extra attributes.
                    augmentedCandidate.toJSON = function() {
                        return {
                            candidate: augmentedCandidate.candidate,
                            sdpMid: augmentedCandidate.sdpMid,
                            sdpMLineIndex: augmentedCandidate.sdpMLineIndex,
                            usernameFragment: augmentedCandidate.usernameFragment,
                        };
                    };
                    return augmentedCandidate;
                };

                // Hook up the augmented candidate in onicecandidate and
                // addEventListener('icecandidate', ...)
                wrapPeerConnectionEvent(window, 'icecandidate', function(e) {
                    if (e.candidate) {
                        Object.defineProperty(e, 'candidate', {
                            value: new window.RTCIceCandidate(e.candidate),
                            writable: 'false'
                        });
                    }
                    return e;
                });
            },

            // shimCreateObjectURL must be called before shimSourceObject to avoid loop.

            shimCreateObjectURL: function(window) {
                var URL = window && window.URL;

                if (!(typeof window === 'object' && window.HTMLMediaElement &&
                    'srcObject' in window.HTMLMediaElement.prototype &&
                    URL.createObjectURL && URL.revokeObjectURL)) {
                    // Only shim CreateObjectURL using srcObject if srcObject exists.
                    return undefined;
                }

                var nativeCreateObjectURL = URL.createObjectURL.bind(URL);
                var nativeRevokeObjectURL = URL.revokeObjectURL.bind(URL);
                var streams = new Map(), newId = 0;

                URL.createObjectURL = function(stream) {
                    if ('getTracks' in stream) {
                        var url = 'polyblob:' + (++newId);
                        streams.set(url, stream);
                        utils.deprecated('URL.createObjectURL(stream)',
                            'elem.srcObject = stream');
                        return url;
                    }
                    return nativeCreateObjectURL(stream);
                };
                URL.revokeObjectURL = function(url) {
                    nativeRevokeObjectURL(url);
                    streams.delete(url);
                };

                var dsc = Object.getOwnPropertyDescriptor(window.HTMLMediaElement.prototype,
                    'src');
                Object.defineProperty(window.HTMLMediaElement.prototype, 'src', {
                    get: function() {
                        return dsc.get.apply(this);
                    },
                    set: function(url) {
                        this.srcObject = streams.get(url) || null;
                        return dsc.set.apply(this, [url]);
                    }
                });

                var nativeSetAttribute = window.HTMLMediaElement.prototype.setAttribute;
                window.HTMLMediaElement.prototype.setAttribute = function() {
                    if (arguments.length === 2 &&
                        ('' + arguments[0]).toLowerCase() === 'src') {
                        this.srcObject = streams.get(arguments[1]) || null;
                    }
                    return nativeSetAttribute.apply(this, arguments);
                };
            }
        };

    },{"./utils":13,"sdp":2}],8:[function(requirecopy,module,exports){
        /*
         *  Copyright (c) 2016 The WebRTC project authors. All Rights Reserved.
         *
         *  Use of this source code is governed by a BSD-style license
         *  that can be found in the LICENSE file in the root of the source
         *  tree.
         */
        /* eslint-env node */
        'use strict';

        var utils = requirecopy('../utils');
        var shimRTCPeerConnection = requirecopy('rtcpeerconnection-shim');

        module.exports = {
            shimGetUserMedia: requirecopy('./getusermedia'),
            shimPeerConnection: function(window) {
                var browserDetails = utils.detectBrowser(window);

                if (window.RTCIceGatherer) {
                    // ORTC defines an RTCIceCandidate object but no constructor.
                    // Not implemented in Edge.
                    if (!window.RTCIceCandidate) {
                        window.RTCIceCandidate = function(args) {
                            return args;
                        };
                    }
                    // ORTC does not have a session description object but
                    // other browsers (i.e. Chrome) that will support both PC and ORTC
                    // in the future might have this defined already.
                    if (!window.RTCSessionDescription) {
                        window.RTCSessionDescription = function(args) {
                            return args;
                        };
                    }
                    // this adds an additional event listener to MediaStrackTrack that signals
                    // when a tracks enabled property was changed. Workaround for a bug in
                    // addStream, see below. No longer required in 15025+
                    if (browserDetails.version < 15025) {
                        var origMSTEnabled = Object.getOwnPropertyDescriptor(
                            window.MediaStreamTrack.prototype, 'enabled');
                        Object.defineProperty(window.MediaStreamTrack.prototype, 'enabled', {
                            set: function(value) {
                                origMSTEnabled.set.call(this, value);
                                var ev = new Event('enabled');
                                ev.enabled = value;
                                this.dispatchEvent(ev);
                            }
                        });
                    }
                }

                // ORTC defines the DTMF sender a bit different.
                // https://github.com/w3c/ortc/issues/714
                if (window.RTCRtpSender && !('dtmf' in window.RTCRtpSender.prototype)) {
                    Object.defineProperty(window.RTCRtpSender.prototype, 'dtmf', {
                        get: function() {
                            if (this._dtmf === undefined) {
                                if (this.track.kind === 'audio') {
                                    this._dtmf = new window.RTCDtmfSender(this);
                                } else if (this.track.kind === 'video') {
                                    this._dtmf = null;
                                }
                            }
                            return this._dtmf;
                        }
                    });
                }

                window.RTCPeerConnection =
                    shimRTCPeerConnection(window, browserDetails.version);
            },
            shimReplaceTrack: function(window) {
                // ORTC has replaceTrack -- https://github.com/w3c/ortc/issues/614
                if (window.RTCRtpSender &&
                    !('replaceTrack' in window.RTCRtpSender.prototype)) {
                    window.RTCRtpSender.prototype.replaceTrack =
                        window.RTCRtpSender.prototype.setTrack;
                }
            }
        };

    },{"../utils":13,"./getusermedia":9,"rtcpeerconnection-shim":1}],9:[function(requirecopy,module,exports){
        /*
         *  Copyright (c) 2016 The WebRTC project authors. All Rights Reserved.
         *
         *  Use of this source code is governed by a BSD-style license
         *  that can be found in the LICENSE file in the root of the source
         *  tree.
         */
        /* eslint-env node */
        'use strict';

// Expose public methods.
        module.exports = function(window) {
            var navigator = window && window.navigator;

            var shimError_ = function(e) {
                return {
                    name: {PermissionDeniedError: 'NotAllowedError'}[e.name] || e.name,
                    message: e.message,
                    constraint: e.constraint,
                    toString: function() {
                        return this.name;
                    }
                };
            };

            // getUserMedia error shim.
            var origGetUserMedia = navigator.mediaDevices.getUserMedia.
            bind(navigator.mediaDevices);
            navigator.mediaDevices.getUserMedia = function(c) {
                return origGetUserMedia(c).catch(function(e) {
                    return Promise.reject(shimError_(e));
                });
            };
        };

    },{}],10:[function(requirecopy,module,exports){
        /*
         *  Copyright (c) 2016 The WebRTC project authors. All Rights Reserved.
         *
         *  Use of this source code is governed by a BSD-style license
         *  that can be found in the LICENSE file in the root of the source
         *  tree.
         */
        /* eslint-env node */
        'use strict';

        var utils = requirecopy('../utils');

        var firefoxShim = {
            shimOnTrack: function(window) {
                if (typeof window === 'object' && window.RTCPeerConnection && !('ontrack' in
                    window.RTCPeerConnection.prototype)) {
                    Object.defineProperty(window.RTCPeerConnection.prototype, 'ontrack', {
                        get: function() {
                            return this._ontrack;
                        },
                        set: function(f) {
                            if (this._ontrack) {
                                this.removeEventListener('track', this._ontrack);
                                this.removeEventListener('addstream', this._ontrackpoly);
                            }
                            this.addEventListener('track', this._ontrack = f);
                            this.addEventListener('addstream', this._ontrackpoly = function(e) {
                                e.stream.getTracks().forEach(function(track) {
                                    var event = new Event('track');
                                    event.track = track;
                                    event.receiver = {track: track};
                                    event.transceiver = {receiver: event.receiver};
                                    event.streams = [e.stream];
                                    this.dispatchEvent(event);
                                }.bind(this));
                            }.bind(this));
                        }
                    });
                }
                if (typeof window === 'object' && window.RTCTrackEvent &&
                    ('receiver' in window.RTCTrackEvent.prototype) &&
                    !('transceiver' in window.RTCTrackEvent.prototype)) {
                    Object.defineProperty(window.RTCTrackEvent.prototype, 'transceiver', {
                        get: function() {
                            return {receiver: this.receiver};
                        }
                    });
                }
            },

            shimSourceObject: function(window) {
                // Firefox has supported mozSrcObject since FF22, unprefixed in 42.
                if (typeof window === 'object') {
                    if (window.HTMLMediaElement &&
                        !('srcObject' in window.HTMLMediaElement.prototype)) {
                        // Shim the srcObject property, once, when HTMLMediaElement is found.
                        Object.defineProperty(window.HTMLMediaElement.prototype, 'srcObject', {
                            get: function() {
                                return this.mozSrcObject;
                            },
                            set: function(stream) {
                                this.mozSrcObject = stream;
                            }
                        });
                    }
                }
            },

            shimPeerConnection: function(window) {
                var browserDetails = utils.detectBrowser(window);

                if (typeof window !== 'object' || !(window.RTCPeerConnection ||
                    window.mozRTCPeerConnection)) {
                    return; // probably media.peerconnection.enabled=false in about:config
                }
                // The RTCPeerConnection object.
                if (!window.RTCPeerConnection) {
                    window.RTCPeerConnection = function(pcConfig, pcConstraints) {
                        if (browserDetails.version < 38) {
                            // .urls is not supported in FF < 38.
                            // create RTCIceServers with a single url.
                            if (pcConfig && pcConfig.iceServers) {
                                var newIceServers = [];
                                for (var i = 0; i < pcConfig.iceServers.length; i++) {
                                    var server = pcConfig.iceServers[i];
                                    if (server.hasOwnProperty('urls')) {
                                        for (var j = 0; j < server.urls.length; j++) {
                                            var newServer = {
                                                url: server.urls[j]
                                            };
                                            if (server.urls[j].indexOf('turn') === 0) {
                                                newServer.username = server.username;
                                                newServer.credential = server.credential;
                                            }
                                            newIceServers.push(newServer);
                                        }
                                    } else {
                                        newIceServers.push(pcConfig.iceServers[i]);
                                    }
                                }
                                pcConfig.iceServers = newIceServers;
                            }
                        }
                        return new window.mozRTCPeerConnection(pcConfig, pcConstraints);
                    };
                    window.RTCPeerConnection.prototype =
                        window.mozRTCPeerConnection.prototype;

                    // wrap static methods. Currently just generateCertificate.
                    if (window.mozRTCPeerConnection.generateCertificate) {
                        Object.defineProperty(window.RTCPeerConnection, 'generateCertificate', {
                            get: function() {
                                return window.mozRTCPeerConnection.generateCertificate;
                            }
                        });
                    }

                    window.RTCSessionDescription = window.mozRTCSessionDescription;
                    window.RTCIceCandidate = window.mozRTCIceCandidate;
                }

                // shim away need for obsolete RTCIceCandidate/RTCSessionDescription.
                ['setLocalDescription', 'setRemoteDescription', 'addIceCandidate']
                    .forEach(function(method) {
                        var nativeMethod = window.RTCPeerConnection.prototype[method];
                        window.RTCPeerConnection.prototype[method] = function() {
                            arguments[0] = new ((method === 'addIceCandidate') ?
                                window.RTCIceCandidate :
                                window.RTCSessionDescription)(arguments[0]);
                            return nativeMethod.apply(this, arguments);
                        };
                    });

                // support for addIceCandidate(null or undefined)
                var nativeAddIceCandidate =
                    window.RTCPeerConnection.prototype.addIceCandidate;
                window.RTCPeerConnection.prototype.addIceCandidate = function() {
                    if (!arguments[0]) {
                        if (arguments[1]) {
                            arguments[1].apply(null);
                        }
                        return Promise.resolve();
                    }
                    return nativeAddIceCandidate.apply(this, arguments);
                };

                // shim getStats with maplike support
                var makeMapStats = function(stats) {
                    var map = new Map();
                    Object.keys(stats).forEach(function(key) {
                        map.set(key, stats[key]);
                        map[key] = stats[key];
                    });
                    return map;
                };

                var modernStatsTypes = {
                    inboundrtp: 'inbound-rtp',
                    outboundrtp: 'outbound-rtp',
                    candidatepair: 'candidate-pair',
                    localcandidate: 'local-candidate',
                    remotecandidate: 'remote-candidate'
                };

                var nativeGetStats = window.RTCPeerConnection.prototype.getStats;
                window.RTCPeerConnection.prototype.getStats = function(
                    selector,
                    onSucc,
                    onErr
                ) {
                    return nativeGetStats.apply(this, [selector || null])
                        .then(function(stats) {
                            if (browserDetails.version < 48) {
                                stats = makeMapStats(stats);
                            }
                            if (browserDetails.version < 53 && !onSucc) {
                                // Shim only promise getStats with spec-hyphens in type names
                                // Leave callback version alone; misc old uses of forEach before Map
                                try {
                                    stats.forEach(function(stat) {
                                        stat.type = modernStatsTypes[stat.type] || stat.type;
                                    });
                                } catch (e) {
                                    if (e.name !== 'TypeError') {
                                        throw e;
                                    }
                                    // Avoid TypeError: "type" is read-only, in old versions. 34-43ish
                                    stats.forEach(function(stat, i) {
                                        stats.set(i, Object.assign({}, stat, {
                                            type: modernStatsTypes[stat.type] || stat.type
                                        }));
                                    });
                                }
                            }
                            return stats;
                        })
                        .then(onSucc, onErr);
                };
            },

            shimRemoveStream: function(window) {
                if (!window.RTCPeerConnection ||
                    'removeStream' in window.RTCPeerConnection.prototype) {
                    return;
                }
                window.RTCPeerConnection.prototype.removeStream = function(stream) {
                    var pc = this;
                    utils.deprecated('removeStream', 'removeTrack');
                    this.getSenders().forEach(function(sender) {
                        if (sender.track && stream.getTracks().indexOf(sender.track) !== -1) {
                            pc.removeTrack(sender);
                        }
                    });
                };
            }
        };

// Expose public methods.
        module.exports = {
            shimOnTrack: firefoxShim.shimOnTrack,
            shimSourceObject: firefoxShim.shimSourceObject,
            shimPeerConnection: firefoxShim.shimPeerConnection,
            shimRemoveStream: firefoxShim.shimRemoveStream,
            shimGetUserMedia: requirecopy('./getusermedia')
        };

    },{"../utils":13,"./getusermedia":11}],11:[function(requirecopy,module,exports){
        /*
         *  Copyright (c) 2016 The WebRTC project authors. All Rights Reserved.
         *
         *  Use of this source code is governed by a BSD-style license
         *  that can be found in the LICENSE file in the root of the source
         *  tree.
         */
        /* eslint-env node */
        'use strict';

        var utils = requirecopy('../utils');
        var logging = utils.log;

// Expose public methods.
        module.exports = function(window) {
            var browserDetails = utils.detectBrowser(window);
            var navigator = window && window.navigator;
            var MediaStreamTrack = window && window.MediaStreamTrack;

            var shimError_ = function(e) {
                return {
                    name: {
                        InternalError: 'NotReadableError',
                        NotSupportedError: 'TypeError',
                        PermissionDeniedError: 'NotAllowedError',
                        SecurityError: 'NotAllowedError'
                    }[e.name] || e.name,
                    message: {
                        'The operation is insecure.': 'The request is not allowed by the ' +
                        'user agent or the platform in the current context.'
                    }[e.message] || e.message,
                    constraint: e.constraint,
                    toString: function() {
                        return this.name + (this.message && ': ') + this.message;
                    }
                };
            };

            // getUserMedia constraints shim.
            var getUserMedia_ = function(constraints, onSuccess, onError) {
                var constraintsToFF37_ = function(c) {
                    if (typeof c !== 'object' || c.require) {
                        return c;
                    }
                    var require = [];
                    Object.keys(c).forEach(function(key) {
                        if (key === 'require' || key === 'advanced' || key === 'mediaSource') {
                            return;
                        }
                        var r = c[key] = (typeof c[key] === 'object') ?
                            c[key] : {ideal: c[key]};
                        if (r.min !== undefined ||
                            r.max !== undefined || r.exact !== undefined) {
                            require.push(key);
                        }
                        if (r.exact !== undefined) {
                            if (typeof r.exact === 'number') {
                                r. min = r.max = r.exact;
                            } else {
                                c[key] = r.exact;
                            }
                            delete r.exact;
                        }
                        if (r.ideal !== undefined) {
                            c.advanced = c.advanced || [];
                            var oc = {};
                            if (typeof r.ideal === 'number') {
                                oc[key] = {min: r.ideal, max: r.ideal};
                            } else {
                                oc[key] = r.ideal;
                            }
                            c.advanced.push(oc);
                            delete r.ideal;
                            if (!Object.keys(r).length) {
                                delete c[key];
                            }
                        }
                    });
                    if (require.length) {
                        c.require = require;
                    }
                    return c;
                };
                constraints = JSON.parse(JSON.stringify(constraints));
                if (browserDetails.version < 38) {
                    logging('spec: ' + JSON.stringify(constraints));
                    if (constraints.audio) {
                        constraints.audio = constraintsToFF37_(constraints.audio);
                    }
                    if (constraints.video) {
                        constraints.video = constraintsToFF37_(constraints.video);
                    }
                    logging('ff37: ' + JSON.stringify(constraints));
                }
                return navigator.mozGetUserMedia(constraints, onSuccess, function(e) {
                    onError(shimError_(e));
                });
            };

            // Returns the result of getUserMedia as a Promise.
            var getUserMediaPromise_ = function(constraints) {
                return new Promise(function(resolve, reject) {
                    getUserMedia_(constraints, resolve, reject);
                });
            };

            // Shim for mediaDevices on older versions.
            if (!navigator.mediaDevices) {
                navigator.mediaDevices = {getUserMedia: getUserMediaPromise_,
                    addEventListener: function() { },
                    removeEventListener: function() { }
                };
            }
            navigator.mediaDevices.enumerateDevices =
                navigator.mediaDevices.enumerateDevices || function() {
                    return new Promise(function(resolve) {
                        var infos = [
                            {kind: 'audioinput', deviceId: 'default', label: '', groupId: ''},
                            {kind: 'videoinput', deviceId: 'default', label: '', groupId: ''}
                        ];
                        resolve(infos);
                    });
                };

            if (browserDetails.version < 41) {
                // Work around http://bugzil.la/1169665
                var orgEnumerateDevices =
                    navigator.mediaDevices.enumerateDevices.bind(navigator.mediaDevices);
                navigator.mediaDevices.enumerateDevices = function() {
                    return orgEnumerateDevices().then(undefined, function(e) {
                        if (e.name === 'NotFoundError') {
                            return [];
                        }
                        throw e;
                    });
                };
            }
            if (browserDetails.version < 49) {
                var origGetUserMedia = navigator.mediaDevices.getUserMedia.
                bind(navigator.mediaDevices);
                navigator.mediaDevices.getUserMedia = function(c) {
                    return origGetUserMedia(c).then(function(stream) {
                        // Work around https://bugzil.la/802326
                        if (c.audio && !stream.getAudioTracks().length ||
                            c.video && !stream.getVideoTracks().length) {
                            stream.getTracks().forEach(function(track) {
                                track.stop();
                            });
                            throw new DOMException('The object can not be found here.',
                                'NotFoundError');
                        }
                        return stream;
                    }, function(e) {
                        return Promise.reject(shimError_(e));
                    });
                };
            }
            if (!(browserDetails.version > 55 &&
                'autoGainControl' in navigator.mediaDevices.getSupportedConstraints())) {
                var remap = function(obj, a, b) {
                    if (a in obj && !(b in obj)) {
                        obj[b] = obj[a];
                        delete obj[a];
                    }
                };

                var nativeGetUserMedia = navigator.mediaDevices.getUserMedia.
                bind(navigator.mediaDevices);
                navigator.mediaDevices.getUserMedia = function(c) {
                    if (typeof c === 'object' && typeof c.audio === 'object') {
                        c = JSON.parse(JSON.stringify(c));
                        remap(c.audio, 'autoGainControl', 'mozAutoGainControl');
                        remap(c.audio, 'noiseSuppression', 'mozNoiseSuppression');
                    }
                    return nativeGetUserMedia(c);
                };

                if (MediaStreamTrack && MediaStreamTrack.prototype.getSettings) {
                    var nativeGetSettings = MediaStreamTrack.prototype.getSettings;
                    MediaStreamTrack.prototype.getSettings = function() {
                        var obj = nativeGetSettings.apply(this, arguments);
                        remap(obj, 'mozAutoGainControl', 'autoGainControl');
                        remap(obj, 'mozNoiseSuppression', 'noiseSuppression');
                        return obj;
                    };
                }

                if (MediaStreamTrack && MediaStreamTrack.prototype.applyConstraints) {
                    var nativeApplyConstraints = MediaStreamTrack.prototype.applyConstraints;
                    MediaStreamTrack.prototype.applyConstraints = function(c) {
                        if (this.kind === 'audio' && typeof c === 'object') {
                            c = JSON.parse(JSON.stringify(c));
                            remap(c, 'autoGainControl', 'mozAutoGainControl');
                            remap(c, 'noiseSuppression', 'mozNoiseSuppression');
                        }
                        return nativeApplyConstraints.apply(this, [c]);
                    };
                }
            }
            navigator.getUserMedia = function(constraints, onSuccess, onError) {
                if (browserDetails.version < 44) {
                    return getUserMedia_(constraints, onSuccess, onError);
                }
                // Replace Firefox 44+'s deprecation warning with unprefixed version.
                utils.deprecated('navigator.getUserMedia',
                    'navigator.mediaDevices.getUserMedia');
                navigator.mediaDevices.getUserMedia(constraints).then(onSuccess, onError);
            };
        };

    },{"../utils":13}],12:[function(requirecopy,module,exports){
        /*
         *  Copyright (c) 2016 The WebRTC project authors. All Rights Reserved.
         *
         *  Use of this source code is governed by a BSD-style license
         *  that can be found in the LICENSE file in the root of the source
         *  tree.
         */
        'use strict';
        var utils = requirecopy('../utils');

        var safariShim = {
            // TODO: DrAlex, should be here, double check against LayoutTests

            // TODO: once the back-end for the mac port is done, add.
            // TODO: check for webkitGTK+
            // shimPeerConnection: function() { },

            shimLocalStreamsAPI: function(window) {
                if (typeof window !== 'object' || !window.RTCPeerConnection) {
                    return;
                }
                if (!('getLocalStreams' in window.RTCPeerConnection.prototype)) {
                    window.RTCPeerConnection.prototype.getLocalStreams = function() {
                        if (!this._localStreams) {
                            this._localStreams = [];
                        }
                        return this._localStreams;
                    };
                }
                if (!('getStreamById' in window.RTCPeerConnection.prototype)) {
                    window.RTCPeerConnection.prototype.getStreamById = function(id) {
                        var result = null;
                        if (this._localStreams) {
                            this._localStreams.forEach(function(stream) {
                                if (stream.id === id) {
                                    result = stream;
                                }
                            });
                        }
                        if (this._remoteStreams) {
                            this._remoteStreams.forEach(function(stream) {
                                if (stream.id === id) {
                                    result = stream;
                                }
                            });
                        }
                        return result;
                    };
                }
                if (!('addStream' in window.RTCPeerConnection.prototype)) {
                    var _addTrack = window.RTCPeerConnection.prototype.addTrack;
                    window.RTCPeerConnection.prototype.addStream = function(stream) {
                        if (!this._localStreams) {
                            this._localStreams = [];
                        }
                        if (this._localStreams.indexOf(stream) === -1) {
                            this._localStreams.push(stream);
                        }
                        var self = this;
                        stream.getTracks().forEach(function(track) {
                            _addTrack.call(self, track, stream);
                        });
                    };

                    window.RTCPeerConnection.prototype.addTrack = function(track, stream) {
                        if (stream) {
                            if (!this._localStreams) {
                                this._localStreams = [stream];
                            } else if (this._localStreams.indexOf(stream) === -1) {
                                this._localStreams.push(stream);
                            }
                        }
                        return _addTrack.call(this, track, stream);
                    };
                }
                if (!('removeStream' in window.RTCPeerConnection.prototype)) {
                    window.RTCPeerConnection.prototype.removeStream = function(stream) {
                        if (!this._localStreams) {
                            this._localStreams = [];
                        }
                        var index = this._localStreams.indexOf(stream);
                        if (index === -1) {
                            return;
                        }
                        this._localStreams.splice(index, 1);
                        var self = this;
                        var tracks = stream.getTracks();
                        this.getSenders().forEach(function(sender) {
                            if (tracks.indexOf(sender.track) !== -1) {
                                self.removeTrack(sender);
                            }
                        });
                    };
                }
            },
            shimRemoteStreamsAPI: function(window) {
                if (typeof window !== 'object' || !window.RTCPeerConnection) {
                    return;
                }
                if (!('getRemoteStreams' in window.RTCPeerConnection.prototype)) {
                    window.RTCPeerConnection.prototype.getRemoteStreams = function() {
                        return this._remoteStreams ? this._remoteStreams : [];
                    };
                }
                if (!('onaddstream' in window.RTCPeerConnection.prototype)) {
                    Object.defineProperty(window.RTCPeerConnection.prototype, 'onaddstream', {
                        get: function() {
                            return this._onaddstream;
                        },
                        set: function(f) {
                            if (this._onaddstream) {
                                this.removeEventListener('addstream', this._onaddstream);
                                this.removeEventListener('track', this._onaddstreampoly);
                            }
                            this.addEventListener('addstream', this._onaddstream = f);
                            this.addEventListener('track', this._onaddstreampoly = function(e) {
                                var stream = e.streams[0];
                                if (!this._remoteStreams) {
                                    this._remoteStreams = [];
                                }
                                if (this._remoteStreams.indexOf(stream) >= 0) {
                                    return;
                                }
                                this._remoteStreams.push(stream);
                                var event = new Event('addstream');
                                event.stream = e.streams[0];
                                this.dispatchEvent(event);
                            }.bind(this));
                        }
                    });
                }
            },
            shimCallbacksAPI: function(window) {
                if (typeof window !== 'object' || !window.RTCPeerConnection) {
                    return;
                }
                var prototype = window.RTCPeerConnection.prototype;
                var createOffer = prototype.createOffer;
                var createAnswer = prototype.createAnswer;
                var setLocalDescription = prototype.setLocalDescription;
                var setRemoteDescription = prototype.setRemoteDescription;
                var addIceCandidate = prototype.addIceCandidate;

                prototype.createOffer = function(successCallback, failureCallback) {
                    var options = (arguments.length >= 2) ? arguments[2] : arguments[0];
                    var promise = createOffer.apply(this, [options]);
                    if (!failureCallback) {
                        return promise;
                    }
                    promise.then(successCallback, failureCallback);
                    return Promise.resolve();
                };

                prototype.createAnswer = function(successCallback, failureCallback) {
                    var options = (arguments.length >= 2) ? arguments[2] : arguments[0];
                    var promise = createAnswer.apply(this, [options]);
                    if (!failureCallback) {
                        return promise;
                    }
                    promise.then(successCallback, failureCallback);
                    return Promise.resolve();
                };

                var withCallback = function(description, successCallback, failureCallback) {
                    var promise = setLocalDescription.apply(this, [description]);
                    if (!failureCallback) {
                        return promise;
                    }
                    promise.then(successCallback, failureCallback);
                    return Promise.resolve();
                };
                prototype.setLocalDescription = withCallback;

                withCallback = function(description, successCallback, failureCallback) {
                    var promise = setRemoteDescription.apply(this, [description]);
                    if (!failureCallback) {
                        return promise;
                    }
                    promise.then(successCallback, failureCallback);
                    return Promise.resolve();
                };
                prototype.setRemoteDescription = withCallback;

                withCallback = function(candidate, successCallback, failureCallback) {
                    var promise = addIceCandidate.apply(this, [candidate]);
                    if (!failureCallback) {
                        return promise;
                    }
                    promise.then(successCallback, failureCallback);
                    return Promise.resolve();
                };
                prototype.addIceCandidate = withCallback;
            },
            shimGetUserMedia: function(window) {
                var navigator = window && window.navigator;

                if (!navigator.getUserMedia) {
                    if (navigator.webkitGetUserMedia) {
                        navigator.getUserMedia = navigator.webkitGetUserMedia.bind(navigator);
                    } else if (navigator.mediaDevices &&
                        navigator.mediaDevices.getUserMedia) {
                        navigator.getUserMedia = function(constraints, cb, errcb) {
                            navigator.mediaDevices.getUserMedia(constraints)
                                .then(cb, errcb);
                        }.bind(navigator);
                    }
                }
            },
            shimRTCIceServerUrls: function(window) {
                // migrate from non-spec RTCIceServer.url to RTCIceServer.urls
                var OrigPeerConnection = window.RTCPeerConnection;
                window.RTCPeerConnection = function(pcConfig, pcConstraints) {
                    if (pcConfig && pcConfig.iceServers) {
                        var newIceServers = [];
                        for (var i = 0; i < pcConfig.iceServers.length; i++) {
                            var server = pcConfig.iceServers[i];
                            if (!server.hasOwnProperty('urls') &&
                                server.hasOwnProperty('url')) {
                                utils.deprecated('RTCIceServer.url', 'RTCIceServer.urls');
                                server = JSON.parse(JSON.stringify(server));
                                server.urls = server.url;
                                delete server.url;
                                newIceServers.push(server);
                            } else {
                                newIceServers.push(pcConfig.iceServers[i]);
                            }
                        }
                        pcConfig.iceServers = newIceServers;
                    }
                    return new OrigPeerConnection(pcConfig, pcConstraints);
                };
                window.RTCPeerConnection.prototype = OrigPeerConnection.prototype;
                // wrap static methods. Currently just generateCertificate.
                if ('generateCertificate' in window.RTCPeerConnection) {
                    Object.defineProperty(window.RTCPeerConnection, 'generateCertificate', {
                        get: function() {
                            return OrigPeerConnection.generateCertificate;
                        }
                    });
                }
            },
            shimTrackEventTransceiver: function(window) {
                // Add event.transceiver member over deprecated event.receiver
                if (typeof window === 'object' && window.RTCPeerConnection &&
                    ('receiver' in window.RTCTrackEvent.prototype) &&
                    // can't check 'transceiver' in window.RTCTrackEvent.prototype, as it is
                    // defined for some reason even when window.RTCTransceiver is not.
                    !window.RTCTransceiver) {
                    Object.defineProperty(window.RTCTrackEvent.prototype, 'transceiver', {
                        get: function() {
                            return {receiver: this.receiver};
                        }
                    });
                }
            },

            shimCreateOfferLegacy: function(window) {
                var origCreateOffer = window.RTCPeerConnection.prototype.createOffer;
                window.RTCPeerConnection.prototype.createOffer = function(offerOptions) {
                    var pc = this;
                    if (offerOptions) {
                        var audioTransceiver = pc.getTransceivers().find(function(transceiver) {
                            return transceiver.sender.track &&
                                transceiver.sender.track.kind === 'audio';
                        });
                        if (offerOptions.offerToReceiveAudio === false && audioTransceiver) {
                            if (audioTransceiver.direction === 'sendrecv') {
                                audioTransceiver.setDirection('sendonly');
                            } else if (audioTransceiver.direction === 'recvonly') {
                                audioTransceiver.setDirection('inactive');
                            }
                        } else if (offerOptions.offerToReceiveAudio === true &&
                            !audioTransceiver) {
                            pc.addTransceiver('audio');
                        }

                        var videoTransceiver = pc.getTransceivers().find(function(transceiver) {
                            return transceiver.sender.track &&
                                transceiver.sender.track.kind === 'video';
                        });
                        if (offerOptions.offerToReceiveVideo === false && videoTransceiver) {
                            if (videoTransceiver.direction === 'sendrecv') {
                                videoTransceiver.setDirection('sendonly');
                            } else if (videoTransceiver.direction === 'recvonly') {
                                videoTransceiver.setDirection('inactive');
                            }
                        } else if (offerOptions.offerToReceiveVideo === true &&
                            !videoTransceiver) {
                            pc.addTransceiver('video');
                        }
                    }
                    return origCreateOffer.apply(pc, arguments);
                };
            }
        };

// Expose public methods.
        module.exports = {
            shimCallbacksAPI: safariShim.shimCallbacksAPI,
            shimLocalStreamsAPI: safariShim.shimLocalStreamsAPI,
            shimRemoteStreamsAPI: safariShim.shimRemoteStreamsAPI,
            shimGetUserMedia: safariShim.shimGetUserMedia,
            shimRTCIceServerUrls: safariShim.shimRTCIceServerUrls,
            shimTrackEventTransceiver: safariShim.shimTrackEventTransceiver,
            shimCreateOfferLegacy: safariShim.shimCreateOfferLegacy
            // TODO
            // shimPeerConnection: safariShim.shimPeerConnection
        };

    },{"../utils":13}],13:[function(requirecopy,module,exports){
        /*
         *  Copyright (c) 2016 The WebRTC project authors. All Rights Reserved.
         *
         *  Use of this source code is governed by a BSD-style license
         *  that can be found in the LICENSE file in the root of the source
         *  tree.
         */
        /* eslint-env node */
        'use strict';

        var logDisabled_ = true;
        var deprecationWarnings_ = true;

// Utility methods.
        var utils = {
            disableLog: function(bool) {
                if (typeof bool !== 'boolean') {
                    return new Error('Argument type: ' + typeof bool +
                        '. Please use a boolean.');
                }
                logDisabled_ = bool;
                return (bool) ? 'adapter.js logging disabled' :
                    'adapter.js logging enabled';
            },

            /**
             * Disable or enable deprecation warnings
             * @param {!boolean} bool set to true to disable warnings.
             */
            disableWarnings: function(bool) {
                if (typeof bool !== 'boolean') {
                    return new Error('Argument type: ' + typeof bool +
                        '. Please use a boolean.');
                }
                deprecationWarnings_ = !bool;
                return 'adapter.js deprecation warnings ' + (bool ? 'disabled' : 'enabled');
            },

            log: function() {
                if (typeof window === 'object') {
                    if (logDisabled_) {
                        return;
                    }
                    if (typeof console !== 'undefined' && typeof console.log === 'function') {
                        console.log.apply(console, arguments);
                    }
                }
            },

            /**
             * Shows a deprecation warning suggesting the modern and spec-compatible API.
             */
            deprecated: function(oldMethod, newMethod) {
                if (!deprecationWarnings_) {
                    return;
                }
                console.warn(oldMethod + ' is deprecated, please use ' + newMethod +
                    ' instead.');
            },

            /**
             * Extract browser version out of the provided user agent string.
             *
             * @param {!string} uastring userAgent string.
             * @param {!string} expr Regular expression used as match criteria.
             * @param {!number} pos position in the version string to be returned.
             * @return {!number} browser version.
             */
            extractVersion: function(uastring, expr, pos) {
                var match = uastring.match(expr);
                return match && match.length >= pos && parseInt(match[pos], 10);
            },

            /**
             * Browser detector.
             *
             * @return {object} result containing browser and version
             *     properties.
             */
            detectBrowser: function(window) {
                var navigator = window && window.navigator;

                // Returned result object.
                var result = {};
                result.browser = null;
                result.version = null;

                // Fail early if it's not a browser
                if (typeof window === 'undefined' || !window.navigator) {
                    result.browser = 'Not a browser.';
                    return result;
                }

                // Firefox.
                if (navigator.mozGetUserMedia) {
                    result.browser = 'firefox';
                    result.version = this.extractVersion(navigator.userAgent,
                        /Firefox\/(\d+)\./, 1);
                } else if (navigator.webkitGetUserMedia) {
                    // Chrome, Chromium, Webview, Opera, all use the chrome shim for now
                    if (window.webkitRTCPeerConnection) {
                        result.browser = 'chrome';
                        result.version = this.extractVersion(navigator.userAgent,
                            /Chrom(e|ium)\/(\d+)\./, 2);
                    } else { // Safari (in an unpublished version) or unknown webkit-based.
                        if (navigator.userAgent.match(/Version\/(\d+).(\d+)/)) {
                            result.browser = 'safari';
                            result.version = this.extractVersion(navigator.userAgent,
                                /AppleWebKit\/(\d+)\./, 1);
                        } else { // unknown webkit-based browser.
                            result.browser = 'Unsupported webkit-based browser ' +
                                'with GUM support but no WebRTC support.';
                            return result;
                        }
                    }
                } else if (navigator.mediaDevices &&
                    navigator.userAgent.match(/Edge\/(\d+).(\d+)$/)) { // Edge.
                    result.browser = 'edge';
                    result.version = this.extractVersion(navigator.userAgent,
                        /Edge\/(\d+).(\d+)$/, 2);
                } else if (navigator.mediaDevices &&
                    navigator.userAgent.match(/AppleWebKit\/(\d+)\./)) {
                    // Safari, with webkitGetUserMedia removed.
                    result.browser = 'safari';
                    result.version = this.extractVersion(navigator.userAgent,
                        /AppleWebKit\/(\d+)\./, 1);
                } else { // Default fallthrough: not supported.
                    result.browser = 'Not a supported browser.';
                    return result;
                }

                return result;
            },

        };

// Export.
        module.exports = {
            log: utils.log,
            deprecated: utils.deprecated,
            disableLog: utils.disableLog,
            disableWarnings: utils.disableWarnings,
            extractVersion: utils.extractVersion,
            shimCreateObjectURL: utils.shimCreateObjectURL,
            detectBrowser: utils.detectBrowser.bind(utils)
        };

    },{}]},{},[3])(3)
    });
    /* jshint ignore:end */

    // END OF INJECTION OF GOOGLE'S ADAPTER.JS CONTENT
    ///////////////////////////////////////////////////////////////////

    ///////////////////////////////////////////////////////////////////
    // EXTENSION FOR CHROME, FIREFOX AND EDGE
    // Includes legacy functions
    // -- createIceServer
    // -- createIceServers
    // -- MediaStreamTrack.getSources
    //
    // and additional shims
    // -- attachMediaStream
    // -- reattachMediaStream
    // -- requestUserMedia
    // -- a call to AdapterJS.maybeThroughWebRTCReady (notifies WebRTC is ready)

    // Add support for legacy functions createIceServer and createIceServers
    if ( navigator.mozGetUserMedia ) {
        // Shim for MediaStreamTrack.getSources.
        MediaStreamTrack.getSources = function(successCb) {
            setTimeout(function() {
                var infos = [
                    { kind: 'audio', id: 'default', label:'', facing:'' },
                    { kind: 'video', id: 'default', label:'', facing:'' }
                ];
                successCb(infos);
            }, 0);
        };

        // Attach a media stream to an element.
        attachMediaStream = function(element, stream) {
            element.srcObject = stream;
            return element;
        };

        reattachMediaStream = function(to, from) {
            to.srcObject = from.srcObject;
            return to;
        };

        createIceServer = function (url, username, password) {
            console.warn('createIceServer is deprecated. It should be replaced with an application level implementation.');
            // Note: Google's import of AJS will auto-reverse to 'url': '...' for FF < 38

            var iceServer = null;
            var urlParts = url.split(':');
            if (urlParts[0].indexOf('stun') === 0) {
                iceServer = { urls : [url] };
            } else if (urlParts[0].indexOf('turn') === 0) {
                if (AdapterJS.webrtcDetectedVersion < 27) {
                    var turnUrlParts = url.split('?');
                    if (turnUrlParts.length === 1 ||
                        turnUrlParts[1].indexOf('transport=udp') === 0) {
                        iceServer = {
                            urls : [turnUrlParts[0]],
                            credential : password,
                            username : username
                        };
                    }
                } else {
                    iceServer = {
                        urls : [url],
                        credential : password,
                        username : username
                    };
                }
            }
            return iceServer;
        };

        createIceServers = function (urls, username, password) {
            console.warn('createIceServers is deprecated. It should be replaced with an application level implementation.');

            var iceServers = [];
            for (i = 0; i < urls.length; i++) {
                var iceServer = createIceServer(urls[i], username, password);
                if (iceServer !== null) {
                    iceServers.push(iceServer);
                }
            }
            return iceServers;
        };
    } else if ( navigator.webkitGetUserMedia ) {
        // Attach a media stream to an element.
        attachMediaStream = function(element, stream) {
            if (AdapterJS.webrtcDetectedVersion >= 43) {
                element.srcObject = stream;
            } else if (typeof element.src !== 'undefined') {
                element.src = URL.createObjectURL(stream);
            } else {
                console.error('Error attaching stream to element.');
                // logging('Error attaching stream to element.');
            }
            return element;
        };

        reattachMediaStream = function(to, from) {
            if (AdapterJS.webrtcDetectedVersion >= 43) {
                to.srcObject = from.srcObject;
            } else {
                to.src = from.src;
            }
            return to;
        };

        createIceServer = function (url, username, password) {
            console.warn('createIceServer is deprecated. It should be replaced with an application level implementation.');

            var iceServer = null;
            var urlParts = url.split(':');
            if (urlParts[0].indexOf('stun') === 0) {
                iceServer = { 'url' : url };
            } else if (urlParts[0].indexOf('turn') === 0) {
                iceServer = {
                    'url' : url,
                    'credential' : password,
                    'username' : username
                };
            }
            return iceServer;
        };

        createIceServers = function (urls, username, password) {
            console.warn('createIceServers is deprecated. It should be replaced with an application level implementation.');

            var iceServers = [];
            if (AdapterJS.webrtcDetectedVersion >= 34) {
                iceServers = {
                    'urls' : urls,
                    'credential' : password,
                    'username' : username
                };
            } else {
                for (i = 0; i < urls.length; i++) {
                    var iceServer = createIceServer(urls[i], username, password);
                    if (iceServer !== null) {
                        iceServers.push(iceServer);
                    }
                }
            }
            return iceServers;
        };
    } else if (AdapterJS.webrtcDetectedType === 'AppleWebKit') {
        attachMediaStream = function(element, stream) {
            element.srcObject = stream;
            return element;
        };
        reattachMediaStream = function(to, from) {
            to.srcObject = from.srcObject;
            return to;
        };

        // Polyfill getUserMedia()
        if (navigator.mediaDevices && navigator.mediaDevices.getUserMedia) {
            navigator.getUserMedia = getUserMedia = function (constraints, successCb, errorCb) {
                navigator.mediaDevices.getUserMedia(constraints).then(successCb).catch(errorCb);
            };
        }
    } else if (navigator.mediaDevices && navigator.userAgent.match(/Edge\/(\d+).(\d+)$/)) {
        // Attach a media stream to an element.
        attachMediaStream = function(element, stream) {
            element.srcObject = stream;
            return element;
        };

        reattachMediaStream = function(to, from) {
            to.srcObject = from.srcObject;
            return to;
        };
    }

    // Need to override attachMediaStream and reattachMediaStream
    // to support the plugin's logic
    attachMediaStream_base = attachMediaStream;

    if (AdapterJS.webrtcDetectedBrowser === 'opera') {
        attachMediaStream_base = function (element, stream) {
            if (AdapterJS.webrtcDetectedVersion > 38) {
                element.srcObject = stream;
            } else if (typeof element.src !== 'undefined') {
                element.src = URL.createObjectURL(stream);
            }
            // Else it doesn't work
        };
    }

    attachMediaStream = function (element, stream) {
        if ((AdapterJS.webrtcDetectedBrowser === 'chrome' ||
            AdapterJS.webrtcDetectedBrowser === 'opera') &&
            !stream) {
            // Chrome does not support "src = null"
            element.src = '';
        } else {
            attachMediaStream_base(element, stream);
        }
        return element;
    };
    reattachMediaStream_base = reattachMediaStream;
    reattachMediaStream = function (to, from) {
        reattachMediaStream_base(to, from);
        return to;
    };

    // Propagate attachMediaStream and gUM in window and AdapterJS
    window.attachMediaStream      = attachMediaStream;
    window.reattachMediaStream    = reattachMediaStream;
    window.getUserMedia           = function(constraints, onSuccess, onFailure) {
        navigator.getUserMedia(constraints, onSuccess, onFailure);
    };
    AdapterJS.attachMediaStream   = attachMediaStream;
    AdapterJS.reattachMediaStream = reattachMediaStream;
    AdapterJS.getUserMedia        = getUserMedia;

    // Removed Google defined promises when promise is not defined
    if (typeof Promise === 'undefined') {
        requestUserMedia = null;
    }

    AdapterJS.maybeThroughWebRTCReady();

    // END OF EXTENSION OF CHROME, FIREFOX AND EDGE
    ///////////////////////////////////////////////////////////////////

} else { // TRY TO USE PLUGIN

    ///////////////////////////////////////////////////////////////////
    // WEBRTC PLUGIN SHIM
    // Will automatically check if the plugin is available and inject it
    // into the DOM if it is.
    // When the plugin is not available, will prompt a banner to suggest installing it
    // Use AdapterJS.options.hidePluginInstallPrompt to prevent this banner from popping
    //
    // Shims the follwing:
    // -- getUserMedia
    // -- MediaStream
    // -- MediaStreamTrack
    // -- MediaStreamTrack.getSources
    // -- RTCPeerConnection
    // -- RTCSessionDescription
    // -- RTCIceCandidate
    // -- createIceServer
    // -- createIceServers
    // -- attachMediaStream
    // -- reattachMediaStream
    // -- webrtcDetectedBrowser
    // -- webrtcDetectedVersion

    // IE 9 is not offering an implementation of console.log until you open a console
    if (typeof console !== 'object' || typeof console.log !== 'function') {
        /* jshint -W020 */
        console = {} || console;
        // Implemented based on console specs from MDN
        // You may override these functions
        console.log = function (arg) {};
        console.info = function (arg) {};
        console.error = function (arg) {};
        console.dir = function (arg) {};
        console.exception = function (arg) {};
        console.trace = function (arg) {};
        console.warn = function (arg) {};
        console.count = function (arg) {};
        console.debug = function (arg) {};
        console.count = function (arg) {};
        console.time = function (arg) {};
        console.timeEnd = function (arg) {};
        console.group = function (arg) {};
        console.groupCollapsed = function (arg) {};
        console.groupEnd = function (arg) {};
        /* jshint +W020 */
    }

    /* jshint -W035 */
    AdapterJS.WebRTCPlugin.WaitForPluginReady = function() {
        while (AdapterJS.WebRTCPlugin.pluginState !== AdapterJS.WebRTCPlugin.PLUGIN_STATES.READY) {
            /* empty because it needs to prevent the function from running. */
        }
    };
    /* jshint +W035 */

    AdapterJS.WebRTCPlugin.callWhenPluginReady = function (callback) {
        if (AdapterJS.WebRTCPlugin.pluginState === AdapterJS.WebRTCPlugin.PLUGIN_STATES.READY) {
            // Call immediately if possible
            // Once the plugin is set, the code will always take this path
            callback();
        } else {
            // otherwise start a 100ms interval
            var checkPluginReadyState = setInterval(function () {
                if (AdapterJS.WebRTCPlugin.pluginState === AdapterJS.WebRTCPlugin.PLUGIN_STATES.READY) {
                    clearInterval(checkPluginReadyState);
                    callback();
                }
            }, 100);
        }
    };

    AdapterJS.WebRTCPlugin.setLogLevel = function(logLevel) {
        AdapterJS.WebRTCPlugin.callWhenPluginReady(function() {
            AdapterJS.WebRTCPlugin.plugin.setLogLevel(logLevel);
        });
    };

    AdapterJS.WebRTCPlugin.injectPlugin = function () {
        // only inject once the page is ready
        if (document.readyState !== 'interactive' && document.readyState !== 'complete') {
            return;
        }

        // Prevent multiple injections
        if (AdapterJS.WebRTCPlugin.pluginState !== AdapterJS.WebRTCPlugin.PLUGIN_STATES.INITIALIZING) {
            return;
        }

        AdapterJS.WebRTCPlugin.pluginState = AdapterJS.WebRTCPlugin.PLUGIN_STATES.INJECTING;

        if (AdapterJS.webrtcDetectedBrowser === 'IE' && AdapterJS.webrtcDetectedVersion <= 10) {
            var frag = document.createDocumentFragment();
            AdapterJS.WebRTCPlugin.plugin = document.createElement('div');
            AdapterJS.WebRTCPlugin.plugin.innerHTML = '<object id="' +
                AdapterJS.WebRTCPlugin.pluginInfo.pluginId + '" type="' +
                AdapterJS.WebRTCPlugin.pluginInfo.type + '" ' + 'width="1" height="1">' +
                '<param name="pluginId" value="' +
                AdapterJS.WebRTCPlugin.pluginInfo.pluginId + '" /> ' +
                '<param name="windowless" value="false" /> ' +
                '<param name="pageId" value="' + AdapterJS.WebRTCPlugin.pageId + '" /> ' +
                '<param name="onload" value="' + AdapterJS.WebRTCPlugin.pluginInfo.onload + '" />' +
                '<param name="tag" value="' + AdapterJS.WebRTCPlugin.TAGS.NONE + '" />' +
                // uncomment to be able to use virtual cams
                (AdapterJS.options.getAllCams ? '<param name="forceGetAllCams" value="True" />':'') +

                '</object>';
            while (AdapterJS.WebRTCPlugin.plugin.firstChild) {
                frag.appendChild(AdapterJS.WebRTCPlugin.plugin.firstChild);
            }
            document.body.appendChild(frag);

            // Need to re-fetch the plugin
            AdapterJS.WebRTCPlugin.plugin =
                document.getElementById(AdapterJS.WebRTCPlugin.pluginInfo.pluginId);
        } else {
            // Load Plugin
            AdapterJS.WebRTCPlugin.plugin = document.createElement('object');
            AdapterJS.WebRTCPlugin.plugin.id =
                AdapterJS.WebRTCPlugin.pluginInfo.pluginId;
            // IE will only start the plugin if it's ACTUALLY visible
            if (AdapterJS.webrtcDetectedBrowser === 'IE') {
                AdapterJS.WebRTCPlugin.plugin.width = '1px';
                AdapterJS.WebRTCPlugin.plugin.height = '1px';
            } else { // The size of the plugin on Safari should be 0x0px
                // so that the autorisation prompt is at the top
                AdapterJS.WebRTCPlugin.plugin.width = '0px';
                AdapterJS.WebRTCPlugin.plugin.height = '0px';
            }
            AdapterJS.WebRTCPlugin.plugin.type = AdapterJS.WebRTCPlugin.pluginInfo.type;
            AdapterJS.WebRTCPlugin.plugin.innerHTML = '<param name="onload" value="' +
                AdapterJS.WebRTCPlugin.pluginInfo.onload + '">' +
                '<param name="pluginId" value="' +
                AdapterJS.WebRTCPlugin.pluginInfo.pluginId + '">' +
                '<param name="windowless" value="false" /> ' +
                (AdapterJS.options.getAllCams ? '<param name="forceGetAllCams" value="True" />':'') +
                '<param name="pageId" value="' + AdapterJS.WebRTCPlugin.pageId + '">' +
                '<param name="tag" value="' + AdapterJS.WebRTCPlugin.TAGS.NONE + '" />';
            document.body.appendChild(AdapterJS.WebRTCPlugin.plugin);
        }


        AdapterJS.WebRTCPlugin.pluginState = AdapterJS.WebRTCPlugin.PLUGIN_STATES.INJECTED;
    };

    AdapterJS.WebRTCPlugin.isPluginInstalled =
        function (comName, plugName, plugType, installedCb, notInstalledCb) {
            if (AdapterJS.webrtcDetectedBrowser !== 'IE') {
                var pluginArray = navigator.mimeTypes;
                for (var i = 0; i < pluginArray.length; i++) {
                    if (pluginArray[i].type.indexOf(plugType) >= 0) {
                        installedCb();
                        return;
                    }
                }
                notInstalledCb();
            } else {
                try {
                    var axo = new ActiveXObject(comName + '.' + plugName);
                } catch (e) {
                    notInstalledCb();
                    return;
                }
                installedCb();
            }
        };

    AdapterJS.WebRTCPlugin.defineWebRTCInterface = function () {
        if (AdapterJS.WebRTCPlugin.pluginState ===
            AdapterJS.WebRTCPlugin.PLUGIN_STATES.READY) {
            console.error('AdapterJS - WebRTC interface has already been defined');
            return;
        }

        AdapterJS.WebRTCPlugin.pluginState = AdapterJS.WebRTCPlugin.PLUGIN_STATES.INITIALIZING;

        AdapterJS.isDefined = function (variable) {
            return variable !== null && variable !== undefined;
        };

        ////////////////////////////////////////////////////////////////////////////
        /// CreateIceServer
        ////////////////////////////////////////////////////////////////////////////
        createIceServer = function (url, username, password) {
            var iceServer = null;
            var urlParts = url.split(':');
            if (urlParts[0].indexOf('stun') === 0) {
                iceServer = {
                    'url' : url,
                    'hasCredentials' : false
                };
            } else if (urlParts[0].indexOf('turn') === 0) {
                iceServer = {
                    'url' : url,
                    'hasCredentials' : true,
                    'credential' : password,
                    'username' : username
                };
            }
            return iceServer;
        };

        ////////////////////////////////////////////////////////////////////////////
        /// CreateIceServers
        ////////////////////////////////////////////////////////////////////////////
        createIceServers = function (urls, username, password) {
            var iceServers = [];
            for (var i = 0; i < urls.length; ++i) {
                iceServers.push(createIceServer(urls[i], username, password));
            }
            return iceServers;
        };

        ////////////////////////////////////////////////////////////////////////////
        /// RTCSessionDescription
        ////////////////////////////////////////////////////////////////////////////
        RTCSessionDescription = function (info) {
            AdapterJS.WebRTCPlugin.WaitForPluginReady();
            return AdapterJS.WebRTCPlugin.plugin.
            ConstructSessionDescription(info.type, info.sdp);
        };

        ////////////////////////////////////////////////////////////////////////////
        /// MediaStream
        ////////////////////////////////////////////////////////////////////////////
        MediaStream = function (mediaStreamOrTracks) {
            AdapterJS.WebRTCPlugin.WaitForPluginReady();
            return AdapterJS.WebRTCPlugin.plugin.MediaStream(mediaStreamOrTracks);
        }

        ////////////////////////////////////////////////////////////////////////////
        /// RTCPeerConnection
        ////////////////////////////////////////////////////////////////////////////
        RTCPeerConnection = function (servers, constraints) {
            // Validate server argumenr
            if (!(servers === undefined ||
                servers === null ||
                Array.isArray(servers.iceServers))) {
                throw new Error('Failed to construct \'RTCPeerConnection\': Malformed RTCConfiguration');
            }

            // Validate constraints argument
            if (typeof constraints !== 'undefined' && constraints !== null) {
                var invalidConstraits = false;
                invalidConstraits |= typeof constraints !== 'object';
                invalidConstraits |= constraints.hasOwnProperty('mandatory') &&
                    constraints.mandatory !== undefined &&
                    constraints.mandatory !== null &&
                    constraints.mandatory.constructor !== Object;
                invalidConstraits |= constraints.hasOwnProperty('optional') &&
                    constraints.optional !== undefined &&
                    constraints.optional !== null &&
                    !Array.isArray(constraints.optional);
                if (invalidConstraits) {
                    throw new Error('Failed to construct \'RTCPeerConnection\': Malformed constraints object');
                }
            }

            // Call relevant PeerConnection constructor according to plugin version
            AdapterJS.WebRTCPlugin.WaitForPluginReady();

            // RTCPeerConnection prototype from the old spec
            var iceServers = null;
            if (servers && Array.isArray(servers.iceServers)) {
                iceServers = servers.iceServers;
                for (var i = 0; i < iceServers.length; i++) {
                    // Legacy plugin versions compatibility
                    if (iceServers[i].urls && !iceServers[i].url) {
                        iceServers[i].url = iceServers[i].urls;
                    }
                    iceServers[i].hasCredentials = AdapterJS.
                        isDefined(iceServers[i].username) &&
                        AdapterJS.isDefined(iceServers[i].credential);
                }
            }

            if (AdapterJS.WebRTCPlugin.plugin.PEER_CONNECTION_VERSION &&
                AdapterJS.WebRTCPlugin.plugin.PEER_CONNECTION_VERSION > 1) {
                // RTCPeerConnection prototype from the new spec
                if (iceServers) {
                    servers.iceServers = iceServers;
                }
                return AdapterJS.WebRTCPlugin.plugin.PeerConnection(servers);
            } else {
                var mandatory = (constraints && constraints.mandatory) ?
                    constraints.mandatory : null;
                var optional = (constraints && constraints.optional) ?
                    constraints.optional : null;
                return AdapterJS.WebRTCPlugin.plugin.
                PeerConnection(AdapterJS.WebRTCPlugin.pageId,
                    iceServers, mandatory, optional);
            }
        };

        ////////////////////////////////////////////////////////////////////////////
        /// MediaStreamTrack
        ////////////////////////////////////////////////////////////////////////////
        MediaStreamTrack = function(){};
        MediaStreamTrack.getSources = function (callback) {
            AdapterJS.WebRTCPlugin.callWhenPluginReady(function() {
                AdapterJS.WebRTCPlugin.plugin.GetSources(callback);
            });
        };

        // getUserMedia constraints shim.
        // Copied from Chrome
        var constraintsToPlugin = function(c) {
            if (typeof c !== 'object' || c.mandatory || c.optional) {
                return c;
            }
            var cc = {};
            Object.keys(c).forEach(function(key) {
                if (key === 'require' || key === 'advanced') {
                    return;
                }
                if (typeof c[key] === 'string') {
                    cc[key] = c[key];
                    return;
                }
                var r = (typeof c[key] === 'object') ? c[key] : {ideal: c[key]};
                if (r.exact !== undefined && typeof r.exact === 'number') {
                    r.min = r.max = r.exact;
                }
                var oldname = function(prefix, name) {
                    if (prefix) {
                        return prefix + name.charAt(0).toUpperCase() + name.slice(1);
                    }
                    return (name === 'deviceId') ? 'sourceId' : name;
                };

                // HACK : Specially handling: if deviceId is an object with exact property,
                //          change it such that deviceId value is not in exact property
                // Reason : AJS-286 (deviceId in WebRTC samples not in the format specified as specifications)
                if ( oldname('', key) === 'sourceId' && r.exact !== undefined ) {
                    r.ideal = r.exact;
                    r.exact = undefined;
                }

                if (r.ideal !== undefined) {
                    cc.optional = cc.optional || [];
                    var oc = {};
                    if (typeof r.ideal === 'number') {
                        oc[oldname('min', key)] = r.ideal;
                        cc.optional.push(oc);
                        oc = {};
                        oc[oldname('max', key)] = r.ideal;
                        cc.optional.push(oc);
                    } else {
                        oc[oldname('', key)] = r.ideal;
                        cc.optional.push(oc);
                    }
                }
                if (r.exact !== undefined && typeof r.exact !== 'number') {
                    cc.mandatory = cc.mandatory || {};
                    cc.mandatory[oldname('', key)] = r.exact;
                } else {
                    ['min', 'max'].forEach(function(mix) {
                        if (r[mix] !== undefined) {
                            cc.mandatory = cc.mandatory || {};
                            cc.mandatory[oldname(mix, key)] = r[mix];
                        }
                    });
                }
            });
            if (c.advanced) {
                cc.optional = (cc.optional || []).concat(c.advanced);
            }
            return cc;
        };

        ////////////////////////////////////////////////////////////////////////////
        /// getUserMedia
        ////////////////////////////////////////////////////////////////////////////
        getUserMedia = function (constraints, successCallback, failureCallback) {
            var cc = {};
            cc.audio = constraints.audio ?
                constraintsToPlugin(constraints.audio) : false;
            cc.video = constraints.video ?
                constraintsToPlugin(constraints.video) : false;

            AdapterJS.WebRTCPlugin.callWhenPluginReady(function() {
                AdapterJS.WebRTCPlugin.plugin.
                getUserMedia(cc, successCallback, failureCallback);
            });
        };
        window.navigator.getUserMedia = getUserMedia;

        ////////////////////////////////////////////////////////////////////////////
        /// mediaDevices
        ////////////////////////////////////////////////////////////////////////////
        if (typeof Promise !== 'undefined') {
            requestUserMedia = function(constraints) {
                return new Promise(function(resolve, reject) {
                    try {
                        getUserMedia(constraints, resolve, reject);
                    } catch (error) {
                        reject(error);
                    }
                });
            };
            navigator.mediaDevices = {getUserMedia: requestUserMedia,
                enumerateDevices: function() {
                    return new Promise(function(resolve) {
                        var kinds = {audio: 'audioinput', video: 'videoinput'};
                        return MediaStreamTrack.getSources(function(devices) {
                            resolve(devices.map(function(device) {
                                return {label: device.label,
                                    kind: kinds[device.kind],
                                    id: device.id,
                                    deviceId: device.id,
                                    groupId: ''};
                            }));
                        });
                    });
                }};
        }

        ////////////////////////////////////////////////////////////////////////////
        /// attachMediaStream
        ////////////////////////////////////////////////////////////////////////////
        attachMediaStream = function (element, stream) {
            if (!element || !element.parentNode) {
                return;
            }

            var streamId;
            if (stream === null) {
                streamId = '';
            } else {
                if (typeof stream.enableSoundTracks !== 'undefined') {
                    stream.enableSoundTracks(true);
                }
                streamId = stream.id;
            }

            var elementId = element.id.length === 0 ? Math.random().toString(36).slice(2) : element.id;
            var nodeName = element.nodeName.toLowerCase();
            if (nodeName !== 'object') { // not a plugin <object> tag yet
                var tag;
                switch(nodeName) {
                    case 'audio':
                        tag = AdapterJS.WebRTCPlugin.TAGS.AUDIO;
                        break;
                    case 'video':
                        tag = AdapterJS.WebRTCPlugin.TAGS.VIDEO;
                        break;
                    default:
                        tag = AdapterJS.WebRTCPlugin.TAGS.NONE;
                }

                var frag = document.createDocumentFragment();
                var temp = document.createElement('div');
                var classHTML = '';
                if (element.className) {
                    classHTML = 'class="' + element.className + '" ';
                } else if (element.attributes && element.attributes['class']) {
                    classHTML = 'class="' + element.attributes['class'].value + '" ';
                }

                temp.innerHTML = '<object id="' + elementId + '" ' + classHTML +
                    'type="' + AdapterJS.WebRTCPlugin.pluginInfo.type + '">' +
                    '<param name="pluginId" value="' + elementId + '" /> ' +
                    '<param name="pageId" value="' + AdapterJS.WebRTCPlugin.pageId + '" /> ' +
                    '<param name="windowless" value="true" /> ' +
                    '<param name="streamId" value="' + streamId + '" /> ' +
                    '<param name="tag" value="' + tag + '" /> ' +
                    '</object>';
                while (temp.firstChild) {
                    frag.appendChild(temp.firstChild);
                }

                var height = '';
                var width = '';
                if (element.clientWidth || element.clientHeight) {
                    width = element.clientWidth;
                    height = element.clientHeight;
                }
                else if (element.width || element.height) {
                    width = element.width;
                    height = element.height;
                }

                element.parentNode.insertBefore(frag, element);
                frag = document.getElementById(elementId);
                frag.width = width;
                frag.height = height;
                element.parentNode.removeChild(element);
            } else { // already an <object> tag, just change the stream id
                var children = element.children;
                for (var i = 0; i !== children.length; ++i) {
                    if (children[i].name === 'streamId') {
                        children[i].value = streamId;
                        break;
                    }
                }
                element.setStreamId(streamId);
            }
            var newElement = document.getElementById(elementId);
            AdapterJS.forwardEventHandlers(newElement, element, Object.getPrototypeOf(element));

            return newElement;
        };

        ////////////////////////////////////////////////////////////////////////////
        /// reattachMediaStream
        ////////////////////////////////////////////////////////////////////////////
        reattachMediaStream = function (to, from) {
            var stream = null;
            var children = from.children;
            for (var i = 0; i !== children.length; ++i) {
                if (children[i].name === 'streamId') {
                    AdapterJS.WebRTCPlugin.WaitForPluginReady();
                    stream = AdapterJS.WebRTCPlugin.plugin
                        .getStreamWithId(AdapterJS.WebRTCPlugin.pageId, children[i].value);
                    break;
                }
            }
            if (stream !== null) {
                return attachMediaStream(to, stream);
            } else {
                console.log('Could not find the stream associated with this element');
            }
        };

        // Propagate attachMediaStream and gUM in window and AdapterJS
        window.attachMediaStream      = attachMediaStream;
        window.reattachMediaStream    = reattachMediaStream;
        window.getUserMedia           = getUserMedia;
        AdapterJS.attachMediaStream   = attachMediaStream;
        AdapterJS.reattachMediaStream = reattachMediaStream;
        AdapterJS.getUserMedia        = getUserMedia;

        AdapterJS.forwardEventHandlers = function (destElem, srcElem, prototype) {
            var properties = Object.getOwnPropertyNames( prototype );
            for(var prop in properties) {
                if (prop) {
                    var propName = properties[prop];

                    if (typeof propName.slice === 'function' &&
                        propName.slice(0,2) === 'on' &&
                        typeof srcElem[propName] === 'function') {
                        AdapterJS.addEvent(destElem, propName.slice(2), srcElem[propName]);
                    }
                }
            }
            var subPrototype = Object.getPrototypeOf(prototype);
            if(!!subPrototype) {
                AdapterJS.forwardEventHandlers(destElem, srcElem, subPrototype);
            }
        };

        ////////////////////////////////////////////////////////////////////////////
        /// RTCIceCandidate
        ////////////////////////////////////////////////////////////////////////////
        RTCIceCandidate = function (candidate) {
            if (!candidate.sdpMid) {
                candidate.sdpMid = '';
            }

            AdapterJS.WebRTCPlugin.WaitForPluginReady();
            return AdapterJS.WebRTCPlugin.plugin.ConstructIceCandidate(
                candidate.sdpMid, candidate.sdpMLineIndex, candidate.candidate
            );
        };

        // inject plugin
        AdapterJS.addEvent(document, 'readystatechange', AdapterJS.WebRTCPlugin.injectPlugin);
        AdapterJS.WebRTCPlugin.injectPlugin();
    };

    // This function will be called if the plugin is needed (browser different
    // from Chrome or Firefox), but the plugin is not installed.
    AdapterJS.WebRTCPlugin.pluginNeededButNotInstalledCb = AdapterJS.WebRTCPlugin.pluginNeededButNotInstalledCb ||
        function() {
            AdapterJS.addEvent(document,
                'readystatechange',
                AdapterJS.WebRTCPlugin.pluginNeededButNotInstalledCbPriv);
            AdapterJS.WebRTCPlugin.pluginNeededButNotInstalledCbPriv();
        };

    AdapterJS.WebRTCPlugin.pluginNeededButNotInstalledCbPriv = function () {
        if (AdapterJS.options.hidePluginInstallPrompt) {
            return;
        }

        var downloadLink = AdapterJS.WebRTCPlugin.pluginInfo.downloadLink;
        if(downloadLink) { // if download link
            var popupString;
            if (AdapterJS.WebRTCPlugin.pluginInfo.portalLink) { // is portal link
                popupString = 'This website requires you to install the ' +
                    ' <a href="' + AdapterJS.WebRTCPlugin.pluginInfo.portalLink +
                    '" target="_blank">' + AdapterJS.WebRTCPlugin.pluginInfo.companyName +
                    ' WebRTC Plugin</a>' +
                    ' to work on this browser.';
            } else { // no portal link, just print a generic explanation
                popupString = AdapterJS.TEXT.PLUGIN.REQUIRE_INSTALLATION;
            }

            AdapterJS.renderNotificationBar(popupString, AdapterJS.TEXT.PLUGIN.BUTTON, function () {
                window.open(downloadLink, '_top');

                var pluginInstallInterval = setInterval(function(){
                    if(AdapterJS.webrtcDetectedBrowser !== 'IE') {
                        navigator.plugins.refresh(false);
                    }
                    AdapterJS.WebRTCPlugin.isPluginInstalled(
                        AdapterJS.WebRTCPlugin.pluginInfo.prefix,
                        AdapterJS.WebRTCPlugin.pluginInfo.plugName,
                        AdapterJS.WebRTCPlugin.pluginInfo.type,
                        function() { // plugin now installed
                            clearInterval(pluginInstallInterval);
                            AdapterJS.WebRTCPlugin.defineWebRTCInterface();
                        },
                        function() {
                            // still no plugin detected, nothing to do
                        });
                } , 500);
            });
        } else { // no download link, just print a generic explanation
            AdapterJS.renderNotificationBar(AdapterJS.TEXT.PLUGIN.NOT_SUPPORTED);
        }
    };


    // Try to detect the plugin and act accordingly
    AdapterJS.WebRTCPlugin.isPluginInstalled(
        AdapterJS.WebRTCPlugin.pluginInfo.prefix,
        AdapterJS.WebRTCPlugin.pluginInfo.plugName,
        AdapterJS.WebRTCPlugin.pluginInfo.type,
        AdapterJS.WebRTCPlugin.defineWebRTCInterface,
        AdapterJS.WebRTCPlugin.pluginNeededButNotInstalledCb);

    // END OF WEBRTC PLUGIN SHIM
    ///////////////////////////////////////////////////////////////////
}

// Placed it here so that the module.exports from the browserified
//   adapterjs will not override our AdapterJS exports
// Browserify compatibility
if(typeof exports !== 'undefined') {
    module.exports = AdapterJS;
}
